@@ -9,14 +9,23 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 class History;
 
+namespace Data {
+class Thread;
+} // namespace Data
+
 namespace Api {
 
+inline constexpr auto kScheduledUntilOnlineTimestamp = TimeId(0x7FFFFFFE);
+
 struct SendOptions {
+	PeerData *sendAs = nullptr;
 	TimeId scheduled = 0;
 	bool silent = false;
 	bool handleSupportSwitch = false;
 	bool removeWebPageId = false;
+	bool hideViaBot = false;
 };
+[[nodiscard]] SendOptions DefaultSendWhenOnlineOptions();
 
 enum class SendType {
 	Normal,
@@ -25,19 +34,22 @@ enum class SendType {
 };
 
 struct SendAction {
-	explicit SendAction(not_null<History*> history) : history(history) {
-	}
+	explicit SendAction(
+		not_null<Data::Thread*> thread,
+		SendOptions options = SendOptions());
 
 	not_null<History*> history;
 	SendOptions options;
-	MsgId replyTo = 0;
+	FullReplyTo replyTo;
 	bool clearDraft = true;
 	bool generateLocal = true;
 	MsgId replaceMediaOf = 0;
+
+	[[nodiscard]] MTPInputReplyTo mtpReplyTo() const;
 };
 
 struct MessageToSend {
-	explicit MessageToSend(not_null<History*> history) : action(history) {
+	explicit MessageToSend(SendAction action) : action(action) {
 	}
 
 	SendAction action;
@@ -45,4 +57,10 @@ struct MessageToSend {
 	WebPageId webPageId = 0;
 };
 
+struct RemoteFileInfo {
+	MTPInputFile file;
+	std::optional<MTPInputFile> thumb;
+	std::vector<MTPInputDocument> attachedStickers;
+};
+
 } // namespace Api
