@@ -10,22 +10,30 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lang/lang_keys.h"
 #include "ui/effects/round_checkbox.h"
 #include "ui/image/image.h"
+#include "ui/chat/attach/attach_extensions.h"
+#include "ui/chat/chat_theme.h"
 #include "ui/ui_utility.h"
 #include "main/main_session.h"
 #include "apiwrap.h"
 #include "mtproto/sender.h"
+#include "core/file_utilities.h"
+#include "data/data_peer.h"
 #include "data/data_session.h"
 #include "data/data_file_origin.h"
 #include "data/data_document.h"
 #include "data/data_document_media.h"
 #include "boxes/background_preview_box.h"
-#include "boxes/confirm_box.h"
+#include "info/profile/info_profile_icon.h"
+#include "settings/settings_common.h"
+#include "ui/boxes/confirm_box.h"
+#include "ui/widgets/buttons.h"
 #include "window/window_session_controller.h"
-#include "app.h"
+#include "window/themes/window_theme.h"
 #include "styles/style_overview.h"
 #include "styles/style_layers.h"
 #include "styles/style_boxes.h"
 #include "styles/style_chat_helpers.h"
+#include "styles/style_info.h"
 
 namespace {
 
@@ -56,17 +64,19 @@ QImage TakeMiddleSample(QImage original, QSize size) {
 
 } // namespace
 
-class BackgroundBox::Inner final
-	: public Ui::RpWidget
-	, private base::Subscriber {
+class BackgroundBox::Inner final : public Ui::RpWidget {
 public:
 	Inner(
 		QWidget *parent,
-		not_null<Main::Session*> session);
+		not_null<Main::Session*> session,
+		PeerData *forPeer);
 	~Inner();
 
-	rpl::producer<Data::WallPaper> chooseEvents() const;
-	rpl::producer<Data::WallPaper> removeRequests() const;
+	[[nodiscard]] rpl::producer<Data::WallPaper> chooseEvents() const;
+	[[nodiscard]] rpl::producer<Data::WallPaper> removeRequests() const;
+
+	[[nodiscard]] auto resolveResetCustomPaper() const
+		->std::optional<Data::WallPaper>;
 
 	void removePaper(const Data::WallPaper &data);
 
@@ -103,25 +113,30 @@ private:
 			return !(*this == other);
 		}
 	};
-	using Selection = base::optional_variant<Selected, DeleteSelected>;
+	using Selection = std::variant<v::null_t, Selected, DeleteSelected>;
 
 	int getSelectionIndex(const Selection &selection) const;
 	void repaintPaper(int index);
 	void resizeToContentAndPreload();
 	void updatePapers();
 	void requestPapers();
+	void pushCustomPapers();
 	void sortPapers();
 	void paintPaper(
-		Painter &p,
+		QPainter &p,
 		const Paper &paper,
 		int column,
 		int row) const;
 	void validatePaperThumbnail(const Paper &paper) const;
 
 	const not_null<Main::Session*> _session;
+	PeerData * const _forPeer = nullptr;
+
 	MTP::Sender _api;
 
 	std::vector<Paper> _papers;
+	uint64 _currentId = 0;
+	uint64 _insertedResetId = 0;
 
 	Selection _over;
 	Selection _overDown;
@@ -134,8 +149,10 @@ private:
 
 BackgroundBox::BackgroundBox(
 	QWidget*,
-	not_null<Window::SessionController*> controller)
-: _controller(controller) {
+	not_null<Window::SessionController*> controller,
+	PeerData *forPeer)
+: _controller(controller)
+, _forPeer(forPeer) {
 }
 
 void BackgroundBox::prepare() {
@@ -145,15 +162,38 @@ void BackgroundBox::prepare() {
 
 	setDimensions(st::boxWideWidth, st::boxMaxListHeight);
 
-	_inner = setInnerWidget(
-		object_ptr<Inner>(this, &_controller->session()),
-		st::backgroundScroll);
+	auto wrap = object_ptr<Ui::VerticalLayout>(this);
+	const auto container = wrap.data();
+
+	Settings::AddSkip(container);
+
+	const auto button = container->add(Settings::CreateButton(
+		container,
+		tr::lng_settings_bg_from_file(),
+		st::infoProfileButton));
+	object_ptr<Info::Profile::FloatingIcon>(
+		button,
+		st::infoIconMediaPhoto,
+		st::infoSharedMediaButtonIconPosition);
+
+	button->setClickedCallback([=] {
+		chooseFromFile();
+	});
+
+	Settings::AddSkip(container);
+	Settings::AddDivider(container);
+
+	_inner = container->add(
+		object_ptr<Inner>(this, &_controller->session(), _forPeer));
+
+	container->resizeToWidth(st::boxWideWidth);
+
+	setInnerWidget(std::move(wrap), st::backgroundScroll);
+	setInnerTopSkip(st::lineWidth);
 
 	_inner->chooseEvents(
 	) | rpl::start_with_next([=](const Data::WallPaper &paper) {
-		Ui::show(
-			Box<BackgroundPreviewBox>(_controller, paper),
-			Ui::LayerOption::KeepOther);
+		chosen(paper);
 	}, _inner->lifetime());
 
 	_inner->removeRequests(
@@ -162,13 +202,124 @@ void BackgroundBox::prepare() {
 	}, _inner->lifetime());
 }
 
-void BackgroundBox::removePaper(const Data::WallPaper &paper) {
-	const auto box = std::make_shared<QPointer<Ui::BoxContent>>();
-	const auto session = &_controller->session();
-	const auto remove = [=, weak = Ui::MakeWeak(this)]{
-		if (*box) {
-			(*box)->closeBox();
+void BackgroundBox::chooseFromFile() {
+	const auto filterStart = _forPeer
+		? u"Image files (*"_q
+		: u"Theme files (*.tdesktop-theme *.tdesktop-palette *"_q;
+	auto filters = QStringList(
+		filterStart
+		+ Ui::ImageExtensions().join(u" *"_q)
+		+ u")"_q);
+	filters.push_back(FileDialog::AllFilesFilter());
+	const auto callback = [=](const FileDialog::OpenResult &result) {
+		if (result.paths.isEmpty() && result.remoteContent.isEmpty()) {
+			return;
 		}
+
+		if (!_forPeer && !result.paths.isEmpty()) {
+			const auto filePath = result.paths.front();
+			const auto hasExtension = [&](QLatin1String extension) {
+				return filePath.endsWith(extension, Qt::CaseInsensitive);
+			};
+			if (hasExtension(qstr(".tdesktop-theme"))
+				|| hasExtension(qstr(".tdesktop-palette"))) {
+				Window::Theme::Apply(filePath);
+				return;
+			}
+		}
+
+		auto image = Images::Read({
+			.path = result.paths.isEmpty() ? QString() : result.paths.front(),
+			.content = result.remoteContent,
+			.forceOpaque = true,
+		}).image;
+		if (image.isNull() || image.width() <= 0 || image.height() <= 0) {
+			return;
+		}
+		auto local = Data::CustomWallPaper();
+		local.setLocalImageAsThumbnail(std::make_shared<Image>(
+			std::move(image)));
+		_controller->show(Box<BackgroundPreviewBox>(
+			_controller,
+			local,
+			BackgroundPreviewArgs{ _forPeer }));
+	};
+	FileDialog::GetOpenPath(
+		this,
+		tr::lng_choose_image(tr::now),
+		filters.join(u";;"_q),
+		crl::guard(this, callback));
+}
+
+bool BackgroundBox::hasDefaultForPeer() const {
+	Expects(_forPeer != nullptr);
+
+	const auto paper = _forPeer->wallPaper();
+	if (!paper) {
+		return true;
+	}
+	const auto reset = _inner->resolveResetCustomPaper();
+	Assert(reset.has_value());
+	return (paper->id() == reset->id());
+}
+
+bool BackgroundBox::chosenDefaultForPeer(
+		const Data::WallPaper &paper) const {
+	if (!_forPeer) {
+		return false;
+	}
+
+	const auto reset = _inner->resolveResetCustomPaper();
+	Assert(reset.has_value());
+	return (paper.id() == reset->id());
+}
+
+void BackgroundBox::chosen(const Data::WallPaper &paper) {
+	if (chosenDefaultForPeer(paper)) {
+		if (!hasDefaultForPeer()) {
+			const auto reset = crl::guard(this, [=](Fn<void()> close) {
+				resetForPeer();
+				close();
+			});
+			_controller->show(Ui::MakeConfirmBox({
+				.text = tr::lng_background_sure_reset_default(),
+				.confirmed = reset,
+				.confirmText = tr::lng_background_reset_default(),
+			}));
+		} else {
+			closeBox();
+		}
+		return;
+	}
+	_controller->show(Box<BackgroundPreviewBox>(
+		_controller,
+		paper,
+		BackgroundPreviewArgs{ _forPeer }));
+}
+
+void BackgroundBox::resetForPeer() {
+	const auto api = &_controller->session().api();
+	api->request(MTPmessages_SetChatWallPaper(
+		MTP_flags(0),
+		_forPeer->input,
+		MTPInputWallPaper(),
+		MTPWallPaperSettings(),
+		MTPint()
+	)).done([=](const MTPUpdates &result) {
+		api->applyUpdates(result);
+	}).send();
+
+	const auto weak = Ui::MakeWeak(this);
+	_forPeer->setWallPaper(std::nullopt);
+	if (weak) {
+		_controller->finishChatThemeEdit(_forPeer);
+	}
+}
+
+void BackgroundBox::removePaper(const Data::WallPaper &paper) {
+	const auto session = &_controller->session();
+	const auto remove = [=, weak = Ui::MakeWeak(this)](Fn<void()> &&close) {
+		close();
 		if (weak) {
 			weak->_inner->removePaper(paper);
 		}
@@ -179,50 +330,57 @@ void BackgroundBox::removePaper(const Data::WallPaper &paper) {
 			paper.mtpSettings()
 		)).send();
 	};
-	*box = Ui::show(
-		Box<ConfirmBox>(
-			tr::lng_background_sure_delete(tr::now),
-			tr::lng_selected_delete(tr::now),
-			tr::lng_cancel(tr::now),
-			remove),
-		Ui::LayerOption::KeepOther);
+	_controller->show(Ui::MakeConfirmBox({
+		.text = tr::lng_background_sure_delete(),
+		.confirmed = remove,
+		.confirmText = tr::lng_selected_delete(),
+	}));
 }
 
 BackgroundBox::Inner::Inner(
 	QWidget *parent,
-	not_null<Main::Session*> session)
+	not_null<Main::Session*> session,
+	PeerData *forPeer)
 : RpWidget(parent)
 , _session(session)
+, _forPeer(forPeer)
 , _api(&_session->mtp())
 , _check(std::make_unique<Ui::RoundCheckbox>(st::overviewCheck, [=] { update(); })) {
 	_check->setChecked(true, anim::type::instant);
-	if (_session->data().wallpapers().empty()) {
-		resize(st::boxWideWidth, 2 * (st::backgroundSize.height() + st::backgroundPadding) + st::backgroundPadding);
-	} else {
+	resize(st::boxWideWidth, 2 * (st::backgroundSize.height() + st::backgroundPadding) + st::backgroundPadding);
+	Window::Theme::IsNightModeValue(
+	) | rpl::start_with_next([=] {
 		updatePapers();
-	}
+	}, lifetime());
 	requestPapers();
 
 	_session->downloaderTaskFinished(
 	) | rpl::start_with_next([=] {
 		update();
 	}, lifetime());
+
+	style::PaletteChanged(
+	) | rpl::start_with_next([=] {
+		_check->invalidateCache();
+	}, lifetime());
+
 	using Update = Window::Theme::BackgroundUpdate;
-	subscribe(Window::Theme::Background(), [=](const Update &update) {
-		if (update.paletteChanged()) {
-			_check->invalidateCache();
-		} else if (update.type == Update::Type::New) {
+	Window::Theme::Background()->updates(
+	) | rpl::start_with_next([=](const Update &update) {
+		if (update.type == Update::Type::New) {
 			sortPapers();
 			requestPapers();
 			this->update();
 		}
-	});
+	}, lifetime());
+
+
 	setMouseTracking(true);
 }
 
 void BackgroundBox::Inner::requestPapers() {
 	_api.request(MTPaccount_GetWallPapers(
-		MTP_int(_session->data().wallpapersHash())
+		MTP_long(_session->data().wallpapersHash())
 	)).done([=](const MTPaccount_WallPapers &result) {
 		if (_session->data().updateWallpapers(result)) {
 			updatePapers();
@@ -230,32 +388,106 @@ void BackgroundBox::Inner::requestPapers() {
 	}).send();
 }
 
+auto BackgroundBox::Inner::resolveResetCustomPaper() const
+-> std::optional<Data::WallPaper> {
+	if (!_forPeer) {
+		return {};
+	}
+	const auto nonCustom = Window::Theme::Background()->paper();
+	const auto themeEmoji = _forPeer->themeEmoji();
+	if (themeEmoji.isEmpty()) {
+		return nonCustom;
+	}
+	const auto &themes = _forPeer->owner().cloudThemes();
+	const auto theme = themes.themeForEmoji(themeEmoji);
+	if (!theme) {
+		return nonCustom;
+	}
+	using Type = Data::CloudTheme::Type;
+	const auto dark = Window::Theme::IsNightMode();
+	const auto i = theme->settings.find(dark ? Type::Dark : Type::Light);
+	if (i != end(theme->settings) && i->second.paper) {
+		return *i->second.paper;
+	}
+	return nonCustom;
+}
+
+void BackgroundBox::Inner::pushCustomPapers() {
+	auto customId = uint64();
+	if (const auto custom = _forPeer ? _forPeer->wallPaper() : nullptr) {
+		customId = custom->id();
+		const auto j = ranges::find(
+			_papers,
+			custom->id(),
+			[](const Paper &paper) { return paper.data.id(); });
+		if (j != end(_papers)) {
+			j->data = j->data.withParamsFrom(*custom);
+		} else {
+			_papers.insert(begin(_papers), Paper{ *custom });
+		}
+	}
+	if (const auto reset = resolveResetCustomPaper()) {
+		_insertedResetId = reset->id();
+		const auto j = ranges::find(
+			_papers,
+			_insertedResetId,
+			[](const Paper &paper) { return paper.data.id(); });
+		if (j != end(_papers)) {
+			if (_insertedResetId != customId) {
+				j->data = j->data.withParamsFrom(*reset);
+			}
+		} else {
+			_papers.insert(begin(_papers), Paper{ *reset });
+		}
+	}
+}
+
 void BackgroundBox::Inner::sortPapers() {
-	const auto current = Window::Theme::Background()->id();
-	const auto night = Window::Theme::IsNightMode();
+	const auto currentCustom = _forPeer ? _forPeer->wallPaper() : nullptr;
+	_currentId = currentCustom
+		? currentCustom->id()
+		: _insertedResetId
+		? _insertedResetId
+		: Window::Theme::Background()->id();
+	const auto dark = Window::Theme::IsNightMode();
 	ranges::stable_sort(_papers, std::greater<>(), [&](const Paper &paper) {
 		const auto &data = paper.data;
 		return std::make_tuple(
-			data.id() == current,
-			night ? data.isDark() : !data.isDark(),
-			!data.isDefault() && !data.isLocal(),
-			!data.isDefault() && data.isLocal());
+			_insertedResetId && (_insertedResetId == data.id()),
+			data.id() == _currentId,
+			dark ? data.isDark() : !data.isDark(),
+			Data::IsDefaultWallPaper(data),
+			!data.isDefault() && !Data::IsLegacy1DefaultWallPaper(data),
+			Data::IsLegacy3DefaultWallPaper(data),
+			Data::IsLegacy2DefaultWallPaper(data),
+			Data::IsLegacy1DefaultWallPaper(data));
 	});
-	if (!_papers.empty() && _papers.front().data.id() == current) {
+	if (!_papers.empty()
+		&& _papers.front().data.id() == _currentId
+		&& !currentCustom
+		&& !_insertedResetId) {
 		_papers.front().data = _papers.front().data.withParamsFrom(
 			Window::Theme::Background()->paper());
 	}
 }
 
 void BackgroundBox::Inner::updatePapers() {
+	if (_session->data().wallpapers().empty()) {
+		return;
+	}
 	_over = _overDown = Selection();
 
 	_papers = _session->data().wallpapers(
-	) | ranges::view::filter([](const Data::WallPaper &paper) {
-		return !paper.isPattern() || paper.backgroundColor().has_value();
-	}) | ranges::view::transform([](const Data::WallPaper &paper) {
+	) | ranges::views::filter([&](const Data::WallPaper &paper) {
+		return (!paper.isPattern() || !paper.backgroundColors().empty())
+			&& (!_forPeer
+				|| (!Data::IsDefaultWallPaper(paper)
+					&& (Data::IsCloudWallPaper(paper)
+						|| Data::IsCustomWallPaper(paper))));
+	}) | ranges::views::transform([](const Data::WallPaper &paper) {
 		return Paper{ paper };
 	}) | ranges::to_vector;
+	pushCustomPapers();
 	sortPapers();
 	resizeToContentAndPreload();
 }
@@ -271,7 +503,7 @@ void BackgroundBox::Inner::resizeToContentAndPreload() {
 			+ st::backgroundPadding));
 
 	const auto preload = kBackgroundsInRow * 3;
-	for (const auto &paper : _papers | ranges::view::take(preload)) {
+	for (const auto &paper : _papers | ranges::views::take(preload)) {
 		if (!paper.data.localThumbnail() && !paper.dataMedia) {
 			if (const auto document = paper.data.document()) {
 				paper.dataMedia = document->createMediaView();
@@ -284,7 +516,7 @@ void BackgroundBox::Inner::resizeToContentAndPreload() {
 
 void BackgroundBox::Inner::paintEvent(QPaintEvent *e) {
 	QRect r(e->rect());
-	Painter p(this);
+	auto p = QPainter(this);
 
 	if (_papers.empty()) {
 		p.setFont(st::noContactsFont);
@@ -323,8 +555,18 @@ void BackgroundBox::Inner::validatePaperThumbnail(
 				paper.dataMedia = document->createMediaView();
 				paper.dataMedia->thumbnailWanted(paper.data.fileOrigin());
 			}
-		}
-		if (!paper.dataMedia || !paper.dataMedia->thumbnail()) {
+			if (!paper.dataMedia->thumbnail()) {
+				return;
+			}
+		} else if (!paper.data.backgroundColors().empty()) {
+			paper.thumbnail = Ui::PixmapFromImage(
+				Ui::GenerateBackgroundImage(
+					st::backgroundSize * cIntRetinaFactor(),
+					paper.data.backgroundColors(),
+					paper.data.gradientRotation()));
+			paper.thumbnail.setDevicePixelRatio(cRetinaFactor());
+			return;
+		} else {
 			return;
 		}
 	}
@@ -333,21 +575,20 @@ void BackgroundBox::Inner::validatePaperThumbnail(
 		: paper.dataMedia->thumbnail();
 	auto original = thumbnail->original();
 	if (paper.data.isPattern()) {
-		const auto color = *paper.data.backgroundColor();
-		original = Data::PreparePatternImage(
+		original = Ui::PreparePatternImage(
 			std::move(original),
-			color,
-			Data::PatternColor(color),
-			paper.data.patternIntensity());
+			paper.data.backgroundColors(),
+			paper.data.gradientRotation(),
+			paper.data.patternOpacity());
 	}
-	paper.thumbnail = App::pixmapFromImageInPlace(TakeMiddleSample(
+	paper.thumbnail = Ui::PixmapFromImage(TakeMiddleSample(
 		original,
 		st::backgroundSize));
 	paper.thumbnail.setDevicePixelRatio(cRetinaFactor());
 }
 
 void BackgroundBox::Inner::paintPaper(
-		Painter &p,
+		QPainter &p,
 		const Paper &paper,
 		int column,
 		int row) const {
@@ -358,16 +599,18 @@ void BackgroundBox::Inner::paintPaper(
 		p.drawPixmap(x, y, paper.thumbnail);
 	}
 
-	const auto over = _overDown ? _overDown : _over;
-	if (paper.data.id() == Window::Theme::Background()->id()) {
+	const auto over = !v::is_null(_overDown) ? _overDown : _over;
+	if (paper.data.id() == _currentId) {
 		const auto checkLeft = x + st::backgroundSize.width() - st::overviewCheckSkip - st::overviewCheck.size;
 		const auto checkTop = y + st::backgroundSize.height() - st::overviewCheckSkip - st::overviewCheck.size;
 		_check->paint(p, checkLeft, checkTop, width());
 	} else if (Data::IsCloudWallPaper(paper.data)
 		&& !Data::IsDefaultWallPaper(paper.data)
-		&& over.has_value()
+		&& !Data::IsLegacy2DefaultWallPaper(paper.data)
+		&& !Data::IsLegacy3DefaultWallPaper(paper.data)
+		&& !v::is_null(over)
 		&& (&paper == &_papers[getSelectionIndex(over)])) {
-		const auto deleteSelected = over.is<DeleteSelected>();
+		const auto deleteSelected = v::is<DeleteSelected>(over);
 		const auto deletePos = QPoint(x + st::backgroundSize.width() - st::stickerPanDeleteIconBg.width(), y);
 		p.setOpacity(deleteSelected ? st::stickerPanDeleteOpacityBgOver : st::stickerPanDeleteOpacityBg);
 		st::stickerPanDeleteIconBg.paint(p, deletePos, width());
@@ -394,16 +637,18 @@ void BackgroundBox::Inner::mouseMoveEvent(QMouseEvent *e) {
 		} else if (result >= _papers.size()) {
 			return Selection();
 		}
+		auto &data = _papers[result].data;
 		const auto deleteLeft = (column + 1) * (width + skip)
 			- st::stickerPanDeleteIconBg.width();
 		const auto deleteBottom = row * (height + skip) + skip
 			+ st::stickerPanDeleteIconBg.height();
-		const auto currentId = Window::Theme::Background()->id();
 		const auto inDelete = (x >= deleteLeft)
 			&& (y < deleteBottom)
-			&& Data::IsCloudWallPaper(_papers[result].data)
-			&& !Data::IsDefaultWallPaper(_papers[result].data)
-			&& (currentId != _papers[result].data.id());
+			&& Data::IsCloudWallPaper(data)
+			&& !Data::IsDefaultWallPaper(data)
+			&& !Data::IsLegacy2DefaultWallPaper(data)
+			&& !Data::IsLegacy3DefaultWallPaper(data)
+			&& (_currentId != data.id());
 		return (result >= _papers.size())
 			? Selection()
 			: inDelete
@@ -414,7 +659,7 @@ void BackgroundBox::Inner::mouseMoveEvent(QMouseEvent *e) {
 		repaintPaper(getSelectionIndex(_over));
 		_over = newOver;
 		repaintPaper(getSelectionIndex(_over));
-		setCursor((_over.has_value() || _overDown.has_value())
+		setCursor((!v::is_null(_over) || !v::is_null(_overDown))
 			? style::cur_pointer
 			: style::cur_default);
 	}
@@ -442,22 +687,22 @@ void BackgroundBox::Inner::mousePressEvent(QMouseEvent *e) {
 
 int BackgroundBox::Inner::getSelectionIndex(
 		const Selection &selection) const {
-	return selection.match([](const Selected &data) {
+	return v::match(selection, [](const Selected &data) {
 		return data.index;
 	}, [](const DeleteSelected &data) {
 		return data.index;
-	}, [](std::nullopt_t) {
+	}, [](v::null_t) {
 		return -1;
 	});
 }
 
 void BackgroundBox::Inner::mouseReleaseEvent(QMouseEvent *e) {
-	if (base::take(_overDown) == _over && _over.has_value()) {
+	if (base::take(_overDown) == _over && !v::is_null(_over)) {
 		const auto index = getSelectionIndex(_over);
 		if (index >= 0 && index < _papers.size()) {
-			if (base::get_if<DeleteSelected>(&_over)) {
+			if (std::get_if<DeleteSelected>(&_over)) {
 				_backgroundRemove.fire_copy(_papers[index].data);
-			} else if (base::get_if<Selected>(&_over)) {
+			} else if (std::get_if<Selected>(&_over)) {
 				auto &paper = _papers[index];
 				if (!paper.dataMedia) {
 					if (const auto document = paper.data.document()) {
@@ -468,7 +713,7 @@ void BackgroundBox::Inner::mouseReleaseEvent(QMouseEvent *e) {
 				_backgroundChosen.fire_copy(paper.data);
 			}
 		}
-	} else if (!_over.has_value()) {
+	} else if (v::is_null(_over)) {
 		setCursor(style::cur_default);
 	}
 }
