@@ -18,7 +18,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "mtproto/facade.h"
 #include "ui/widgets/checkbox.h"
 #include "ui/widgets/buttons.h"
-#include "ui/widgets/input_fields.h"
+#include "ui/widgets/fields/input_field.h"
+#include "ui/widgets/fields/number_input.h"
+#include "ui/widgets/fields/password_input.h"
 #include "ui/widgets/labels.h"
 #include "ui/widgets/dropdown_menu.h"
 #include "ui/wrap/slide_wrap.h"
@@ -29,6 +31,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/text/text_options.h"
 #include "ui/text/text_utilities.h"
 #include "ui/basic_click_handlers.h"
+#include "ui/painter.h"
 #include "boxes/abstract_box.h" // Ui::show().
 #include "styles/style_layers.h"
 #include "styles/style_boxes.h"
@@ -458,7 +461,7 @@ void ProxyRow::paintEvent(QPaintEvent *e) {
 void ProxyRow::paintCheck(Painter &p) {
 	const auto loading = _progress
 		? _progress->computeState()
-		: Ui::RadialState{ 0., 0, FullArcLength };
+		: Ui::RadialState{ 0., 0, arc::kFullLength };
 	const auto toggled = _toggled.value(_view.selected ? 1. : 0.)
 		* (1. - loading.shown);
 	const auto _st = &st::defaultRadio;
@@ -483,7 +486,7 @@ void ProxyRow::paintCheck(Painter &p) {
 			_st->thickness,
 			pen.color(),
 			_st->bg);
-	} else if (loading.arcLength < FullArcLength) {
+	} else if (loading.arcLength < arc::kFullLength) {
 		p.drawArc(rect, loading.arcFrom, loading.arcLength);
 	} else {
 		p.drawEllipse(rect);
@@ -1091,17 +1094,17 @@ ProxiesBoxController::ProxiesBoxController(not_null<Main::Account*> account)
 void ProxiesBoxController::ShowApplyConfirmation(
 		Type type,
 		const QMap<QString, QString> &fields) {
-	const auto server = fields.value(qsl("server"));
-	const auto port = fields.value(qsl("port")).toUInt();
+	const auto server = fields.value(u"server"_q);
+	const auto port = fields.value(u"port"_q).toUInt();
 	auto proxy = ProxyData();
 	proxy.type = type;
 	proxy.host = server;
 	proxy.port = port;
 	if (type == Type::Socks5) {
-		proxy.user = fields.value(qsl("user"));
-		proxy.password = fields.value(qsl("pass"));
+		proxy.user = fields.value(u"user"_q);
+		proxy.password = fields.value(u"pass"_q);
 	} else if (type == Type::Mtproto) {
-		proxy.password = fields.value(qsl("secret"));
+		proxy.password = fields.value(u"secret"_q);
 	}
 	if (proxy) {
 		const auto displayed = "https://" + server + "/";
@@ -1264,7 +1267,7 @@ object_ptr<Ui::BoxContent> ProxiesBoxController::CreateOwningBox(
 
 object_ptr<Ui::BoxContent> ProxiesBoxController::create() {
 	auto result = Box<ProxiesBox>(this, _settings);
-	_toastParent = Ui::BoxShow(result.data()).toastParent();
+	_show = result->uiShow();
 	for (const auto &item : _list) {
 		updateView(item);
 	}
@@ -1503,12 +1506,9 @@ void ProxiesBoxController::updateView(const Item &item) {
 	const auto deleted = item.deleted;
 	const auto type = [&] {
 		switch (item.data.type) {
-		case Type::Http:
-			return qsl("HTTP");
-		case Type::Socks5:
-			return qsl("SOCKS5");
-		case Type::Mtproto:
-			return qsl("MTPROTO");
+		case Type::Http: return u"HTTP"_q;
+		case Type::Socks5: return u"SOCKS5"_q;
+		case Type::Mtproto: return u"MTPROTO"_q;
 		}
 		Unexpected("Proxy type in ProxiesBoxController::updateView.");
 	}();
@@ -1540,7 +1540,7 @@ void ProxiesBoxController::share(const ProxyData &proxy) {
 	if (proxy.type == Type::Http) {
 		return;
 	}
-	const auto link = qsl("https://t.me/")
+	const auto link = u"https://t.me/"_q
 		+ (proxy.type == Type::Socks5 ? "socks" : "proxy")
 		+ "?server=" + proxy.host + "&port=" + QString::number(proxy.port)
 		+ ((proxy.type == Type::Socks5 && !proxy.user.isEmpty())
@@ -1550,9 +1550,7 @@ void ProxiesBoxController::share(const ProxyData &proxy) {
 		+ ((proxy.type == Type::Mtproto && !proxy.password.isEmpty())
 			? "&secret=" + proxy.password : "");
 	QGuiApplication::clipboard()->setText(link);
-	if (_toastParent) {
-		Ui::Toast::Show(_toastParent, tr::lng_username_copied(tr::now));
-	}
+	_show->showToast(tr::lng_username_copied(tr::now));
 }
 
 ProxiesBoxController::~ProxiesBoxController() {
