@@ -7,18 +7,19 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "boxes/peers/edit_peer_type_box.h"
 
-#include "apiwrap.h"
-#include "api/api_invite_links.h"
 #include "main/main_session.h"
 #include "boxes/add_contact_box.h"
-#include "boxes/confirm_box.h"
+#include "ui/boxes/confirm_box.h"
+#include "boxes/premium_limits_box.h"
 #include "boxes/peer_list_controllers.h"
 #include "boxes/peers/edit_participants_box.h"
+#include "boxes/peers/edit_peer_common.h"
 #include "boxes/peers/edit_peer_info_box.h" // CreateButton.
 #include "boxes/peers/edit_peer_invite_link.h"
 #include "boxes/peers/edit_peer_invite_links.h"
+#include "boxes/peers/edit_peer_usernames_list.h"
+#include "boxes/username_box.h"
 #include "chat_helpers/emoji_suggestions_widget.h"
-#include "core/application.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
 #include "data/data_peer.h"
@@ -26,21 +27,18 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_changes.h"
 #include "info/profile/info_profile_values.h"
 #include "lang/lang_keys.h"
-#include "mainwindow.h"
 #include "mtproto/sender.h"
 #include "ui/rp_widget.h"
-#include "ui/special_buttons.h"
-#include "ui/toast/toast.h"
+#include "ui/controls/userpic_button.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/checkbox.h"
-#include "ui/widgets/input_fields.h"
+#include "ui/widgets/fields/input_field.h"
 #include "ui/widgets/labels.h"
-#include "ui/widgets/popup_menu.h"
 #include "ui/widgets/box_content_divider.h"
 #include "ui/wrap/padding_wrap.h"
 #include "ui/wrap/slide_wrap.h"
 #include "ui/wrap/vertical_layout.h"
-#include "ui/special_fields.h"
+#include "ui/widgets/fields/special_fields.h"
 #include "window/window_session_controller.h"
 #include "settings/settings_common.h"
 #include "styles/style_layers.h"
@@ -48,45 +46,53 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "styles/style_info.h"
 #include "styles/style_settings.h"
 
-#include <QtGui/QGuiApplication>
-#include <QtGui/QClipboard>
-
-#include <rpl/flatten_latest.h>
-
 namespace {
 
-constexpr auto kUsernameCheckTimeout = crl::time(200);
-constexpr auto kMinUsernameLength = 5;
-
 class Controller : public base::has_weak_ptr {
 public:
 	Controller(
+		Window::SessionNavigation *navigation,
+		std::shared_ptr<Ui::Show> show,
 		not_null<Ui::VerticalLayout*> container,
 		not_null<PeerData*> peer,
 		bool useLocationPhrases,
-		std::optional<Privacy> privacySavedValue,
-		std::optional<QString> usernameSavedValue);
+		std::optional<EditPeerTypeData> dataSavedValue);
 
 	void createContent();
-	QString getUsernameInput();
+	[[nodiscard]] QString getUsernameInput() const;
+	[[nodiscard]] std::vector<QString> usernamesOrder() const;
 	void setFocusUsername();
 
-	rpl::producer<QString> getTitle() {
-		return !_privacySavedValue
+	[[nodiscard]] rpl::producer<QString> getTitle() const {
+		return !_dataSavedValue
 			? tr::lng_create_invite_link_title()
 			: _isGroup
 			? tr::lng_manage_peer_group_type()
 			: tr::lng_manage_peer_channel_type();
 	}
 
-	bool isAllowSave() {
-		return _isAllowSave;
+	[[nodiscard]] bool goodUsername() const {
+		return _goodUsername;
 	}
 
-	Privacy getPrivacy() {
+	[[nodiscard]] Privacy getPrivacy() const {
 		return _controls.privacy->value();
 	}
 
+	[[nodiscard]] bool noForwards() const {
+		return _controls.noForwards->toggled();
+	}
+	[[nodiscard]] bool joinToWrite() const {
+		return _controls.joinToWrite && _controls.joinToWrite->toggled();
+	}
+	[[nodiscard]] bool requestToJoin() const {
+		return _controls.requestToJoin && _controls.requestToJoin->toggled();
+	}
+
+	[[nodiscard]] rpl::producer<int> scrollToRequests() const {
+		return _scrollToRequests.events();
+	}
+
 	void showError(rpl::producer<QString> text) {
 		_controls.usernameInput->showError();
 		showUsernameError(std::move(text));
@@ -97,11 +103,16 @@ private:
 		std::shared_ptr<Ui::RadioenumGroup<Privacy>> privacy;
 		Ui::SlideWrap<Ui::RpWidget> *usernameWrap = nullptr;
 		Ui::UsernameInput *usernameInput = nullptr;
-		base::unique_qptr<Ui::FlatLabel> usernameResult;
-		const style::FlatLabel *usernameResultStyle = nullptr;
+		UsernamesList *usernamesList = nullptr;
+		base::unique_qptr<Ui::FlatLabel> usernameCheckResult;
 
-		Ui::SlideWrap<Ui::RpWidget> *inviteLinkWrap = nullptr;
+		Ui::SlideWrap<> *inviteLinkWrap = nullptr;
 		Ui::FlatLabel *inviteLink = nullptr;
+
+		Ui::SlideWrap<Ui::VerticalLayout> *whoSendWrap = nullptr;
+		Ui::SettingsButton *noForwards = nullptr;
+		Ui::SettingsButton *joinToWrite = nullptr;
+		Ui::SettingsButton *requestToJoin = nullptr;
 	};
 
 	Controls _controls;
@@ -116,9 +127,8 @@ private:
 	void usernameChanged();
 	void showUsernameError(rpl::producer<QString> &&error);
 	void showUsernameGood();
-	void showUsernameResult(
-		rpl::producer<QString> &&text,
-		not_null<const style::FlatLabel*> st);
+	void showUsernamePending();
+	void showUsernameEmpty();
 
 	void fillPrivaciesButtons(
 		not_null<Ui::VerticalLayout*> parent,
@@ -129,41 +139,51 @@ private:
 		const QString &text,
 		rpl::producer<QString> about);
 
+	Window::SessionNavigation *_navigation = nullptr;
+	std::shared_ptr<Ui::Show> _show;
+
 	not_null<PeerData*> _peer;
 	bool _linkOnly = false;
 
 	MTP::Sender _api;
-	std::optional<Privacy> _privacySavedValue;
-	std::optional<QString> _usernameSavedValue;
+	std::optional<EditPeerTypeData> _dataSavedValue;
 
 	bool _useLocationPhrases = false;
 	bool _isGroup = false;
-	bool _isAllowSave = false;
+	bool _goodUsername = false;
 
 	base::unique_qptr<Ui::VerticalLayout> _wrap;
 	base::Timer _checkUsernameTimer;
 	mtpRequestId _checkUsernameRequestId = 0;
 	UsernameState _usernameState = UsernameState::Normal;
-	rpl::event_stream<rpl::producer<QString>> _usernameResultTexts;
+
+	rpl::event_stream<UsernameCheckInfo> _usernameCheckInfo;
+	rpl::lifetime _usernameCheckInfoLifetime;
+
+	rpl::event_stream<int> _scrollToRequests;
 
 	rpl::lifetime _lifetime;
 
 };
 
 Controller::Controller(
+	Window::SessionNavigation *navigation,
+	std::shared_ptr<Ui::Show> show,
 	not_null<Ui::VerticalLayout*> container,
 	not_null<PeerData*> peer,
 	bool useLocationPhrases,
-	std::optional<Privacy> privacySavedValue,
-	std::optional<QString> usernameSavedValue)
-: _peer(peer)
-, _linkOnly(!privacySavedValue.has_value())
+	std::optional<EditPeerTypeData> dataSavedValue)
+: _navigation(navigation)
+, _show(show)
+, _peer(peer)
+, _linkOnly(!dataSavedValue.has_value())
 , _api(&_peer->session().mtp())
-, _privacySavedValue(privacySavedValue)
-, _usernameSavedValue(usernameSavedValue)
+, _dataSavedValue(dataSavedValue)
 , _useLocationPhrases(useLocationPhrases)
 , _isGroup(_peer->isChat() || _peer->isMegagroup())
-, _isAllowSave(!_usernameSavedValue.value_or(QString()).isEmpty())
+, _goodUsername(_dataSavedValue
+	? !_dataSavedValue->username.isEmpty()
+	: (_peer->isChannel() && !_peer->asChannel()->editableUsername().isEmpty()))
 , _wrap(container)
 , _checkUsernameTimer([=] { checkUsernameAvailability(); }) {
 	_peer->updateFull();
@@ -172,7 +192,12 @@ Controller::Controller(
 void Controller::createContent() {
 	_controls = Controls();
 
-	fillPrivaciesButtons(_wrap, _privacySavedValue);
+	fillPrivaciesButtons(
+		_wrap,
+		(_dataSavedValue
+			? _dataSavedValue->privacy
+			: std::optional<Privacy>()));
+
 	// Skip.
 	if (!_linkOnly) {
 		_wrap->add(object_ptr<Ui::BoxContentDivider>(_wrap));
@@ -184,27 +209,102 @@ void Controller::createContent() {
 	}
 
 	using namespace Settings;
-	AddSkip(_wrap.get());
-	_wrap->add(EditPeerInfoBox::CreateButton(
-		_wrap.get(),
-		tr::lng_group_invite_manage(),
-		rpl::single(QString()),
-		[=] { Ui::show(
-			Box(ManageInviteLinksBox, _peer, _peer->session().user(), 0, 0),
-			Ui::LayerOption::KeepOther);
-		},
-		st::manageGroupButton,
-		&st::infoIconInviteLinks));
-	AddSkip(_wrap.get());
-	AddDividerText(_wrap.get(), tr::lng_group_invite_manage_about());
 
+	if (!_linkOnly) {
+		if (_peer->isMegagroup()) {
+			_controls.whoSendWrap = _wrap->add(
+				object_ptr<Ui::SlideWrap<Ui::VerticalLayout>>(
+					_wrap.get(),
+					object_ptr<Ui::VerticalLayout>(_wrap.get())));
+			const auto wrap = _controls.whoSendWrap->entity();
+
+			AddSkip(wrap);
+			if (_dataSavedValue->hasLinkedChat) {
+				AddSubsectionTitle(wrap, tr::lng_manage_peer_send_title());
+
+				_controls.joinToWrite = wrap->add(EditPeerInfoBox::CreateButton(
+					wrap,
+					tr::lng_manage_peer_send_only_members(),
+					rpl::single(QString()),
+					[=] {},
+					st::peerPermissionsButton,
+					{}
+				));
+				_controls.joinToWrite->toggleOn(
+					rpl::single(_dataSavedValue->joinToWrite)
+				)->toggledValue(
+				) | rpl::start_with_next([=](bool toggled) {
+					_dataSavedValue->joinToWrite = toggled;
+				}, wrap->lifetime());
+			} else {
+				_controls.whoSendWrap->toggle(
+					(_controls.privacy->value() == Privacy::HasUsername),
+					anim::type::instant);
+			}
+			auto joinToWrite = _controls.joinToWrite
+				? _controls.joinToWrite->toggledValue()
+				: rpl::single(true);
+
+			const auto requestToJoinWrap = wrap->add(
+				object_ptr<Ui::SlideWrap<Ui::SettingsButton>>(
+					wrap,
+					EditPeerInfoBox::CreateButton(
+						wrap,
+						tr::lng_manage_peer_send_approve_members(),
+						rpl::single(QString()),
+						[=] {},
+						st::peerPermissionsButton,
+						{})))->setDuration(0);
+			requestToJoinWrap->toggleOn(rpl::duplicate(joinToWrite));
+			_controls.requestToJoin = requestToJoinWrap->entity();
+			_controls.requestToJoin->toggleOn(
+				rpl::single(_dataSavedValue->requestToJoin)
+			)->toggledValue(
+			) | rpl::start_with_next([=](bool toggled) {
+				_dataSavedValue->requestToJoin = toggled;
+			}, wrap->lifetime());
+
+			AddSkip(wrap);
+			AddDividerText(
+				wrap,
+				rpl::conditional(
+					std::move(joinToWrite),
+					tr::lng_manage_peer_send_approve_members_about(),
+					tr::lng_manage_peer_send_only_members_about()));
+		}
+		AddSkip(_wrap.get());
+		AddSubsectionTitle(
+			_wrap.get(),
+			tr::lng_manage_peer_no_forwards_title());
+		_controls.noForwards = _wrap->add(EditPeerInfoBox::CreateButton(
+			_wrap.get(),
+			tr::lng_manage_peer_no_forwards(),
+			rpl::single(QString()),
+			[] {},
+			st::peerPermissionsButton,
+			{}));
+		_controls.noForwards->toggleOn(
+			rpl::single(_dataSavedValue->noForwards)
+		)->toggledValue(
+		) | rpl::start_with_next([=](bool toggled) {
+			_dataSavedValue->noForwards = toggled;
+		}, _wrap->lifetime());
+		AddSkip(_wrap.get());
+		AddDividerText(
+			_wrap.get(),
+			(_isGroup
+				? tr::lng_manage_peer_no_forwards_about
+				: tr::lng_manage_peer_no_forwards_about_channel)());
+	}
 	if (_linkOnly) {
 		_controls.inviteLinkWrap->show(anim::type::instant);
 	} else {
 		if (_controls.privacy->value() == Privacy::NoUsername) {
 			checkUsernameAvailability();
 		}
-		const auto forShowing = _privacySavedValue.value_or(Privacy::NoUsername);
+		const auto forShowing = _dataSavedValue
+			? _dataSavedValue->privacy
+			: Privacy::NoUsername;
 		_controls.inviteLinkWrap->toggle(
 			(forShowing != Privacy::HasUsername),
 			anim::type::instant);
@@ -240,15 +340,7 @@ void Controller::addRoundButton(
 void Controller::fillPrivaciesButtons(
 		not_null<Ui::VerticalLayout*> parent,
 		std::optional<Privacy> savedValue) {
-	const auto canEditUsername = [&] {
-		if (const auto chat = _peer->asChat()) {
-			return chat->canEditUsername();
-		} else if (const auto channel = _peer->asChannel()) {
-			return channel->canEditUsername();
-		}
-		Unexpected("Peer type in Controller::createPrivaciesEdit.");
-	}();
-	if (!canEditUsername || _linkOnly) {
+	if (_linkOnly) {
 		return;
 	}
 
@@ -301,16 +393,23 @@ void Controller::setFocusUsername() {
 	}
 }
 
-QString Controller::getUsernameInput() {
+QString Controller::getUsernameInput() const {
 	return _controls.usernameInput->getLastText().trimmed();
 }
 
+std::vector<QString> Controller::usernamesOrder() const {
+	return _controls.usernamesList
+		? _controls.usernamesList->order()
+		: std::vector<QString>();
+}
+
 object_ptr<Ui::RpWidget> Controller::createUsernameEdit() {
 	Expects(_wrap != nullptr);
 
 	const auto channel = _peer->asChannel();
-	const auto username =
-		_usernameSavedValue.value_or(channel ? channel->username : QString());
+	const auto username = (!_dataSavedValue || !channel)
+		? QString()
+		: channel->editableUsername();
 
 	auto result = object_ptr<Ui::SlideWrap<Ui::VerticalLayout>>(
 		_wrap,
@@ -352,10 +451,24 @@ object_ptr<Ui::RpWidget> Controller::createUsernameEdit() {
 	}, placeholder->lifetime());
 	_controls.usernameInput->move(placeholder->pos());
 
+	AddUsernameCheckLabel(container, _usernameCheckInfo.events());
+
 	AddDividerText(
 		container,
 		tr::lng_create_channel_link_about());
 
+	if (channel) {
+		const auto focusCallback = [=] {
+			_scrollToRequests.fire(container->y());
+			_controls.usernameInput->setFocusFast();
+		};
+		_controls.usernamesList = container->add(object_ptr<UsernamesList>(
+			container,
+			channel,
+			_show,
+			focusCallback));
+	}
+
 	QObject::connect(
 		_controls.usernameInput,
 		&Ui::UsernameInput::changed,
@@ -378,6 +491,15 @@ void Controller::privacyChanged(Privacy value) {
 			(value == Privacy::HasUsername),
 			anim::type::instant);
 	};
+	const auto toggleWhoSendWrap = [&] {
+		if (!_controls.whoSendWrap) {
+			return;
+		}
+		_controls.whoSendWrap->toggle(
+			(value == Privacy::HasUsername
+				|| (_dataSavedValue && _dataSavedValue->hasLinkedChat)),
+			anim::type::instant);
+	};
 	const auto refreshVisibilities = [&] {
 		// Now first we need to hide that was shown.
 		// Otherwise box will change own Y position.
@@ -385,10 +507,12 @@ void Controller::privacyChanged(Privacy value) {
 		if (value == Privacy::HasUsername) {
 			toggleInviteLink();
 			toggleEditUsername();
+			toggleWhoSendWrap();
 
-			_controls.usernameResult = nullptr;
+			showUsernameEmpty();
 			checkUsernameAvailability();
 		} else {
+			toggleWhoSendWrap();
 			toggleEditUsername();
 			toggleInviteLink();
 		}
@@ -417,16 +541,16 @@ void Controller::checkUsernameAvailability() {
 	}
 	const auto initial = (_controls.privacy->value() != Privacy::HasUsername);
 	const auto checking = initial
-		? qsl(".bad.")
+		? u".bad."_q
 		: getUsernameInput();
-	if (checking.size() < kMinUsernameLength) {
+	if (checking.size() < Ui::EditPeer::kMinUsernameLength) {
 		return;
 	}
 	if (_checkUsernameRequestId) {
 		_api.request(_checkUsernameRequestId).cancel();
 	}
 	const auto channel = _peer->migrateToOrMe()->asChannel();
-	const auto username = channel ? channel->username : QString();
+	const auto username = channel ? channel->editableUsername() : QString();
 	_checkUsernameRequestId = _api.request(MTPchannels_CheckUsername(
 		channel ? channel->inputChannel : MTP_inputChannelEmpty(),
 		MTP_string(checking)
@@ -444,23 +568,27 @@ void Controller::checkUsernameAvailability() {
 		_checkUsernameRequestId = 0;
 		const auto &type = error.type();
 		_usernameState = UsernameState::Normal;
-		if (type == qstr("CHANNEL_PUBLIC_GROUP_NA")) {
+		if (type == u"CHANNEL_PUBLIC_GROUP_NA"_q) {
 			_usernameState = UsernameState::NotAvailable;
 			_controls.privacy->setValue(Privacy::NoUsername);
-		} else if (type == qstr("CHANNELS_ADMIN_PUBLIC_TOO_MUCH")) {
+		} else if (type == u"CHANNELS_ADMIN_PUBLIC_TOO_MUCH"_q) {
 			_usernameState = UsernameState::TooMany;
 			if (_controls.privacy->value() == Privacy::HasUsername) {
 				askUsernameRevoke();
 			}
 		} else if (initial) {
 			if (_controls.privacy->value() == Privacy::HasUsername) {
-				_controls.usernameResult = nullptr;
+				showUsernameEmpty();
 				setFocusUsername();
 			}
-		} else if (type == qstr("USERNAME_INVALID")) {
+		} else if (type == u"USERNAME_INVALID"_q) {
 			showUsernameError(tr::lng_create_channel_link_invalid());
-		} else if (type == qstr("USERNAME_OCCUPIED")
-			&& checking != username) {
+		} else if (type == u"USERNAME_PURCHASE_AVAILABLE"_q) {
+			_goodUsername = false;
+			_usernameCheckInfo.fire({
+				.type = UsernameCheckInfo::Type::PurchaseAvailable,
+			});
+		} else if (type == u"USERNAME_OCCUPIED"_q && checking != username) {
 			showUsernameError(tr::lng_create_channel_link_occupied());
 		}
 	}).send();
@@ -473,18 +601,14 @@ void Controller::askUsernameRevoke() {
 		_controls.privacy->setValue(Privacy::HasUsername);
 		checkUsernameAvailability();
 	});
-	Ui::show(
-		Box<RevokePublicLinkBox>(
-			&_peer->session(),
-			std::move(revokeCallback)),
-		Ui::LayerOption::KeepOther);
+	_show->showBox(Box(PublicLinksLimitBox, _navigation, revokeCallback));
 }
 
 void Controller::usernameChanged() {
-	_isAllowSave = false;
+	_goodUsername = false;
 	const auto username = getUsernameInput();
 	if (username.isEmpty()) {
-		_controls.usernameResult = nullptr;
+		showUsernameEmpty();
 		_checkUsernameTimer.cancel();
 		return;
 	}
@@ -496,46 +620,47 @@ void Controller::usernameChanged() {
 	});
 	if (bad) {
 		showUsernameError(tr::lng_create_channel_link_bad_symbols());
-	} else if (username.size() < kMinUsernameLength) {
+	} else if (username.size() < Ui::EditPeer::kMinUsernameLength) {
 		showUsernameError(tr::lng_create_channel_link_too_short());
 	} else {
-		_controls.usernameResult = nullptr;
-		_checkUsernameTimer.callOnce(kUsernameCheckTimeout);
+		showUsernamePending();
+		_checkUsernameTimer.callOnce(Ui::EditPeer::kUsernameCheckTimeout);
 	}
 }
 
 void Controller::showUsernameError(rpl::producer<QString> &&error) {
-	_isAllowSave = false;
-	showUsernameResult(std::move(error), &st::editPeerUsernameError);
+	_goodUsername = false;
+	_usernameCheckInfoLifetime.destroy();
+	std::move(
+		error
+	) | rpl::map([](QString s) {
+		return UsernameCheckInfo{
+			.type = UsernameCheckInfo::Type::Error,
+			.text = { std::move(s) },
+		};
+	}) | rpl::start_to_stream(_usernameCheckInfo, _usernameCheckInfoLifetime);
 }
 
 void Controller::showUsernameGood() {
-	_isAllowSave = true;
-	showUsernameResult(
-		tr::lng_create_channel_link_available(),
-		&st::editPeerUsernameGood);
+	_goodUsername = true;
+	_usernameCheckInfoLifetime.destroy();
+	_usernameCheckInfo.fire({
+		.type = UsernameCheckInfo::Type::Good,
+		.text = { tr::lng_create_channel_link_available(tr::now) },
+	});
 }
 
-void Controller::showUsernameResult(
-		rpl::producer<QString> &&text,
-		not_null<const style::FlatLabel*> st) {
-	if (!_controls.usernameResult
-		|| _controls.usernameResultStyle != st) {
-		_controls.usernameResultStyle = st;
-		_controls.usernameResult = base::make_unique_q<Ui::FlatLabel>(
-			_controls.usernameWrap,
-			_usernameResultTexts.events() | rpl::flatten_latest(),
-			*st);
-		const auto label = _controls.usernameResult.get();
-		label->show();
-		label->widthValue(
-		) | rpl::start_with_next([label] {
-			label->moveToRight(
-				st::editPeerUsernamePosition.x(),
-				st::editPeerUsernamePosition.y());
-		}, label->lifetime());
-	}
-	_usernameResultTexts.fire(std::move(text));
+void Controller::showUsernamePending() {
+	_usernameCheckInfoLifetime.destroy();
+	_usernameCheckInfo.fire({
+		.type = UsernameCheckInfo::Type::Default,
+		.text = { .text = tr::lng_create_channel_link_pending(tr::now) },
+	});
+}
+
+void Controller::showUsernameEmpty() {
+	_usernameCheckInfoLifetime.destroy();
+	_usernameCheckInfo.fire({ .type = UsernameCheckInfo::Type::Default });
 }
 
 object_ptr<Ui::RpWidget> Controller::createInviteLinkBlock() {
@@ -549,12 +674,13 @@ object_ptr<Ui::RpWidget> Controller::createInviteLinkBlock() {
 	const auto container = result->entity();
 
 	using namespace Settings;
-	if (_privacySavedValue) {
+	if (_dataSavedValue) {
 		AddSkip(container);
 
 		AddSubsectionTitle(container, tr::lng_create_permanent_link_title());
 	}
 	AddPermanentLinkBlock(
+		_show,
 		container,
 		_peer,
 		_peer->session().user(),
@@ -575,24 +701,24 @@ object_ptr<Ui::RpWidget> Controller::createInviteLinkBlock() {
 
 EditPeerTypeBox::EditPeerTypeBox(
 	QWidget*,
+	Window::SessionNavigation *navigation,
 	not_null<PeerData*> peer,
 	bool useLocationPhrases,
-	std::optional<FnMut<void(Privacy, QString)>> savedCallback,
-	std::optional<Privacy> privacySaved,
-	std::optional<QString> usernameSaved,
+	std::optional<FnMut<void(EditPeerTypeData)>> savedCallback,
+	std::optional<EditPeerTypeData> dataSaved,
 	std::optional<rpl::producer<QString>> usernameError)
-: _peer(peer)
+: _navigation(navigation)
+, _peer(peer)
 , _useLocationPhrases(useLocationPhrases)
 , _savedCallback(std::move(savedCallback))
-, _privacySavedValue(privacySaved)
-, _usernameSavedValue(usernameSaved)
+, _dataSavedValue(dataSaved)
 , _usernameError(usernameError) {
 }
 
 EditPeerTypeBox::EditPeerTypeBox(
 	QWidget*,
 	not_null<PeerData*> peer)
-: EditPeerTypeBox(nullptr, peer, {}, {}, {}, {}, {}) {
+: EditPeerTypeBox(nullptr, nullptr, peer, {}, {}, {}) {
 }
 
 void EditPeerTypeBox::setInnerFocus() {
@@ -606,11 +732,16 @@ void EditPeerTypeBox::prepare() {
 
 	const auto controller = Ui::CreateChild<Controller>(
 		this,
+		_navigation,
+		uiShow(),
 		content.data(),
 		_peer,
 		_useLocationPhrases,
-		_privacySavedValue,
-		_usernameSavedValue);
+		_dataSavedValue);
+	controller->scrollToRequests(
+	) | rpl::start_with_next([=, raw = content.data()](int y) {
+		scrollToY(raw->y() + y);
+	}, lifetime());
 	_focusRequests.events(
 	) | rpl::start_with_next(
 		[=] {
@@ -628,16 +759,27 @@ void EditPeerTypeBox::prepare() {
 	if (_savedCallback.has_value()) {
 		addButton(tr::lng_settings_save(), [=] {
 			const auto v = controller->getPrivacy();
-			if (!controller->isAllowSave() && (v == Privacy::HasUsername)) {
-				controller->setFocusUsername();
-				return;
+			if ((v == Privacy::HasUsername) && !controller->goodUsername()) {
+				if (!controller->getUsernameInput().isEmpty()
+					|| controller->usernamesOrder().empty()) {
+					controller->setFocusUsername();
+					return;
+				}
 			}
 
 			auto local = std::move(*_savedCallback);
-			local(v,
-				(v == Privacy::HasUsername)
+			local(EditPeerTypeData{
+				.privacy = v,
+				.username = (v == Privacy::HasUsername
 					? controller->getUsernameInput()
-					: QString()); // We don't need username with private type.
+					: QString()),
+				.usernamesOrder = (v == Privacy::HasUsername
+					? controller->usernamesOrder()
+					: std::vector<QString>()),
+				.noForwards = controller->noForwards(),
+				.joinToWrite = controller->joinToWrite(),
+				.requestToJoin = controller->requestToJoin(),
+			}); // We don't need username with private type.
 			closeBox();
 		});
 		addButton(tr::lng_cancel(), [=] { closeBox(); });
