@@ -11,13 +11,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "calls/group/calls_group_common.h"
 #include "calls/group/calls_choose_join_as.h"
 #include "calls/group/calls_group_call.h"
+#include "calls/group/calls_group_rtmp.h"
 #include "mtproto/mtproto_dh_utils.h"
 #include "core/application.h"
+#include "core/core_settings.h"
 #include "main/main_session.h"
 #include "main/main_account.h"
 #include "apiwrap.h"
 #include "lang/lang_keys.h"
-#include "boxes/confirm_box.h"
+#include "ui/boxes/confirm_box.h"
 #include "calls/group/calls_group_call.h"
 #include "calls/group/calls_group_panel.h"
 #include "calls/calls_call.h"
@@ -31,10 +33,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "platform/platform_specific.h"
 #include "ui/toast/toast.h"
 #include "base/unixtime.h"
-#include "mainwidget.h"
 #include "mtproto/mtproto_config.h"
-#include "boxes/rate_call_box.h"
-#include "app.h" // App::quitting
+#include "boxes/abstract_box.h" // Ui::show().
 
 #include <tgcalls/VideoCaptureInterface.h>
 #include <tgcalls/StaticThreads.h>
@@ -173,7 +173,8 @@ FnMut<void()> Instance::Delegate::groupCallAddAsyncWaiter() {
 Instance::Instance()
 : _delegate(std::make_unique<Delegate>(this))
 , _cachedDhConfig(std::make_unique<DhConfig>())
-, _chooseJoinAs(std::make_unique<Group::ChooseJoinAsProcess>()) {
+, _chooseJoinAs(std::make_unique<Group::ChooseJoinAsProcess>())
+, _startWithRtmp(std::make_unique<Group::StartRtmpProcess>()) {
 }
 
 Instance::~Instance() {
@@ -192,8 +193,10 @@ void Instance::startOutgoingCall(not_null<UserData*> user, bool video) {
 	if (user->callsStatus() == UserData::CallsStatus::Private) {
 		// Request full user once more to refresh the setting in case it was changed.
 		user->session().api().requestFullPeer(user);
-		Ui::show(Box<InformBox>(
-			tr::lng_call_error_not_available(tr::now, lt_user, user->name)));
+		Ui::show(Ui::MakeInformBox(tr::lng_call_error_not_available(
+			tr::now,
+			lt_user,
+			user->name())));
 		return;
 	}
 	requestPermissionsOrFail(crl::guard(this, [=] {
@@ -202,24 +205,89 @@ void Instance::startOutgoingCall(not_null<UserData*> user, bool video) {
 }
 
 void Instance::startOrJoinGroupCall(
+		std::shared_ptr<Ui::Show> show,
 		not_null<PeerData*> peer,
-		const QString &joinHash,
-		bool confirmNeeded) {
-	const auto context = confirmNeeded
-		? Group::ChooseJoinAsProcess::Context::JoinWithConfirm
-		: peer->groupCall()
-		? Group::ChooseJoinAsProcess::Context::Join
-		: Group::ChooseJoinAsProcess::Context::Create;
-	_chooseJoinAs->start(peer, context, [=](object_ptr<Ui::BoxContent> box) {
-		Ui::show(std::move(box), Ui::LayerOption::KeepOther);
-	}, [=](QString text) {
-		Ui::Toast::Show(text);
-	}, [=](Group::JoinInfo info) {
-		const auto call = info.peer->groupCall();
-		info.joinHash = joinHash;
-		createGroupCall(
-			std::move(info),
-			call ? call->input() : MTP_inputGroupCall(MTPlong(), MTPlong()));
+		StartGroupCallArgs args) {
+	confirmLeaveCurrent(show, peer, args, [=](StartGroupCallArgs args) {
+		using JoinConfirm = Calls::StartGroupCallArgs::JoinConfirm;
+		const auto context = (args.confirm == JoinConfirm::Always)
+			? Group::ChooseJoinAsProcess::Context::JoinWithConfirm
+			: peer->groupCall()
+			? Group::ChooseJoinAsProcess::Context::Join
+			: args.scheduleNeeded
+			? Group::ChooseJoinAsProcess::Context::CreateScheduled
+			: Group::ChooseJoinAsProcess::Context::Create;
+		_chooseJoinAs->start(peer, context, show, [=](Group::JoinInfo info) {
+			const auto call = info.peer->groupCall();
+			info.joinHash = args.joinHash;
+			if (call) {
+				info.rtmp = call->rtmp();
+			}
+			createGroupCall(
+				std::move(info),
+				call ? call->input() : MTP_inputGroupCall({}, {}));
+		});
+	});
+}
+
+void Instance::confirmLeaveCurrent(
+		std::shared_ptr<Ui::Show> show,
+		not_null<PeerData*> peer,
+		StartGroupCallArgs args,
+		Fn<void(StartGroupCallArgs)> confirmed) {
+	using JoinConfirm = Calls::StartGroupCallArgs::JoinConfirm;
+
+	auto confirmedArgs = args;
+	confirmedArgs.confirm = JoinConfirm::None;
+
+	const auto askConfirmation = [&](QString text, QString button) {
+		show->showBox(Ui::MakeConfirmBox({
+			.text = text,
+			.confirmed = [=] {
+				show->hideLayer();
+				confirmed(confirmedArgs);
+			},
+			.confirmText = button,
+		}));
+	};
+	if (args.confirm != JoinConfirm::None && inCall()) {
+		// Do you want to leave your active voice chat
+		// to join a voice chat in this group?
+		askConfirmation(
+			(peer->isBroadcast()
+				? tr::lng_call_leave_to_other_sure_channel
+				: tr::lng_call_leave_to_other_sure)(tr::now),
+			tr::lng_call_bar_hangup(tr::now));
+	} else if (args.confirm != JoinConfirm::None && inGroupCall()) {
+		const auto now = currentGroupCall()->peer();
+		if (now == peer) {
+			activateCurrentCall(args.joinHash);
+		} else if (currentGroupCall()->scheduleDate()) {
+			confirmed(confirmedArgs);
+		} else {
+			askConfirmation(
+				((peer->isBroadcast() && now->isBroadcast())
+					? tr::lng_group_call_leave_channel_to_other_sure_channel
+					: now->isBroadcast()
+					? tr::lng_group_call_leave_channel_to_other_sure
+					: peer->isBroadcast()
+					? tr::lng_group_call_leave_to_other_sure_channel
+					: tr::lng_group_call_leave_to_other_sure)(tr::now),
+				tr::lng_group_call_leave(tr::now));
+		}
+	} else {
+		confirmed(args);
+	}
+}
+
+void Instance::showStartWithRtmp(
+		std::shared_ptr<Ui::Show> show,
+		not_null<PeerData*> peer) {
+	_startWithRtmp->start(peer, show, [=](Group::JoinInfo info) {
+		confirmLeaveCurrent(show, peer, {}, [=](auto) {
+			_startWithRtmp->close();
+			createGroupCall(std::move(info), MTP_inputGroupCall({}, {}));
+		});
 	});
 }
 
@@ -249,33 +317,59 @@ void Instance::destroyCall(not_null<Call*> call) {
 		_currentCallChanges.fire(nullptr);
 		taken.reset();
 
-		if (App::quitting()) {
+		if (Core::Quitting()) {
 			LOG(("Calls::Instance doesn't prevent quit any more."));
 		}
 		Core::App().quitPreventFinished();
 	}
 }
 
-void Instance::createCall(not_null<UserData*> user, Call::Type type, bool video) {
-	auto call = std::make_unique<Call>(_delegate.get(), user, type, video);
-	const auto raw = call.get();
+void Instance::createCall(
+		not_null<UserData*> user,
+		Call::Type type,
+		bool isVideo) {
+	struct Performer final {
+		explicit Performer(Fn<void(bool, bool, const Performer &)> callback)
+		: callback(std::move(callback)) {
+		}
+		Fn<void(bool, bool, const Performer &)> callback;
+	};
+	const auto performer = Performer([=](
+			bool video,
+			bool isConfirmed,
+			const Performer &repeater) {
+		const auto delegate = _delegate.get();
+		auto call = std::make_unique<Call>(delegate, user, type, video);
+		if (isConfirmed) {
+			call->applyUserConfirmation();
+		}
+		const auto raw = call.get();
 
-	user->session().account().sessionChanges(
-	) | rpl::start_with_next([=] {
-		destroyCall(raw);
-	}, raw->lifetime());
+		user->session().account().sessionChanges(
+		) | rpl::start_with_next([=] {
+			destroyCall(raw);
+		}, raw->lifetime());
 
-	if (_currentCall) {
-		_currentCallPanel->replaceCall(raw);
-		std::swap(_currentCall, call);
-		call->hangup();
-	} else {
-		_currentCallPanel = std::make_unique<Panel>(raw);
-		_currentCall = std::move(call);
-	}
-	_currentCallChanges.fire_copy(raw);
-	refreshServerConfig(&user->session());
-	refreshDhConfig();
+		if (_currentCall) {
+			_currentCallPanel->replaceCall(raw);
+			std::swap(_currentCall, call);
+			call->hangup();
+		} else {
+			_currentCallPanel = std::make_unique<Panel>(raw);
+			_currentCall = std::move(call);
+		}
+		if (raw->state() == Call::State::WaitingUserConfirmation) {
+			_currentCallPanel->startOutgoingRequests(
+			) | rpl::start_with_next([=](bool video) {
+				repeater.callback(video, true, repeater);
+			}, raw->lifetime());
+		} else {
+			refreshServerConfig(&user->session());
+			refreshDhConfig();
+		}
+		_currentCallChanges.fire_copy(raw);
+	});
+	performer.callback(isVideo, false, performer);
 }
 
 void Instance::destroyGroupCall(not_null<GroupCall*> call) {
@@ -287,7 +381,7 @@ void Instance::destroyGroupCall(not_null<GroupCall*> call) {
 		_currentGroupCallChanges.fire(nullptr);
 		taken.reset();
 
-		if (App::quitting()) {
+		if (Core::Quitting()) {
 			LOG(("Calls::Instance doesn't prevent quit any more."));
 		}
 		Core::App().quitPreventFinished();
@@ -334,7 +428,7 @@ void Instance::refreshDhConfig() {
 		} else {
 			_delegate->callFailed(call);
 		}
-	}).fail([=](const MTP::Error &error) {
+	}).fail([=] {
 		const auto call = weak.get();
 		if (!call) {
 			return;
@@ -393,7 +487,7 @@ void Instance::refreshServerConfig(not_null<Main::Session*> session) {
 
 		const auto &json = result.c_dataJSON().vdata().v;
 		UpdateConfig(std::string(json.data(), json.size()));
-	}).fail([=](const MTP::Error &error) {
+	}).fail([=] {
 		_serverConfigRequestSession = nullptr;
 	}).send();
 }
@@ -462,6 +556,11 @@ FnMut<void()> Instance::addAsyncWaiter() {
 	};
 }
 
+bool Instance::isSharingScreen() const {
+	return (_currentCall && _currentCall->isSharingScreen())
+		|| (_currentGroupCall && _currentGroupCall->isSharingScreen());
+}
+
 bool Instance::isQuitPrevent() {
 	if (!_currentCall || _currentCall->isIncomingWaiting()) {
 		return false;
@@ -484,7 +583,21 @@ void Instance::handleCallUpdate(
 			LOG(("API Error: User not loaded for phoneCallRequested."));
 		} else if (user->isSelf()) {
 			LOG(("API Error: Self found in phoneCallRequested."));
+		} else if (_currentCall
+			&& _currentCall->user() == user
+			&& _currentCall->id() == phoneCall.vid().v) {
+			// May be a repeated phoneCallRequested update from getDifference.
+			return;
 		}
+		if (inCall()
+			&& _currentCall->type() == Call::Type::Outgoing
+			&& _currentCall->user()->id == session->userPeerId()
+			&& (peerFromUser(phoneCall.vparticipant_id())
+				== _currentCall->user()->session().userPeerId())) {
+			// Ignore call from the same running app, other account.
+			return;
+		}
+
 		const auto &config = session->serverConfig();
 		if (inCall() || inGroupCall() || !user || user->isSelf()) {
 			const auto flags = phoneCall.is_video()
@@ -500,8 +613,6 @@ void Instance::handleCallUpdate(
 		} else if (phoneCall.vdate().v + (config.callRingTimeoutMs / 1000)
 			< base::unixtime::now()) {
 			LOG(("Ignoring too old call."));
-		} else if (Core::App().settings().disableCalls()) {
-			LOG(("Ignoring call because of 'accept calls' settings."));
 		} else {
 			createCall(user, Call::Type::Incoming, phoneCall.is_video());
 			_currentCall->handleUpdate(call);
@@ -537,7 +648,7 @@ void Instance::handleGroupCallUpdate(
 		return data.vcall().match([&](const MTPDinputGroupCall &data) {
 			return data.vid().v;
 		});
-	}, [](const auto &) -> uint64 {
+	}, [](const auto &) -> CallId {
 		Unexpected("Type in Instance::handleGroupCallUpdate.");
 	});
 	if (const auto existing = session->data().groupCall(callId)) {
@@ -637,6 +748,17 @@ bool Instance::minimizeCurrentActiveCall() {
 	return false;
 }
 
+bool Instance::toggleFullScreenCurrentActiveCall() {
+	if (inCall() && _currentCallPanel->isActive()) {
+		_currentCallPanel->toggleFullScreen();
+		return true;
+	} else if (inGroupCall() && _currentGroupCallPanel->isActive()) {
+		_currentGroupCallPanel->toggleFullScreen();
+		return true;
+	}
+	return false;
+}
+
 bool Instance::closeCurrentActiveCall() {
 	if (inGroupCall() && _currentGroupCallPanel->isActive()) {
 		_currentGroupCallPanel->close();
@@ -695,10 +817,14 @@ void Instance::requestPermissionOrFail(Platform::PermissionType type, Fn<void()>
 		if (inGroupCall()) {
 			_currentGroupCall->hangup();
 		}
-		Ui::show(Box<ConfirmBox>(tr::lng_no_mic_permission(tr::now), tr::lng_menu_settings(tr::now), crl::guard(this, [=] {
-			Platform::OpenSystemSettingsForPermission(type);
-			Ui::hideLayer();
-		})));
+		Ui::show(Ui::MakeConfirmBox({
+			.text = tr::lng_no_mic_permission(),
+			.confirmed = crl::guard(this, [=](Fn<void()> &&close) {
+				Platform::OpenSystemSettingsForPermission(type);
+				close();
+			}),
+			.confirmText = tr::lng_menu_settings(),
+		}));
 	}
 }
 
