@@ -7,9 +7,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
-#include "ui/widgets/input_fields.h"
+#include "ui/widgets/fields/input_field.h"
 #include "base/timer.h"
-#include "base/qt_connection.h"
+#include "chat_helpers/compose/compose_features.h"
 
 #ifndef TDESKTOP_DISABLE_SPELLCHECK
 #include "boxes/dictionaries_manager.h"
@@ -20,33 +20,39 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 namespace Main {
 class Session;
+class SessionShow;
 } // namespace Main
 
 namespace Window {
 class SessionController;
-enum class GifPauseReason;
 } // namespace Window
 
+namespace ChatHelpers {
+enum class PauseReason;
+class Show;
+} // namespace ChatHelpers
+
 namespace Ui {
 class PopupMenu;
-class Show;
 } // namespace Ui
 
-QString PrepareMentionTag(not_null<UserData*> user);
-TextWithTags PrepareEditText(not_null<HistoryItem*> item);
+[[nodiscard]] QString PrepareMentionTag(not_null<UserData*> user);
+[[nodiscard]] TextWithTags PrepareEditText(not_null<HistoryItem*> item);
+[[nodiscard]] bool EditTextChanged(
+	not_null<HistoryItem*> item,
+	const TextWithTags &updated);
 
 Fn<bool(
 	Ui::InputField::EditLinkSelection selection,
 	QString text,
 	QString link,
 	Ui::InputField::EditLinkAction action)> DefaultEditLinkCallback(
-		std::shared_ptr<Ui::Show> show,
-		not_null<Main::Session*> session,
+		std::shared_ptr<Main::SessionShow> show,
 		not_null<Ui::InputField*> field,
 		const style::InputField *fieldStyle = nullptr);
 void InitMessageFieldHandlers(
 	not_null<Main::Session*> session,
-	std::shared_ptr<Ui::Show> show,
+	std::shared_ptr<Main::SessionShow> show, // may be null
 	not_null<Ui::InputField*> field,
 	Fn<bool()> customEmojiPaused,
 	Fn<bool(not_null<DocumentData*>)> allowPremiumEmoji = nullptr,
@@ -54,16 +60,19 @@ void InitMessageFieldHandlers(
 void InitMessageFieldHandlers(
 	not_null<Window::SessionController*> controller,
 	not_null<Ui::InputField*> field,
-	Window::GifPauseReason pauseReasonLevel,
+	ChatHelpers::PauseReason pauseReasonLevel,
 	Fn<bool(not_null<DocumentData*>)> allowPremiumEmoji = nullptr);
+void InitMessageField(
+	std::shared_ptr<ChatHelpers::Show> show,
+	not_null<Ui::InputField*> field,
+	Fn<bool(not_null<DocumentData*>)> allowPremiumEmoji);
 void InitMessageField(
 	not_null<Window::SessionController*> controller,
 	not_null<Ui::InputField*> field,
 	Fn<bool(not_null<DocumentData*>)> allowPremiumEmoji);
 
 void InitSpellchecker(
-	std::shared_ptr<Ui::Show> show,
-	not_null<Main::Session*> session,
+	std::shared_ptr<Main::SessionShow> show,
 	not_null<Ui::InputField*> field,
 	bool skipDictionariesManager = false);
 
@@ -84,13 +93,15 @@ struct AutocompleteQuery {
 	bool fromStart = false;
 };
 AutocompleteQuery ParseMentionHashtagBotCommandQuery(
-	not_null<const Ui::InputField*> field);
+	not_null<const Ui::InputField*> field,
+	ChatHelpers::ComposeFeatures features);
 
 class MessageLinksParser : private QObject {
 public:
 	MessageLinksParser(not_null<Ui::InputField*> field);
 
 	void parseNow();
+	void setDisabled(bool disabled);
 
 	[[nodiscard]] const rpl::variable<QStringList> &list() const;
 
@@ -118,7 +129,12 @@ private:
 	not_null<Ui::InputField*> _field;
 	rpl::variable<QStringList> _list;
 	int _lastLength = 0;
+	bool _disabled = false;
 	base::Timer _timer;
-	base::qt_connection _connection;
+	rpl::lifetime _lifetime;
 
 };
+
+[[nodiscard]] base::unique_qptr<Ui::RpWidget> CreateDisabledFieldView(
+	QWidget *parent,
+	not_null<PeerData*> peer);
