@@ -20,6 +20,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/launcher.h"
 #include "core/local_url_handlers.h"
 #include "core/update_checker.h"
+#include "core/deadlock_detector.h"
 #include "base/timer.h"
 #include "base/concurrent_timer.h"
 #include "base/invoke_queued.h"
@@ -78,13 +79,9 @@ QString _escapeFrom7bit(const QString &str) {
 
 bool Sandbox::QuitOnStartRequested = false;
 
-Sandbox::Sandbox(
-	not_null<Core::Launcher*> launcher,
-	int &argc,
-	char **argv)
+Sandbox::Sandbox(int &argc, char **argv)
 : QApplication(argc, argv)
-, _mainThreadId(QThread::currentThreadId())
-, _launcher(launcher) {
+, _mainThreadId(QThread::currentThreadId()) {
 	setQuitOnLastWindowClosed(false);
 }
 
@@ -107,7 +104,8 @@ int Sandbox::start() {
 		hashMd5Hex(d.constData(), d.size(), h.data());
 		_lockFile = std::make_unique<QLockFile>(QDir::tempPath() + '/' + h + '-' + cGUIDStr());
 		_lockFile->setStaleLockTime(0);
-		if (!_lockFile->tryLock() && _launcher->customWorkingDir()) {
+		if (!_lockFile->tryLock()
+			&& Launcher::Instance().customWorkingDir()) {
 			// On Windows, QLockFile has problems detecting a stale lock
 			// if the machine's hostname contains characters outside the US-ASCII character set.
 			if constexpr (Platform::IsWindows()) {
@@ -200,7 +198,14 @@ void Sandbox::launchApplication() {
 		}
 		setupScreenScale();
 
-		_application = std::make_unique<Application>(_launcher);
+#ifndef _DEBUG
+		if (Logs::DebugEnabled()) {
+			using DeadlockDetector::PingThread;
+			_deadlockDetector = std::make_unique<PingThread>(this);
+		}
+#endif // !_DEBUG
+
+		_application = std::make_unique<Application>();
 
 		// Ideally this should go to constructor.
 		// But we want to catch all native events and Application installs
@@ -269,6 +274,10 @@ bool Sandbox::event(QEvent *e) {
 		return false;
 	} else if (e->type() == QEvent::Close) {
 		Quit();
+	} else if (e->type() == DeadlockDetector::PingPongEvent::Type()) {
+		postEvent(
+			static_cast<DeadlockDetector::PingPongEvent*>(e)->sender(),
+			new DeadlockDetector::PingPongEvent(this));
 	}
 	return QApplication::event(e);
 }
@@ -401,7 +410,6 @@ void Sandbox::singleInstanceChecked() {
 		}
 		_lastCrashDump = crashdump;
 		auto window = new LastCrashedWindow(
-			_launcher,
 			_lastCrashDump,
 			[=] { launchApplication(); });
 		window->proxyChanges(
@@ -529,14 +537,6 @@ void Sandbox::refreshGlobalProxy() {
 	}
 }
 
-bool Sandbox::customWorkingDir() const {
-	return _launcher->customWorkingDir();
-}
-
-uint64 Sandbox::installationTag() const {
-	return _launcher->installationTag();
-}
-
 void Sandbox::checkForEmptyLoopNestingLevel() {
 	// _loopNestingLevel == _eventNestingLevel means that we had a
 	// native event in a nesting loop that didn't get a notify() call
@@ -592,7 +592,7 @@ void Sandbox::registerEnterFromEventLoop() {
 }
 
 bool Sandbox::notifyOrInvoke(QObject *receiver, QEvent *e) {
-	if (e->type() == base::InvokeQueuedEvent::kType) {
+	if (e->type() == base::InvokeQueuedEvent::Type()) {
 		static_cast<base::InvokeQueuedEvent*>(e)->invoke();
 		return true;
 	}
