@@ -13,6 +13,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_user.h"
 #include "data/data_chat.h"
 #include "data/data_session.h"
+#include "data/data_stories.h"
 #include "data/data_folder.h"
 #include "data/data_forum.h"
 #include "data/data_forum_icons.h"
@@ -530,6 +531,11 @@ bool ChannelData::canBanMembers() const {
 		|| (adminRights() & AdminRight::BanUsers);
 }
 
+bool ChannelData::canPostMessages() const {
+	return amCreator()
+		|| (adminRights() & AdminRight::PostMessages);
+}
+
 bool ChannelData::canEditMessages() const {
 	return amCreator()
 		|| (adminRights() & AdminRight::EditMessages);
@@ -540,6 +546,30 @@ bool ChannelData::canDeleteMessages() const {
 		|| (adminRights() & AdminRight::DeleteMessages);
 }
 
+bool ChannelData::canPostStories() const {
+	if (!isBroadcast()) {
+		return false;
+	}
+	return amCreator()
+		|| (adminRights() & AdminRight::PostStories);
+}
+
+bool ChannelData::canEditStories() const {
+	if (!isBroadcast()) {
+		return false;
+	}
+	return amCreator()
+		|| (adminRights() & AdminRight::EditStories);
+}
+
+bool ChannelData::canDeleteStories() const {
+	if (!isBroadcast()) {
+		return false;
+	}
+	return amCreator()
+		|| (adminRights() & AdminRight::DeleteStories);
+}
+
 bool ChannelData::anyoneCanAddMembers() const {
 	return !(defaultRestrictions() & Restriction::AddParticipants);
 }
@@ -559,11 +589,6 @@ bool ChannelData::canAddAdmins() const {
 		|| (adminRights() & AdminRight::AddAdmins);
 }
 
-bool ChannelData::canPublish() const {
-	return amCreator()
-		|| (adminRights() & AdminRight::PostMessages);
-}
-
 bool ChannelData::allowsForwarding() const {
 	return !(flags() & Flag::NoForwards);
 }
@@ -877,6 +902,38 @@ const Data::AllowedReactions &ChannelData::allowedReactions() const {
 	return _allowedReactions;
 }
 
+bool ChannelData::hasActiveStories() const {
+	return flags() & Flag::HasActiveStories;
+}
+
+bool ChannelData::hasUnreadStories() const {
+	return flags() & Flag::HasUnreadStories;
+}
+
+void ChannelData::setStoriesState(StoriesState state) {
+	Expects(state != StoriesState::Unknown);
+
+	const auto was = flags();
+	switch (state) {
+	case StoriesState::None:
+		_flags.remove(Flag::HasActiveStories | Flag::HasUnreadStories);
+		break;
+	case StoriesState::HasRead:
+		_flags.set(
+			(flags() & ~Flag::HasUnreadStories) | Flag::HasActiveStories);
+		break;
+	case StoriesState::HasUnread:
+		_flags.add(Flag::HasActiveStories | Flag::HasUnreadStories);
+		break;
+	}
+	if (flags() != was) {
+		if (const auto history = owner().historyLoaded(this)) {
+			history->updateChatListEntryPostponed();
+		}
+		session().changes().peerUpdated(this, UpdateFlag::StoriesState);
+	}
+}
+
 void ChannelData::processTopics(const MTPVector<MTPForumTopic> &topics) {
 	if (const auto forum = this->forum()) {
 		forum->applyReceivedTopics(topics);
@@ -1046,6 +1103,7 @@ void ApplyChannelUpdate(
 	} else {
 		channel->setAllowedReactions({});
 	}
+	channel->owner().stories().apply(channel, update.vstories());
 	channel->fullUpdated();
 	channel->setPendingRequestsCount(
 		update.vrequests_pending().value_or_empty(),
