@@ -23,7 +23,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "storage/storage_account.h"
 #include "history/history.h"
 #include "history/history_item.h"
-#include "history/history_message.h"
+#include "history/history_item_helpers.h"
 #include "core/application.h"
 #include "core/mime_type.h"
 #include "ui/controls/download_bar.h"
@@ -64,26 +64,43 @@ constexpr auto ByDocument = [](const auto &entry) {
 	return 0;
 }
 
-[[nodiscard]] PhotoData *ItemPhoto(not_null<HistoryItem*> item) {
-	if (const auto media = item->media()) {
-		if (const auto page = media->webpage()) {
-			return page->document ? nullptr : page->photo;
-		} else if (const auto photo = media->photo()) {
-			return photo;
+[[nodiscard]] bool ItemContainsMedia(const DownloadObject &object) {
+	if (const auto photo = object.photo) {
+		if (const auto media = object.item->media()) {
+			if (const auto page = media->webpage()) {
+				if (page->photo == photo) {
+					return true;
+				}
+				for (const auto &item : page->collage.items) {
+					if (const auto v = std::get_if<PhotoData*>(&item)) {
+						if ((*v) == photo) {
+							return true;
+						}
+					}
+				}
+			} else {
+				return (media->photo() == photo);
+			}
+		}
+	} else if (const auto document = object.document) {
+		if (const auto media = object.item->media()) {
+			if (const auto page = media->webpage()) {
+				if (page->document == document) {
+					return true;
+				}
+				for (const auto &item : page->collage.items) {
+					if (const auto v = std::get_if<DocumentData*>(&item)) {
+						if ((*v) == document) {
+							return true;
+						}
+					}
+				}
+			} else {
+				return (media->document() == document);
+			}
 		}
 	}
-	return nullptr;
-}
-
-[[nodiscard]] DocumentData *ItemDocument(not_null<HistoryItem*> item) {
-	if (const auto media = item->media()) {
-		if (const auto page = media->webpage()) {
-			return page->document;
-		} else if (const auto document = media->document()) {
-			return document;
-		}
-	}
-	return nullptr;
+	return false;
 }
 
 struct DocumentDescriptor {
@@ -242,12 +259,12 @@ void DownloadManager::check(
 		std::vector<DownloadingId>::iterator i) {
 	auto &entry = *i;
 
-	const auto photo = ItemPhoto(entry.object.item);
-	const auto document = ItemDocument(entry.object.item);
-	if (entry.object.photo != photo || entry.object.document != document) {
+	if (!ItemContainsMedia(entry.object)) {
 		cancel(data, i);
 		return;
 	}
+	const auto document = entry.object.document;
+
 	// Load with progress only documents for now.
 	Assert(document != nullptr);
 
@@ -508,9 +525,13 @@ HistoryItem *DownloadManager::lookupLoadingItem(
 void DownloadManager::loadingStopWithConfirmation(
 		Fn<void()> callback,
 		Main::Session *onlyInSession) {
-	const auto window = Core::App().primaryWindow();
 	const auto item = lookupLoadingItem(onlyInSession);
-	if (!window || !item) {
+	if (!item) {
+		return;
+	}
+	const auto window = Core::App().windowFor(
+		&item->history()->session().account());
+	if (!window) {
 		return;
 	}
 	const auto weak = base::make_weak(&item->history()->session());
@@ -540,7 +561,7 @@ void DownloadManager::loadingStopWithConfirmation(
 			if (const auto strong = weak.get()) {
 				if (const auto item = strong->data().message(id)) {
 					if (const auto window = strong->tryResolveWindow()) {
-						window->showPeerHistoryAtItem(item);
+						window->showMessage(item);
 					}
 				}
 			}
@@ -790,11 +811,14 @@ void DownloadManager::cancel(
 		SessionData &data,
 		std::vector<DownloadingId>::iterator i) {
 	const auto object = i->object;
+	const auto item = object.item;
 	remove(data, i);
-	if (const auto document = object.document) {
-		document->cancel();
-	} else if (const auto photo = object.photo) {
-		photo->cancel();
+	if (!item->isAdminLogEntry()) {
+		if (const auto document = object.document) {
+			document->cancel();
+		} else if (const auto photo = object.photo) {
+			photo->cancel();
+		}
 	}
 }
 
@@ -861,7 +885,7 @@ not_null<HistoryItem*> DownloadManager::generateItem(
 		? previousItem->history()
 		: session->data().history(session->user());
 	const auto flags = MessageFlag::FakeHistoryItem;
-	const auto replyTo = MsgId();
+	const auto replyTo = FullReplyTo();
 	const auto viaBotId = UserId();
 	const auto date = base::unixtime::now();
 	const auto postAuthor = QString();
@@ -930,7 +954,7 @@ void DownloadManager::writePostponed(not_null<Main::Session*> session) {
 
 Fn<std::optional<QByteArray>()> DownloadManager::serializator(
 		not_null<Main::Session*> session) const {
-	return [this, weak = base::make_weak(session.get())]()
+	return [this, weak = base::make_weak(session)]()
 		-> std::optional<QByteArray> {
 		const auto strong = weak.get();
 		if (!strong) {
