@@ -11,8 +11,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_channel.h"
 #include "data/data_chat.h"
 #include "data/data_folder.h"
+#include "data/data_forum.h"
+#include "data/data_forum_topic.h"
 #include "data/data_scheduled_messages.h"
+#include "data/data_user.h"
 #include "base/unixtime.h"
+#include "base/random.h"
 #include "main/main_session.h"
 #include "window/notifications_manager.h"
 #include "history/history.h"
@@ -28,6 +32,29 @@ constexpr auto kReadRequestTimeout = 3 * crl::time(1000);
 
 } // namespace
 
+MTPInputReplyTo ReplyToForMTP(
+		not_null<Session*> owner,
+		FullReplyTo replyTo) {
+	if (replyTo.storyId) {
+		if (const auto peer = owner->peerLoaded(replyTo.storyId.peer)) {
+			if (const auto user = peer->asUser()) {
+				return MTP_inputReplyToStory(
+					user->inputUser,
+					MTP_int(replyTo.storyId.story));
+			}
+		}
+	} else if (replyTo.msgId || replyTo.topicRootId) {
+		using Flag = MTPDinputReplyToMessage::Flag;
+		return MTP_inputReplyToMessage(
+			(replyTo.topicRootId
+				? MTP_flags(Flag::f_top_msg_id)
+				: MTP_flags(0)),
+			MTP_int(replyTo.msgId ? replyTo.msgId : replyTo.topicRootId),
+			MTP_int(replyTo.topicRootId));
+	}
+	return MTPInputReplyTo();
+}
+
 Histories::Histories(not_null<Session*> owner)
 : _owner(owner)
 , _readRequestsTimer([=] { sendReadRequests(); }) {
@@ -244,7 +271,7 @@ void Histories::readInboxOnNewMessage(not_null<HistoryItem*> item) {
 }
 
 void Histories::readClientSideMessage(not_null<HistoryItem*> item) {
-	if (item->out() || !item->unread()) {
+	if (item->out() || !item->unread(item->history())) {
 		return;
 	}
 	const auto history = item->history();
@@ -442,7 +469,8 @@ void Histories::requestFakeChatListMessage(
 void Histories::requestGroupAround(not_null<HistoryItem*> item) {
 	const auto history = item->history();
 	const auto id = item->id;
-	const auto i = _chatListGroupRequests.find(history);
+	const auto key = GroupRequestKey{ history, item->topicRootId() };
+	const auto i = _chatListGroupRequests.find(key);
 	if (i != end(_chatListGroupRequests)) {
 		if (i->second.aroundId == id) {
 			return;
@@ -467,18 +495,18 @@ void Histories::requestGroupAround(not_null<HistoryItem*> item) {
 			_owner->processExistingMessages(
 				history->peer->asChannel(),
 				result);
-			_chatListGroupRequests.remove(history);
+			_chatListGroupRequests.remove(key);
 			history->migrateToOrMe()->applyChatListGroup(
 				history->peer->id,
 				result);
 			finish();
 		}).fail([=] {
-			_chatListGroupRequests.remove(history);
+			_chatListGroupRequests.remove(key);
 			finish();
 		}).send();
 	});
 	_chatListGroupRequests.emplace(
-		history,
+		key,
 		ChatListGroupRequest{ .aroundId = id, .requestId = requestId });
 }
 
@@ -636,7 +664,7 @@ void Histories::deleteAllMessages(
 			)).done([=](const MTPUpdates &result) {
 				session().api().applyUpdates(result);
 			//}).fail([=](const MTP::Error &error) {
-			//	if (error.type() == qstr("CHANNEL_TOO_LARGE")) {
+			//	if (error.type() == u"CHANNEL_TOO_LARGE"_q) {
 			//		Ui::show(Box<Ui::InformBox>(tr::lng_cant_delete_channel(tr::now)));
 			//	}
 			}).send();
@@ -841,6 +869,153 @@ int Histories::sendRequest(
 	return id;
 }
 
+void Histories::sendCreateTopicRequest(
+		not_null<History*> history,
+		MsgId rootId) {
+	Expects(history->peer->isChannel());
+
+	const auto forum = history->asForum();
+	Assert(forum != nullptr);
+	const auto topic = forum->topicFor(rootId);
+	Assert(topic != nullptr);
+	const auto randomId = base::RandomValue<uint64>();
+	session().data().registerMessageRandomId(
+		randomId,
+		{ history->peer->id, rootId });
+	const auto api = &session().api();
+	using Flag = MTPchannels_CreateForumTopic::Flag;
+	api->request(MTPchannels_CreateForumTopic(
+		MTP_flags(Flag::f_icon_color
+			| (topic->iconId() ? Flag::f_icon_emoji_id : Flag(0))),
+		history->peer->asChannel()->inputChannel,
+		MTP_string(topic->title()),
+		MTP_int(topic->colorId()),
+		MTP_long(topic->iconId()),
+		MTP_long(randomId),
+		MTPInputPeer() // send_as
+	)).done([=](const MTPUpdates &result) {
+		api->applyUpdates(result, randomId);
+	}).fail([=](const MTP::Error &error) {
+		api->sendMessageFail(error, history->peer, randomId);
+	}).send();
+}
+
+bool Histories::isCreatingTopic(
+		not_null<History*> history,
+		MsgId rootId) const {
+	const auto forum = history->asForum();
+	return forum && forum->creating(rootId);
+}
+
+int Histories::sendPreparedMessage(
+		not_null<History*> history,
+		FullReplyTo replyTo,
+		uint64 randomId,
+		Fn<PreparedMessage(not_null<Session*>, FullReplyTo)> message,
+		Fn<void(const MTPUpdates&, const MTP::Response&)> done,
+		Fn<void(const MTP::Error&, const MTP::Response&)> fail) {
+	if (isCreatingTopic(history, replyTo.topicRootId)) {
+		const auto id = ++_requestAutoincrement;
+		const auto creatingId = FullMsgId(
+			history->peer->id,
+			replyTo.topicRootId);
+		auto i = _creatingTopics.find(creatingId);
+		if (i == end(_creatingTopics)) {
+			sendCreateTopicRequest(history, replyTo.topicRootId);
+			i = _creatingTopics.emplace(creatingId).first;
+		}
+		i->second.push_back({
+			.randomId = randomId,
+			.replyTo = replyTo.msgId,
+			.message = std::move(message),
+			.done = std::move(done),
+			.fail = std::move(fail),
+			.requestId = id,
+		});
+		_creatingTopicRequests.emplace(id);
+		return id;
+	}
+	const auto realReplyTo = FullReplyTo{
+		.msgId = convertTopicReplyToId(history, replyTo.msgId),
+		.topicRootId = convertTopicReplyToId(history, replyTo.topicRootId),
+		.storyId = replyTo.storyId,
+	};
+	return v::match(message(_owner, realReplyTo), [&](const auto &request) {
+		const auto type = RequestType::Send;
+		return sendRequest(history, type, [=](Fn<void()> finish) {
+			const auto session = &_owner->session();
+			const auto api = &session->api();
+			history->sendRequestId = api->request(
+				base::duplicate(request)
+			).done([=](
+					const MTPUpdates &result,
+					const MTP::Response &response) {
+				api->applyUpdates(result, randomId);
+				done(result, response);
+				finish();
+			}).fail([=](
+					const MTP::Error &error,
+					const MTP::Response &response) {
+				fail(error, response);
+				finish();
+			}).afterRequest(
+				history->sendRequestId
+			).send();
+			return history->sendRequestId;
+		});
+	});
+}
+
+void Histories::checkTopicCreated(FullMsgId rootId, MsgId realRoot) {
+	const auto i = _creatingTopics.find(rootId);
+	if (i != end(_creatingTopics)) {
+		auto scheduled = base::take(i->second);
+		_creatingTopics.erase(i);
+
+		_createdTopicIds.emplace(rootId, realRoot);
+
+		const auto history = _owner->history(rootId.peer);
+		if (const auto forum = history->asForum()) {
+			forum->created(rootId.msg, realRoot);
+		}
+
+		for (auto &entry : scheduled) {
+			_creatingTopicRequests.erase(entry.requestId);
+			sendPreparedMessage(
+				history,
+				FullReplyTo{
+					.msgId = entry.replyTo,
+					.topicRootId = realRoot,
+				},
+				entry.randomId,
+				std::move(entry.message),
+				std::move(entry.done),
+				std::move(entry.fail));
+		}
+		for (const auto &item : history->clientSideMessages()) {
+			const auto replace = [&](MsgId nowId) {
+				return (nowId == rootId.msg) ? realRoot : nowId;
+			};
+			if (item->topicRootId() == rootId.msg) {
+				item->setReplyFields(
+					replace(item->replyToId()),
+					realRoot,
+					true);
+			}
+		}
+	}
+}
+
+MsgId Histories::convertTopicReplyToId(
+		not_null<History*> history,
+		MsgId replyToId) const {
+	if (!replyToId) {
+		return {};
+	}
+	const auto i = _createdTopicIds.find({ history->peer->id, replyToId });
+	return (i != end(_createdTopicIds)) ? i->second : replyToId;
+}
+
 void Histories::checkPostponed(not_null<History*> history, int id) {
 	if (const auto state = lookup(history)) {
 		finishSentRequest(history, state, id);
@@ -850,6 +1025,9 @@ void Histories::checkPostponed(not_null<History*> history, int id) {
 void Histories::cancelRequest(int id) {
 	if (!id) {
 		return;
+	} else if (_creatingTopicRequests.contains(id)) {
+		cancelDelayedByTopicRequest(id);
+		return;
 	}
 	const auto history = _historyByRequest.take(id);
 	if (!history) {
@@ -863,6 +1041,15 @@ void Histories::cancelRequest(int id) {
 	finishSentRequest(*history, state, id);
 }
 
+void Histories::cancelDelayedByTopicRequest(int id) {
+	for (auto &[rootId, messages] : _creatingTopics) {
+		messages.erase(
+			ranges::remove(messages, id, &DelayedByTopicMessage::requestId),
+			end(messages));
+	}
+	_creatingTopicRequests.remove(id);
+}
+
 void Histories::finishSentRequest(
 		not_null<History*> history,
 		not_null<State*> state,
