@@ -22,7 +22,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lottie/lottie_frame_generator.h"
 #include "ffmpeg/ffmpeg_frame_generator.h"
 #include "chat_helpers/stickers_lottie.h"
-#include "ui/widgets/input_fields.h"
+#include "storage/file_download.h" // kMaxFileInMemory
+#include "ui/widgets/fields/input_field.h"
 #include "ui/text/text_custom_emoji.h"
 #include "ui/text/text_utilities.h"
 #include "ui/ui_utility.h"
@@ -344,7 +345,12 @@ void CustomEmojiLoader::check() {
 			sizeOverride);
 	};
 	auto put = [=, key = cacheKey(document)](QByteArray value) {
-		document->owner().cacheBigFile().put(key, std::move(value));
+		const auto size = value.size();
+		if (size <= Storage::kMaxFileInMemory) {
+			document->owner().cacheBigFile().put(key, std::move(value));
+		} else {
+			LOG(("Data Error: Cached emoji size too big: %1.").arg(size));
+		}
 	};
 	const auto type = document->sticker()->type;
 	auto generator = [=, bytes = Lottie::ReadContent(data, filepath)]()
