@@ -71,7 +71,11 @@ QByteArray SerializeString(const QByteArray &value) {
 
 QByteArray SerializeDate(TimeId date) {
 	return SerializeString(
-		QDateTime::fromTime_t(date).toString(Qt::ISODate).toUtf8());
+		QDateTime::fromSecsSinceEpoch(date).toString(Qt::ISODate).toUtf8());
+}
+
+QByteArray SerializeDateRaw(TimeId date) {
+	return SerializeString(QString::number(date).toUtf8());
 }
 
 QByteArray StringAllowEmpty(const Data::Utf8String &data) {
@@ -141,19 +145,20 @@ QByteArray SerializeArray(
 
 QByteArray SerializeText(
 		Context &context,
-		const std::vector<Data::TextPart> &data) {
+		const std::vector<Data::TextPart> &data,
+		bool serializeToObjects = false) {
 	using Type = Data::TextPart::Type;
 
 	if (data.empty()) {
-		return SerializeString("");
+		return serializeToObjects ? QByteArray("[]") : SerializeString("");
 	}
 
 	context.nesting.push_back(Context::kArray);
 
-	const auto text = ranges::view::all(
+	const auto text = ranges::views::all(
 		data
-	) | ranges::view::transform([&](const Data::TextPart &part) {
-		if (part.type == Type::Text) {
+	) | ranges::views::transform([&](const Data::TextPart &part) {
+		if ((part.type == Type::Text) && !serializeToObjects) {
 			return SerializeString(part.text);
 		}
 		const auto typeString = [&] {
@@ -168,6 +173,7 @@ QByteArray SerializeText(
 			case Type::Italic: return "italic";
 			case Type::Code: return "code";
 			case Type::Pre: return "pre";
+			case Type::Text: return "plain";
 			case Type::TextUrl: return "text_link";
 			case Type::MentionName: return "mention_name";
 			case Type::Phone: return "phone";
@@ -176,11 +182,15 @@ QByteArray SerializeText(
 			case Type::Strike: return "strikethrough";
 			case Type::Blockquote: return "blockquote";
 			case Type::BankCard: return "bank_card";
+			case Type::Spoiler: return "spoiler";
+			case Type::CustomEmoji: return "custom_emoji";
 			}
 			Unexpected("Type in SerializeText.");
 		}();
 		const auto additionalName = (part.type == Type::MentionName)
 			? "user_id"
+			: (part.type == Type::CustomEmoji)
+			? "document_id"
 			: (part.type == Type::Pre)
 			? "language"
 			: (part.type == Type::TextUrl)
@@ -188,7 +198,9 @@ QByteArray SerializeText(
 			: "none";
 		const auto additionalValue = (part.type == Type::MentionName)
 			? part.additional
-			: (part.type == Type::Pre || part.type == Type::TextUrl)
+			: (part.type == Type::Pre
+				|| part.type == Type::TextUrl
+				|| part.type == Type::CustomEmoji)
 			? SerializeString(part.additional)
 			: QByteArray();
 		return SerializeObject(context, {
@@ -200,8 +212,10 @@ QByteArray SerializeText(
 
 	context.nesting.pop_back();
 
-	if (data.size() == 1 && data[0].type == Data::TextPart::Type::Text) {
-		return text[0];
+	if (!serializeToObjects) {
+		if (data.size() == 1 && data[0].type == Data::TextPart::Type::Text) {
+			return text[0];
+		}
 	}
 	return SerializeArray(context, text);
 }
@@ -217,11 +231,11 @@ QByteArray FormatFilePath(const Data::File &file) {
 QByteArray SerializeMessage(
 		Context &context,
 		const Data::Message &message,
-		const std::map<Data::PeerId, Data::Peer> &peers,
+		const std::map<PeerId, Data::Peer> &peers,
 		const QString &internalLinksDomain) {
 	using namespace Data;
 
-	if (message.media.content.is<UnsupportedMedia>()) {
+	if (v::is<UnsupportedMedia>(message.media.content)) {
 		return SerializeObject(context, {
 			{ "id", Data::NumberToString(message.id) },
 			{ "type", SerializeString("unsupported") }
@@ -235,28 +249,24 @@ QByteArray SerializeMessage(
 		static auto empty = Peer{ User() };
 		return empty;
 	};
-	const auto user = [&](int32 userId) -> const User& {
-		if (const auto result = peer(UserPeerId(userId)).user()) {
+	const auto user = [&](UserId userId) -> const User& {
+		if (const auto result = peer(userId).user()) {
 			return *result;
 		}
 		static auto empty = User();
 		return empty;
 	};
-	const auto chat = [&](int32 chatId) -> const Chat& {
-		if (const auto result = peer(ChatPeerId(chatId)).chat()) {
-			return *result;
-		}
-		static auto empty = Chat();
-		return empty;
-	};
 
 	auto values = std::vector<std::pair<QByteArray, QByteArray>>{
 	{ "id", NumberToString(message.id) },
 	{
 		"type",
-		SerializeString(message.action.content ? "service" : "message")
+		SerializeString(!v::is_null(message.action.content)
+			? "service"
+			: "message")
 	},
 	{ "date", SerializeDate(message.date) },
+	{ "date_unixtime", SerializeDateRaw(message.date) },
 	};
 	context.nesting.push_back(Context::kObject);
 	const auto serialized = [&] {
@@ -273,11 +283,31 @@ QByteArray SerializeMessage(
 	};
 	if (message.edited) {
 		pushBare("edited", SerializeDate(message.edited));
+		pushBare("edited_unixtime", SerializeDateRaw(message.edited));
 	}
 
 	const auto push = [&](const QByteArray &key, const auto &value) {
 		if constexpr (std::is_arithmetic_v<std::decay_t<decltype(value)>>) {
 			pushBare(key, Data::NumberToString(value));
+		} else if constexpr (std::is_same_v<
+				std::decay_t<decltype(value)>,
+				PeerId>) {
+			if (const auto chat = peerToChat(value)) {
+				pushBare(
+					key,
+					SerializeString("chat"
+						+ Data::NumberToString(chat.bare)));
+			} else if (const auto channel = peerToChannel(value)) {
+				pushBare(
+					key,
+					SerializeString("channel"
+						+ Data::NumberToString(channel.bare)));
+			} else {
+				pushBare(
+					key,
+					SerializeString("user"
+						+ Data::NumberToString(peerToUser(value).bare)));
+			}
 		} else {
 			const auto wrapped = QByteArray(value);
 			if (!wrapped.isEmpty()) {
@@ -288,26 +318,29 @@ QByteArray SerializeMessage(
 	const auto wrapPeerName = [&](PeerId peerId) {
 		return StringAllowNull(peer(peerId).name());
 	};
-	const auto wrapUserName = [&](int32 userId) {
+	const auto wrapUserName = [&](UserId userId) {
 		return StringAllowNull(user(userId).name());
 	};
 	const auto pushFrom = [&](const QByteArray &label = "from") {
 		if (message.fromId) {
-			pushBare(label, wrapUserName(message.fromId));
-			pushBare(label+"_id", Data::NumberToString(message.fromId));
+			pushBare(label, wrapPeerName(message.fromId));
+			push(label + "_id", message.fromId);
 		}
 	};
 	const auto pushReplyToMsgId = [&](
 			const QByteArray &label = "reply_to_message_id") {
 		if (message.replyToMsgId) {
 			push(label, message.replyToMsgId);
+			if (message.replyToPeerId) {
+				push("reply_to_peer_id", message.replyToPeerId);
+			}
 		}
 	};
 	const auto pushUserNames = [&](
-			const std::vector<int32> &data,
+			const std::vector<UserId> &data,
 			const QByteArray &label = "members") {
 		auto list = std::vector<QByteArray>();
-		for (const auto userId : data) {
+		for (const auto &userId : data) {
 			list.push_back(wrapUserName(userId));
 		}
 		pushBare(label, SerializeArray(context, list));
@@ -357,7 +390,7 @@ QByteArray SerializeMessage(
 		}
 	};
 
-	message.action.content.match([&](const ActionChatCreate &data) {
+	v::match(message.action.content, [&](const ActionChatCreate &data) {
 		pushActor();
 		pushAction("create_group");
 		push("title", data.title);
@@ -412,7 +445,13 @@ QByteArray SerializeMessage(
 		pushAction("send_payment");
 		push("amount", data.amount);
 		push("currency", data.currency);
+		const auto amount = FormatMoneyAmount(data.amount, data.currency);
 		pushReplyToMsgId("invoice_message_id");
+		if (data.recurringUsed) {
+			push("recurring", "used");
+		} else if (data.recurringInit) {
+			push("recurring", "init");
+		}
 	}, [&](const ActionPhoneCall &data) {
 		pushActor();
 		pushAction("phone_call");
@@ -436,8 +475,18 @@ QByteArray SerializeMessage(
 		pushActor();
 		push("information_text", data.message);
 	}, [&](const ActionBotAllowed &data) {
-		pushAction("allow_sending_messages");
-		push("reason_domain", data.domain);
+		if (data.attachMenu) {
+			pushAction("attach_menu_bot_allowed");
+		} else if (data.fromRequest) {
+			pushAction("web_app_bot_allowed");
+		} else if (data.appId) {
+			pushAction("allow_sending_messages");
+			push("reason_app_id", data.appId);
+			push("reason_app_name", data.app);
+		} else {
+			pushAction("allow_sending_messages");
+			push("reason_domain", data.domain);
+		}
 	}, [&](const ActionSecureValuesSent &data) {
 		pushAction("send_passport_values");
 		auto list = std::vector<QByteArray>();
@@ -468,12 +517,91 @@ QByteArray SerializeMessage(
 	}, [&](const ActionContactSignUp &data) {
 		pushActor();
 		pushAction("joined_telegram");
+	}, [&](const ActionGeoProximityReached &data) {
+		pushAction("proximity_reached");
+		if (data.fromId) {
+			pushBare("from", wrapPeerName(data.fromId));
+			push("from_id", data.fromId);
+		}
+		if (data.toId) {
+			pushBare("to", wrapPeerName(data.toId));
+			push("to_id", data.toId);
+		}
+		push("distance", data.distance);
 	}, [&](const ActionPhoneNumberRequest &data) {
 		pushActor();
 		pushAction("requested_phone_number");
-	}, [](std::nullopt_t) {});
+	}, [&](const ActionGroupCall &data) {
+		pushActor();
+		pushAction("group_call");
+		if (data.duration) {
+			push("duration", data.duration);
+		}
+	}, [&](const ActionInviteToGroupCall &data) {
+		pushActor();
+		pushAction("invite_to_group_call");
+		pushUserNames(data.userIds);
+	}, [&](const ActionSetMessagesTTL &data) {
+		pushActor();
+		pushAction("set_messages_ttl");
+		push("period", data.period);
+	}, [&](const ActionGroupCallScheduled &data) {
+		pushActor();
+		pushAction("group_call_scheduled");
+		push("schedule_date", data.date);
+	}, [&](const ActionSetChatTheme &data) {
+		pushActor();
+		pushAction("edit_chat_theme");
+		if (!data.emoji.isEmpty()) {
+			push("emoticon", data.emoji.toUtf8());
+		}
+	}, [&](const ActionChatJoinedByRequest &data) {
+		pushActor();
+		pushAction("join_group_by_request");
+	}, [&](const ActionWebViewDataSent &data) {
+		pushAction("send_webview_data");
+		push("text", data.text);
+	}, [&](const ActionGiftPremium &data) {
+		pushActor();
+		pushAction("send_premium_gift");
+		if (!data.cost.isEmpty()) {
+			push("cost", data.cost);
+		}
+		if (data.months) {
+			push("months", data.months);
+		}
+	}, [&](const ActionTopicCreate &data) {
+		pushActor();
+		pushAction("topic_created");
+		push("title", data.title);
+	}, [&](const ActionTopicEdit &data) {
+		pushActor();
+		pushAction("topic_edit");
+		if (!data.title.isEmpty()) {
+			push("new_title", data.title);
+		}
+		if (data.iconEmojiId) {
+			push("new_icon_emoji_id", *data.iconEmojiId);
+		}
+	}, [&](const ActionSuggestProfilePhoto &data) {
+		pushActor();
+		pushAction("suggest_profile_photo");
+		pushPhoto(data.photo.image);
+	}, [&](const ActionRequestedPeer &data) {
+		pushActor();
+		pushAction("requested_peer");
+		push("button_id", data.buttonId);
+		push("peer_id", data.peerId.value);
+	}, [&](const ActionSetChatWallPaper &data) {
+		pushActor();
+		pushAction("set_chat_wallpaper");
+	}, [&](const ActionSetSameChatWallPaper &data) {
+		pushActor();
+		pushAction("set_same_chat_wallpaper");
+		pushReplyToMsgId("message_id");
+	}, [](v::null_t) {});
 
-	if (!message.action.content) {
+	if (v::is_null(message.action.content)) {
 		pushFrom();
 		push("author", message.signature);
 		if (message.forwardedFromId) {
@@ -498,7 +626,7 @@ QByteArray SerializeMessage(
 		}
 	}
 
-	message.media.content.match([&](const Photo &photo) {
+	v::match(message.media.content, [&](const Photo &photo) {
 		pushPhoto(photo.image);
 		pushTTL();
 	}, [&](const Document &data) {
@@ -589,9 +717,9 @@ QByteArray SerializeMessage(
 		}));
 	}, [&](const Poll &data) {
 		context.nesting.push_back(Context::kObject);
-		const auto answers = ranges::view::all(
+		const auto answers = ranges::views::all(
 			data.answers
-		) | ranges::view::transform([&](const Poll::Answer &answer) {
+		) | ranges::views::transform([&](const Poll::Answer &answer) {
 			context.nesting.push_back(Context::kArray);
 			auto result = SerializeObject(context, {
 				{ "text", SerializeString(answer.text) },
@@ -612,9 +740,10 @@ QByteArray SerializeMessage(
 		}));
 	}, [](const UnsupportedMedia &data) {
 		Unexpected("Unsupported message.");
-	}, [](std::nullopt_t) {});
+	}, [](v::null_t) {});
 
 	pushBare("text", SerializeText(context, message.text));
+	pushBare("text_entities", SerializeText(context, message.text, true));
 
 	return serialized();
 }
@@ -682,7 +811,7 @@ Result JsonWriter::writePersonal(const Data::PersonalInfo &data) {
 	return _output->writeBlock(
 		prepareObjectItemStart("personal_information")
 		+ SerializeObject(_context, {
-		{ "user_id", Data::NumberToString(data.user.id) },
+		{ "user_id", Data::NumberToString(data.user.bareId) },
 		{ "first_name", SerializeString(info.firstName) },
 		{ "last_name", SerializeString(info.lastName) },
 		{
@@ -741,6 +870,10 @@ Result JsonWriter::writeUserpicsSlice(const Data::UserpicsSlice &data) {
 				"date",
 				userpic.date ? SerializeDate(userpic.date) : QByteArray()
 			},
+			{
+				"date_unixtime",
+				userpic.date ? SerializeDateRaw(userpic.date) : QByteArray()
+			},
 			{
 				"photo",
 				SerializeString(path)
@@ -756,6 +889,77 @@ Result JsonWriter::writeUserpicsEnd() {
 	return _output->writeBlock(popNesting());
 }
 
+Result JsonWriter::writeStoriesStart(const Data::StoriesInfo &data) {
+	Expects(_output != nullptr);
+
+	auto block = prepareObjectItemStart("stories");
+	return _output->writeBlock(block + pushNesting(Context::kArray));
+}
+
+Result JsonWriter::writeStoriesSlice(const Data::StoriesSlice &data) {
+	Expects(_output != nullptr);
+
+	if (data.list.empty()) {
+		return Result::Success();
+	}
+
+	auto block = QByteArray();
+	for (const auto &story : data.list) {
+		using SkipReason = Data::File::SkipReason;
+		const auto &file = story.file();
+		Assert(!file.relativePath.isEmpty()
+			|| file.skipReason != SkipReason::None);
+		const auto path = [&]() -> Data::Utf8String {
+			switch (file.skipReason) {
+			case SkipReason::Unavailable:
+				return "(Photo unavailable, please try again later)";
+			case SkipReason::FileSize:
+				return "(Photo exceeds maximum size. "
+					"Change data exporting settings to download.)";
+			case SkipReason::FileType:
+				return "(Photo not included. "
+					"Change data exporting settings to download.)";
+			case SkipReason::None: return FormatFilePath(file);
+			}
+			Unexpected("Skip reason while writing story path.");
+		}();
+		block.append(prepareArrayItemStart());
+		block.append(SerializeObject(_context, {
+			{
+				"date",
+				story.date ? SerializeDate(story.date) : QByteArray()
+			},
+			{
+				"date_unixtime",
+				story.date ? SerializeDateRaw(story.date) : QByteArray()
+			},
+			{
+				"expires",
+				story.expires ? SerializeDate(story.expires) : QByteArray()
+			},
+			{
+				"expires_unixtime",
+				story.expires ? SerializeDateRaw(story.expires) : QByteArray()
+			},
+			{
+				"pinned",
+				story.pinned ? "true" : "false"
+			},
+			{
+				"media",
+				SerializeString(path)
+			},
+		}));
+	}
+	return _output->writeBlock(block);
+}
+
+Result JsonWriter::writeStoriesEnd() {
+	Expects(_output != nullptr);
+
+	return _output->writeBlock(popNesting());
+}
+
 Result JsonWriter::writeContactsList(const Data::ContactsList &data) {
 	Expects(_output != nullptr);
 
@@ -784,11 +988,17 @@ Result JsonWriter::writeSavedContacts(const Data::ContactsList &data) {
 			&& contact.lastName.isEmpty()
 			&& contact.phoneNumber.isEmpty()) {
 			block.append(SerializeObject(_context, {
-				{ "date", SerializeDate(contact.date) }
+				{ "date", SerializeDate(contact.date) },
+				{ "date_unixtime", SerializeDateRaw(contact.date) },
 			}));
 		} else {
 			block.append(SerializeObject(_context, {
-				{ "user_id", Data::NumberToString(contact.userId) },
+				{
+					"user_id",
+					(contact.userId
+						? Data::NumberToString(contact.userId.bare)
+						: QByteArray())
+				},
 				{ "first_name", SerializeString(contact.firstName) },
 				{ "last_name", SerializeString(contact.lastName) },
 				{
@@ -796,7 +1006,8 @@ Result JsonWriter::writeSavedContacts(const Data::ContactsList &data) {
 					SerializeString(
 						Data::FormatPhoneNumber(contact.phoneNumber))
 				},
-				{ "date", SerializeDate(contact.date) }
+				{ "date", SerializeDate(contact.date) },
+				{ "date_unixtime", SerializeDateRaw(contact.date) },
 			}));
 		}
 	}
@@ -833,7 +1044,7 @@ Result JsonWriter::writeFrequentContacts(const Data::ContactsList &data) {
 			}();
 			block.append(prepareArrayItemStart());
 			block.append(SerializeObject(_context, {
-				{ "id", Data::NumberToString(top.peer.id()) },
+				{ "id", Data::NumberToString(Data::PeerToBareId(top.peer.id())) },
 				{ "category", SerializeString(category) },
 				{ "type", SerializeString(type) },
 				{ "name",  StringAllowNull(top.peer.name()) },
@@ -943,6 +1154,7 @@ Result JsonWriter::writeSessions(const Data::SessionsList &data) {
 		block.append(prepareArrayItemStart());
 		block.append(SerializeObject(_context, {
 			{ "last_active", SerializeDate(session.lastActive) },
+			{ "last_active_unixtime", SerializeDateRaw(session.lastActive) },
 			{ "last_ip", SerializeString(session.ip) },
 			{ "last_country", SerializeString(session.country) },
 			{ "last_region", SerializeString(session.region) },
@@ -958,6 +1170,7 @@ Result JsonWriter::writeSessions(const Data::SessionsList &data) {
 			{ "platform", SerializeString(session.platform) },
 			{ "system_version", SerializeString(session.systemVersion) },
 			{ "created", SerializeDate(session.created) },
+			{ "created_unixtime", SerializeDateRaw(session.created) },
 		}));
 	}
 	block.append(popNesting());
@@ -977,6 +1190,7 @@ Result JsonWriter::writeWebSessions(const Data::SessionsList &data) {
 		block.append(prepareArrayItemStart());
 		block.append(SerializeObject(_context, {
 			{ "last_active", SerializeDate(session.lastActive) },
+			{ "last_active_unixtime", SerializeDateRaw(session.lastActive) },
 			{ "last_ip", SerializeString(session.ip) },
 			{ "last_region", SerializeString(session.region) },
 			{ "bot_username", StringAllowNull(session.botUsername) },
@@ -984,6 +1198,7 @@ Result JsonWriter::writeWebSessions(const Data::SessionsList &data) {
 			{ "browser", SerializeString(session.browser) },
 			{ "platform", SerializeString(session.platform) },
 			{ "created", SerializeDate(session.created) },
+			{ "created_unixtime", SerializeDateRaw(session.created) },
 		}));
 	}
 	block.append(popNesting());
@@ -1009,6 +1224,7 @@ Result JsonWriter::writeDialogStart(const Data::DialogInfo &data) {
 		switch (type) {
 		case Type::Unknown: return "";
 		case Type::Self: return "saved_messages";
+		case Type::Replies: return "replies";
 		case Type::Personal: return "personal_chat";
 		case Type::Bot: return "bot_chat";
 		case Type::PrivateGroup: return "private_group";
@@ -1024,14 +1240,14 @@ Result JsonWriter::writeDialogStart(const Data::DialogInfo &data) {
 		? QByteArray()
 		: prepareArrayItemStart();
 	block.append(pushNesting(Context::kObject));
-	if (data.type != Type::Self) {
+	if (data.type != Type::Self && data.type != Type::Replies) {
 		block.append(prepareObjectItemStart("name")
 			+ StringAllowNull(data.name));
 	}
 	block.append(prepareObjectItemStart("type")
 		+ StringAllowNull(TypeString(data.type)));
 	block.append(prepareObjectItemStart("id")
-		+ Data::NumberToString(data.peerId));
+		+ Data::NumberToString(Data::PeerToBareId(data.peerId)));
 	block.append(prepareObjectItemStart("messages"));
 	block.append(pushNesting(Context::kArray));
 	return _output->writeBlock(block);
