@@ -10,19 +10,19 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_types.h"
 #include "data/data_peer.h"
 #include "data/data_drafts.h"
-#include "dialogs/dialogs_entry.h"
+#include "data/data_thread.h"
 #include "history/view/history_view_send_action.h"
-#include "base/observer.h"
-#include "base/timer.h"
 #include "base/variant.h"
 #include "base/flat_set.h"
 #include "base/flags.h"
 
 class History;
 class HistoryBlock;
+class HistoryTranslation;
 class HistoryItem;
-class HistoryMessage;
-class HistoryService;
+struct HistoryMessageMarkupData;
+class HistoryMainElementDelegateMixin;
+struct LanguageId;
 
 namespace Main {
 class Session;
@@ -33,6 +33,8 @@ struct Draft;
 class Session;
 class Folder;
 class ChatFilter;
+struct SponsoredFrom;
+class SponsoredMessages;
 
 enum class ForwardOptions {
 	PreserveInfo,
@@ -43,8 +45,14 @@ enum class ForwardOptions {
 struct ForwardDraft {
 	MessageIdsList ids;
 	ForwardOptions options = ForwardOptions::PreserveInfo;
+
+	friend inline auto operator<=>(
+		const ForwardDraft&,
+		const ForwardDraft&) = default;
 };
 
+using ForwardDrafts = base::flat_map<MsgId, ForwardDraft>;
+
 struct ResolvedForwardDraft {
 	HistoryItemsList items;
 	ForwardOptions options = ForwardOptions::PreserveInfo;
@@ -67,29 +75,34 @@ enum class NewMessageType {
 	Existing,
 };
 
-enum class UnreadMentionType {
-	New, // when new message is added to history
-	Existing, // when some messages slice was received
-};
-
-class History final : public Dialogs::Entry {
+class History final : public Data::Thread {
 public:
 	using Element = HistoryView::Element;
 
 	History(not_null<Data::Session*> owner, PeerId peerId);
-	History(const History &) = delete;
-	History &operator=(const History &) = delete;
 	~History();
 
-	ChannelId channelId() const;
-	bool isChannel() const;
-	bool isMegagroup() const;
+	not_null<History*> owningHistory() override {
+		return this;
+	}
+	[[nodiscard]] Data::Thread *threadFor(MsgId topicRootId);
+	[[nodiscard]] const Data::Thread *threadFor(MsgId topicRootId) const;
+
+	[[nodiscard]] auto delegateMixin() const
+			-> not_null<HistoryMainElementDelegateMixin*> {
+		return _delegateMixin.get();
+	}
+
+	void forumChanged(Data::Forum *old);
+	[[nodiscard]] bool isForum() const;
+
 	not_null<History*> migrateToOrMe() const;
 	History *migrateFrom() const;
 	MsgRange rangeForDifferenceRequest() const;
 	void checkLocalMessages();
 	void removeJoinedMessage();
 
+	void reactionsEnabledChanged(bool enabled);
 
 	bool isEmpty() const;
 	bool isDisplayedEmpty() const;
@@ -98,9 +111,8 @@ public:
 	Element *findLastNonEmpty() const;
 	Element *findLastDisplayed() const;
 	bool hasOrphanMediaGroupPart() const;
-	bool removeOrphanMediaGroupPart();
-	QVector<MsgId> collectMessagesFromUserToDelete(
-		not_null<UserData*> user) const;
+	[[nodiscard]] std::vector<MsgId> collectMessagesFromParticipantToDelete(
+		not_null<PeerData*> participant) const;
 
 	enum class ClearType {
 		Unload,
@@ -113,44 +125,36 @@ public:
 	void applyGroupAdminChanges(const base::flat_set<UserId> &changes);
 
 	template <typename ...Args>
-	not_null<HistoryMessage*> makeMessage(Args &&...args) {
-		return static_cast<HistoryMessage*>(
+	not_null<HistoryItem*> makeMessage(Args &&...args) {
+		return static_cast<HistoryItem*>(
 			insertItem(
-				std::make_unique<HistoryMessage>(
+				std::make_unique<HistoryItem>(
 					this,
 					std::forward<Args>(args)...)).get());
 	}
 
-	template <typename ...Args>
-	not_null<HistoryService*> makeServiceMessage(Args &&...args) {
-		return static_cast<HistoryService*>(
-			insertItem(
-				std::make_unique<HistoryService>(
-					this,
-					std::forward<Args>(args)...)).get());
-	}
 	void destroyMessage(not_null<HistoryItem*> item);
+	void destroyMessagesByDates(TimeId minDate, TimeId maxDate);
+	void destroyMessagesByTopic(MsgId topicRootId);
 
-	void unpinAllMessages();
+	void unpinMessagesFor(MsgId topicRootId);
 
-	HistoryItem *addNewMessage(
+	not_null<HistoryItem*> addNewMessage(
+		MsgId id,
 		const MTPMessage &msg,
 		MessageFlags localFlags,
 		NewMessageType type);
-	HistoryItem *addToHistory(
-		const MTPMessage &msg,
-		MessageFlags localFlags);
 	not_null<HistoryItem*> addNewLocalMessage(
 		MsgId id,
 		MessageFlags flags,
 		UserId viaBotId,
-		MsgId replyTo,
+		FullReplyTo replyTo,
 		TimeId date,
 		PeerId from,
 		const QString &postAuthor,
 		const TextWithEntities &text,
 		const MTPMessageMedia &media,
-		const MTPReplyMarkup &markup,
+		HistoryMessageMarkupData &&markup,
 		uint64 groupedId = 0);
 	not_null<HistoryItem*> addNewLocalMessage(
 		MsgId id,
@@ -158,42 +162,48 @@ public:
 		TimeId date,
 		PeerId from,
 		const QString &postAuthor,
-		not_null<HistoryMessage*> forwardOriginal);
+		not_null<HistoryItem*> forwardOriginal,
+		MsgId topicRootId);
 	not_null<HistoryItem*> addNewLocalMessage(
 		MsgId id,
 		MessageFlags flags,
 		UserId viaBotId,
-		MsgId replyTo,
+		FullReplyTo replyTo,
 		TimeId date,
 		PeerId from,
 		const QString &postAuthor,
 		not_null<DocumentData*> document,
 		const TextWithEntities &caption,
-		const MTPReplyMarkup &markup);
+		HistoryMessageMarkupData &&markup);
 	not_null<HistoryItem*> addNewLocalMessage(
 		MsgId id,
 		MessageFlags flags,
 		UserId viaBotId,
-		MsgId replyTo,
+		FullReplyTo replyTo,
 		TimeId date,
 		PeerId from,
 		const QString &postAuthor,
 		not_null<PhotoData*> photo,
 		const TextWithEntities &caption,
-		const MTPReplyMarkup &markup);
+		HistoryMessageMarkupData &&markup);
 	not_null<HistoryItem*> addNewLocalMessage(
 		MsgId id,
 		MessageFlags flags,
 		UserId viaBotId,
-		MsgId replyTo,
+		FullReplyTo replyTo,
 		TimeId date,
 		PeerId from,
 		const QString &postAuthor,
 		not_null<GameData*> game,
-		const MTPReplyMarkup &markup);
+		HistoryMessageMarkupData &&markup);
+	not_null<HistoryItem*> addNewLocalMessage(
+		MsgId id,
+		Data::SponsoredFrom from,
+		const TextWithEntities &textWithEntities); // sponsored
 
 	// Used only internally and for channel admin log.
-	HistoryItem *createItem(
+	not_null<HistoryItem*> createItem(
+		MsgId id,
 		const MTPMessage &message,
 		MessageFlags localFlags,
 		bool detachExistingItem);
@@ -205,9 +215,9 @@ public:
 
 	void newItemAdded(not_null<HistoryItem*> item);
 
-	void registerLocalMessage(not_null<HistoryItem*> item);
-	void unregisterLocalMessage(not_null<HistoryItem*> item);
-	[[nodiscard]] auto localMessages()
+	void registerClientSideMessage(not_null<HistoryItem*> item);
+	void unregisterClientSideMessage(not_null<HistoryItem*> item);
+	[[nodiscard]] auto clientSideMessages()
 		-> const base::flat_set<not_null<HistoryItem*>> &;
 	[[nodiscard]] HistoryItem *latestSendingMessage() const;
 
@@ -221,12 +231,13 @@ public:
 	void inboxRead(not_null<const HistoryItem*> wasRead);
 	void outboxRead(MsgId upTo);
 	void outboxRead(not_null<const HistoryItem*> wasRead);
-	[[nodiscard]] bool isServerSideUnread(
-		not_null<const HistoryItem*> item) const;
 	[[nodiscard]] MsgId loadAroundId() const;
 	[[nodiscard]] MsgId inboxReadTillId() const;
 	[[nodiscard]] MsgId outboxReadTillId() const;
 
+	[[nodiscard]] bool isServerSideUnread(
+		not_null<const HistoryItem*> item) const override;
+
 	[[nodiscard]] bool trackUnreadMessages() const;
 	[[nodiscard]] int unreadCount() const;
 	[[nodiscard]] bool unreadCountKnown() const;
@@ -236,20 +247,15 @@ public:
 
 	void setUnreadCount(int newUnreadCount);
 	void setUnreadMark(bool unread);
-	[[nodiscard]] bool unreadMark() const;
 	void setFakeUnreadWhileOpened(bool enabled);
 	[[nodiscard]] bool fakeUnreadWhileOpened() const;
-	[[nodiscard]] int unreadCountForBadge() const; // unreadCount || unreadMark ? 1 : 0.
-	[[nodiscard]] bool mute() const;
-	bool changeMute(bool newMute);
+	void setMuted(bool muted) override;
 	void addUnreadBar();
 	void destroyUnreadBar();
 	[[nodiscard]] Element *unreadBar() const;
 	void calculateFirstUnreadMessage();
 	void unsetFirstUnreadMessage();
 	[[nodiscard]] Element *firstUnreadMessage() const;
-	void clearNotifications();
-	void clearIncomingNotifications();
 
 	[[nodiscard]] bool loadedAtBottom() const; // last message is in the list
 	void setNotLoadedAtBottom();
@@ -276,7 +282,7 @@ public:
 		bool promoted,
 		const QString &type,
 		const QString &message);
-	[[nodiscard]] QStringRef topPromotionType() const;
+	[[nodiscard]] QStringView topPromotionType() const;
 	[[nodiscard]] QString topPromotionMessage() const;
 	[[nodiscard]] bool topPromotionAboutShown() const;
 	void markTopPromotionAboutShown();
@@ -293,40 +299,17 @@ public:
 	void itemRemoved(not_null<HistoryItem*> item);
 	void itemVanished(not_null<HistoryItem*> item);
 
-	HistoryItem *currentNotification();
-	bool hasNotification() const;
-	void skipNotification();
-	void popNotification(HistoryItem *item);
-
 	bool hasPendingResizedItems() const;
 	void setHasPendingResizedItems();
 
 	[[nodiscard]] auto sendActionPainter()
-	-> not_null<HistoryView::SendActionPainter*> {
+	-> not_null<HistoryView::SendActionPainter*> override {
 		return &_sendActionPainter;
 	}
 
 	void clearLastKeyboard();
-
-	int getUnreadMentionsLoadedCount() const {
-		return _unreadMentions.size();
-	}
-	MsgId getMinLoadedUnreadMention() const {
-		return _unreadMentions.empty() ? 0 : _unreadMentions.front();
-	}
-	MsgId getMaxLoadedUnreadMention() const {
-		return _unreadMentions.empty() ? 0 : _unreadMentions.back();
-	}
-	int getUnreadMentionsCount(int notLoadedValue = -1) const {
-		return _unreadMentionsCount ? *_unreadMentionsCount : notLoadedValue;
-	}
-	bool hasUnreadMentions() const {
-		return (getUnreadMentionsCount() > 0);
-	}
-	void setUnreadMentionsCount(int count);
-	bool addToUnreadMentions(MsgId msgId, UnreadMentionType type);
-	void eraseFromUnreadMentions(MsgId msgId);
-	void addUnreadMentionsSlice(const MTPmessages_Messages &result);
+	void clearUnreadMentionsFor(MsgId topicRootId);
+	void clearUnreadReactionsFor(MsgId topicRootId);
 
 	Data::Draft *draft(Data::DraftKey key) const;
 	void setDraft(Data::DraftKey key, std::unique_ptr<Data::Draft> &&draft);
@@ -335,59 +318,67 @@ public:
 	[[nodiscard]] const Data::HistoryDrafts &draftsMap() const;
 	void setDraftsMap(Data::HistoryDrafts &&map);
 
-	Data::Draft *localDraft() const {
-		return draft(Data::DraftKey::Local());
+	Data::Draft *localDraft(MsgId topicRootId) const {
+		return draft(Data::DraftKey::Local(topicRootId));
 	}
-	Data::Draft *localEditDraft() const {
-		return draft(Data::DraftKey::LocalEdit());
+	Data::Draft *localEditDraft(MsgId topicRootId) const {
+		return draft(Data::DraftKey::LocalEdit(topicRootId));
 	}
-	Data::Draft *cloudDraft() const {
-		return draft(Data::DraftKey::Cloud());
+	Data::Draft *cloudDraft(MsgId topicRootId) const {
+		return draft(Data::DraftKey::Cloud(topicRootId));
 	}
 	void setLocalDraft(std::unique_ptr<Data::Draft> &&draft) {
-		setDraft(Data::DraftKey::Local(), std::move(draft));
+		setDraft(
+			Data::DraftKey::Local(draft->topicRootId),
+			std::move(draft));
 	}
 	void setLocalEditDraft(std::unique_ptr<Data::Draft> &&draft) {
-		setDraft(Data::DraftKey::LocalEdit(), std::move(draft));
+		setDraft(
+			Data::DraftKey::LocalEdit(draft->topicRootId),
+			std::move(draft));
 	}
 	void setCloudDraft(std::unique_ptr<Data::Draft> &&draft) {
-		setDraft(Data::DraftKey::Cloud(), std::move(draft));
+		setDraft(
+			Data::DraftKey::Cloud(draft->topicRootId),
+			std::move(draft));
 	}
-	void clearLocalDraft() {
-		clearDraft(Data::DraftKey::Local());
+	void clearLocalDraft(MsgId topicRootId) {
+		clearDraft(Data::DraftKey::Local(topicRootId));
 	}
-	void clearCloudDraft() {
-		clearDraft(Data::DraftKey::Cloud());
+	void clearCloudDraft(MsgId topicRootId) {
+		clearDraft(Data::DraftKey::Cloud(topicRootId));
 	}
-	void clearLocalEditDraft() {
-		clearDraft(Data::DraftKey::LocalEdit());
+	void clearLocalEditDraft(MsgId topicRootId) {
+		clearDraft(Data::DraftKey::LocalEdit(topicRootId));
 	}
 	void clearDrafts();
-	Data::Draft *createCloudDraft(const Data::Draft *fromDraft);
-	bool skipCloudDraftUpdate(TimeId date) const;
-	void startSavingCloudDraft();
-	void finishSavingCloudDraft(TimeId savedAt);
+	Data::Draft *createCloudDraft(
+		MsgId topicRootId,
+		const Data::Draft *fromDraft);
+	[[nodiscard]] bool skipCloudDraftUpdate(
+		MsgId topicRootId,
+		TimeId date) const;
+	void startSavingCloudDraft(MsgId topicRootId);
+	void finishSavingCloudDraft(MsgId topicRootId, TimeId savedAt);
 	void takeLocalDraft(not_null<History*> from);
-	void applyCloudDraft();
-	void draftSavedToCloud();
+	void applyCloudDraft(MsgId topicRootId);
+	void draftSavedToCloud(MsgId topicRootId);
 
-	[[nodiscard]] const Data::ForwardDraft &forwardDraft() const {
-		return _forwardDraft;
-	}
+	[[nodiscard]] const Data::ForwardDraft &forwardDraft(
+		MsgId topicRootId) const;
 	[[nodiscard]] Data::ResolvedForwardDraft resolveForwardDraft(
 		const Data::ForwardDraft &draft) const;
-	[[nodiscard]] Data::ResolvedForwardDraft resolveForwardDraft();
-	void setForwardDraft(Data::ForwardDraft &&draft);
+	[[nodiscard]] Data::ResolvedForwardDraft resolveForwardDraft(
+		MsgId topicRootId);
+	void setForwardDraft(MsgId topicRootId, Data::ForwardDraft &&draft);
 
 	History *migrateSibling() const;
 	[[nodiscard]] bool useTopPromotion() const;
 	int fixedOnTopIndex() const override;
 	void updateChatListExistence() override;
 	bool shouldBeInChatList() const override;
-	int chatListUnreadCount() const override;
-	bool chatListUnreadMark() const override;
-	bool chatListMutedBadge() const override;
 	Dialogs::UnreadState chatListUnreadState() const override;
+	Dialogs::BadgesState chatListBadgesState() const override;
 	HistoryItem *chatListMessage() const override;
 	bool chatListMessageKnown() const override;
 	void requestChatListMessage() override;
@@ -395,19 +386,21 @@ public:
 	const QString &chatListNameSortKey() const override;
 	const base::flat_set<QString> &chatListNameWords() const override;
 	const base::flat_set<QChar> &chatListFirstLetters() const override;
-	void loadUserpic() override;
+	void chatListPreloadData() override;
 	void paintUserpic(
 		Painter &p,
-		std::shared_ptr<Data::CloudImageView> &view,
-		int x,
-		int y,
-		int size) const override;
+		Ui::PeerUserpicView &view,
+		const Dialogs::Ui::PaintContext &context) const override;
 
 	void refreshChatListNameSortKey();
 
 	void setFakeChatListMessageFrom(const MTPmessages_Messages &data);
 	void checkChatListMessageRemoved(not_null<HistoryItem*> item);
 
+	void applyChatListGroup(
+		PeerId dataPeerId,
+		const MTPmessages_Messages &data);
+
 	void forgetScrollState() {
 		scrollTopItem = nullptr;
 	}
@@ -418,7 +411,7 @@ public:
 
 	[[nodiscard]] std::pair<Element*, int> findItemAndOffset(int top) const;
 
-	MsgId nextNonHistoryEntryId();
+	[[nodiscard]] MsgId nextNonHistoryEntryId();
 
 	bool folderKnown() const override;
 	Data::Folder *folder() const override;
@@ -431,14 +424,20 @@ public:
 	void setInboxReadTill(MsgId upTo);
 	std::optional<int> countStillUnreadLocal(MsgId readTillId) const;
 
-	[[nodiscard]] bool hasPinnedMessages() const;
-	void setHasPinnedMessages(bool has);
+	[[nodiscard]] bool isTopPromoted() const;
+
+	void translateOfferFrom(LanguageId id);
+	[[nodiscard]] LanguageId translateOfferedFrom() const;
+	void translateTo(LanguageId id);
+	[[nodiscard]] LanguageId translatedTo() const;
+
+	[[nodiscard]] HistoryTranslation *translation() const;
+
+	const not_null<PeerData*> peer;
 
 	// Still public data.
 	std::deque<std::unique_ptr<HistoryBlock>> blocks;
 
-	not_null<PeerData*> peer;
-
 	// we save the last showAtMsgId to restore the state when switching
 	// between different conversation histories
 	MsgId showAtMsgId = ShowAtUnreadMsgId;
@@ -457,19 +456,25 @@ public:
 
 	mtpRequestId sendRequestId = 0;
 
-	Ui::Text::String cloudDraftTextCache;
-
 private:
 	friend class HistoryBlock;
 
-	enum class Flag {
-		f_has_pending_resized_items = (1 << 0),
+	enum class Flag : uchar {
+		HasPendingResizedItems = (1 << 0),
+		PendingAllItemsResize = (1 << 1),
+		IsTopPromoted = (1 << 2),
+		IsForum = (1 << 3),
+		FakeUnreadWhileOpened = (1 << 4),
+		HasPinnedMessages = (1 << 5),
+		ResolveChatListMessage = (1 << 6),
 	};
 	using Flags = base::flags<Flag>;
 	friend inline constexpr auto is_flag_type(Flag) {
 		return true;
 	};
 
+	void cacheTopPromoted(bool promoted);
+
 	// when this item is destroyed scrollTopItem just points to the next one
 	// and scrollTopOffset remains the same
 	// if we are at the bottom of the window scrollTopItem == nullptr and
@@ -492,6 +497,8 @@ private:
 	not_null<HistoryItem*> addNewToBack(
 		not_null<HistoryItem*> item,
 		bool unread);
+
+	friend class Data::SponsoredMessages;
 	not_null<HistoryItem*> addNewInTheMiddle(
 		not_null<HistoryItem*> item,
 		int blockIndex,
@@ -514,11 +521,13 @@ private:
 		return _buildingFrontBlock != nullptr;
 	}
 
+	void addCreatedOlderSlice(
+		const std::vector<not_null<HistoryItem*>> &items);
+
 	void checkForLoadedAtTop(not_null<HistoryItem*> added);
 	void mainViewRemoved(
 		not_null<HistoryBlock*> block,
 		not_null<Element*> view);
-	void removeNotification(not_null<HistoryItem*> item);
 
 	TimeId adjustedChatListTimeId() const override;
 	void changedChatListPinHook() override;
@@ -544,6 +553,8 @@ private:
 	void setChatListMessageFromLast();
 	void setChatListMessageUnknown();
 	void setFakeChatListMessage();
+	void allowChatListMessageResolve();
+	void resolveChatListMessageGroup();
 
 	// Add all items to the unread mentions if we were not loaded at bottom and now are.
 	void checkAddAllToUnreadMentions();
@@ -565,20 +576,30 @@ private:
 
 	void viewReplaced(not_null<const Element*> was, Element *now);
 
-	void createLocalDraftFromCloud();
+	void createLocalDraftFromCloud(MsgId topicRootId);
 
-	HistoryService *insertJoinedMessage();
-	void insertLocalMessage(not_null<HistoryItem*> item);
+	HistoryItem *insertJoinedMessage();
+	void insertMessageToBlocks(not_null<HistoryItem*> item);
 
+	[[nodiscard]] Dialogs::BadgesState computeBadgesState() const;
+	[[nodiscard]] Dialogs::BadgesState adjustBadgesStateByFolder(
+		Dialogs::BadgesState state) const;
+	[[nodiscard]] Dialogs::UnreadState computeUnreadState() const;
 	void setFolderPointer(Data::Folder *folder);
 
+	int chatListNameVersion() const override;
+
+	void hasUnreadMentionChanged(bool has) override;
+	void hasUnreadReactionChanged(bool has) override;
+
+	const std::unique_ptr<HistoryMainElementDelegateMixin> _delegateMixin;
+
 	Flags _flags = 0;
-	bool _mute = false;
 	int _width = 0;
 	int _height = 0;
 	Element *_unreadBarView = nullptr;
 	Element *_firstUnreadView = nullptr;
-	HistoryService *_joinedMessage = nullptr;
+	HistoryItem *_joinedMessage = nullptr;
 	bool _loadedAtTop = false;
 	bool _loadedAtBottom = true;
 
@@ -587,11 +608,9 @@ private:
 	std::optional<MsgId> _inboxReadBefore;
 	std::optional<MsgId> _outboxReadBefore;
 	std::optional<int> _unreadCount;
-	std::optional<int> _unreadMentionsCount;
-	base::flat_set<MsgId> _unreadMentions;
 	std::optional<HistoryItem*> _lastMessage;
 	std::optional<HistoryItem*> _lastServerMessage;
-	base::flat_set<not_null<HistoryItem*>> _localMessages;
+	base::flat_set<not_null<HistoryItem*>> _clientSideMessages;
 	std::unordered_set<std::unique_ptr<HistoryItem>> _messages;
 
 	// This almost always is equal to _lastMessage. The only difference is
@@ -601,10 +620,6 @@ private:
 
 	QString _chatListNameSortKey;
 
-	bool _unreadMark = false;
-	bool _fakeUnreadWhileOpened = false;
-	bool _hasPinnedMessages = false;
-
 	// A pointer to the block that is currently being built.
 	// We hold this pointer so we can destroy it while building
 	// and then create a new one if it is necessary.
@@ -613,25 +628,31 @@ private:
 		HistoryBlock *block = nullptr;
 	};
 	std::unique_ptr<BuildingBlock> _buildingFrontBlock;
+	std::unique_ptr<HistoryTranslation> _translation;
 
 	Data::HistoryDrafts _drafts;
-	TimeId _acceptCloudDraftsAfter = 0;
-	int _savingCloudDraftRequests = 0;
-	Data::ForwardDraft _forwardDraft;
+	base::flat_map<MsgId, TimeId> _acceptCloudDraftsAfter;
+	base::flat_map<MsgId, int> _savingCloudDraftRequests;
+	Data::ForwardDrafts _forwardDrafts;
 
 	QString _topPromotedMessage;
 	QString _topPromotedType;
 
 	HistoryView::SendActionPainter _sendActionPainter;
 
-	std::deque<not_null<HistoryItem*>> _notifications;
 
- };
+};
 
 class HistoryBlock {
 public:
 	using Element = HistoryView::Element;
 
+	enum class ResizeRequest {
+		ReinitAll = 0,
+		ResizeAll = 1,
+		ResizePending = 2,
+	};
+
 	HistoryBlock(not_null<History*> history);
 	HistoryBlock(const HistoryBlock &) = delete;
 	HistoryBlock &operator=(const HistoryBlock &) = delete;
@@ -642,7 +663,7 @@ public:
 	void remove(not_null<Element*> view);
 	void refreshView(not_null<Element*> view);
 
-	int resizeGetHeight(int newWidth, bool resizeAllItems);
+	int resizeGetHeight(int newWidth, ResizeRequest request);
 	int y() const {
 		return _y;
 	}
