@@ -7,14 +7,14 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "history/view/history_view_top_bar_widget.h"
 
-#include <rpl/combine.h>
-#include <rpl/combine_previous.h>
 #include "history/history.h"
 #include "history/view/history_view_send_action.h"
 #include "boxes/add_contact_box.h"
 #include "ui/boxes/confirm_box.h"
 #include "info/info_memento.h"
 #include "info/info_controller.h"
+#include "info/profile/info_profile_values.h"
+#include "storage/storage_media_prepare.h"
 #include "storage/storage_shared_media.h"
 #include "mainwidget.h"
 #include "mainwindow.h"
@@ -24,16 +24,17 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/shortcuts.h"
 #include "core/application.h"
 #include "core/core_settings.h"
+#include "ui/controls/userpic_button.h"
+#include "ui/wrap/fade_wrap.h"
 #include "ui/widgets/buttons.h"
+#include "ui/widgets/fields/input_field.h"
 #include "ui/widgets/popup_menu.h"
 #include "ui/widgets/menu/menu_add_action_callback_factory.h"
 #include "ui/effects/radial_animation.h"
-#include "ui/toasts/common_toasts.h"
 #include "ui/boxes/report_box.h" // Ui::ReportReason
 #include "ui/text/text.h"
 #include "ui/text/text_options.h"
 #include "ui/painter.h"
-#include "ui/special_buttons.h"
 #include "ui/unread_badge.h"
 #include "ui/ui_utility.h"
 #include "window/window_adaptive.h"
@@ -44,10 +45,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_group_call.h" // GroupCall::input.
 #include "data/data_folder.h"
 #include "data/data_session.h"
+#include "data/data_stories.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
 #include "data/data_user.h"
 #include "data/data_changes.h"
+#include "data/data_forum_topic.h"
 #include "data/data_send_action.h"
 #include "chat_helpers/emoji_interactions.h"
 #include "base/unixtime.h"
@@ -80,6 +83,10 @@ struct TopBarWidget::EmojiInteractionSeenAnimation {
 	crl::time till = 0;
 };
 
+QString SwitchToChooseFromQuery() {
+	return u"from:"_q;
+}
+
 TopBarWidget::TopBarWidget(
 	QWidget *parent,
 	not_null<Window::SessionController*> controller)
@@ -117,7 +124,10 @@ TopBarWidget::TopBarWidget(
 	_groupCall->setClickedCallback([=] { groupCall(); });
 	_menuToggle->setClickedCallback([=] { showPeerMenu(); });
 	_infoToggle->setClickedCallback([=] { toggleInfoSection(); });
-	_back->addClickHandler([=] { backClicked(); });
+	_back->setAcceptBoth();
+	_back->addClickHandler([=](Qt::MouseButton) {
+		InvokeQueued(_back.data(), [=] { backClicked(); });
+	});
 	_cancelChoose->setClickedCallback(
 		[=] { _cancelChooseForReport.fire({}); });
 
@@ -153,7 +163,7 @@ TopBarWidget::TopBarWidget(
 		using AnimationUpdate = Data::SendActionManager::AnimationUpdate;
 		session().data().sendActionManager().animationUpdated(
 		) | rpl::filter([=](const AnimationUpdate &update) {
-			return (update.history == _activeChat.key.history());
+			return (update.thread == _activeChat.key.thread());
 		}) | rpl::start_with_next([=] {
 			update();
 		}, lifetime());
@@ -178,11 +188,15 @@ TopBarWidget::TopBarWidget(
 			&& (_activeChat.key.peer() == update.peer)) {
 			updateControlsVisibility();
 		}
-		if (update.flags
-			& (UpdateFlag::OnlineStatus
-				| UpdateFlag::Members
-				| UpdateFlag::SupportInfo)) {
+		if ((update.flags & UpdateFlag::OnlineStatus)
+			&& trackOnlineOf(update.peer)) {
 			updateOnlineDisplay();
+		} else if (update.flags
+			& (UpdateFlag::Members | UpdateFlag::SupportInfo)) {
+			if (update.peer == _activeChat.key.peer()
+				&& !_activeChat.key.topic()) {
+				updateOnlineDisplay();
+			}
 		}
 		if ((update.flags & UpdateFlag::EmojiStatus)
 			&& (_activeChat.key.peer() == update.peer)) {
@@ -330,34 +344,8 @@ void TopBarWidget::showGroupCallMenu(not_null<PeerData*> peer) {
 	if (!created) {
 		return;
 	}
-	const auto controller = _controller;
-	const auto callback = [=](Calls::StartGroupCallArgs &&args) {
-		controller->startOrJoinGroupCall(peer, std::move(args));
-	};
-	const auto rtmpCallback = [=] {
-		Core::App().calls().showStartWithRtmp(
-			std::make_shared<Window::Show>(controller),
-			peer);
-	};
-	const auto livestream = !peer->isMegagroup() && peer->isChannel();
-	_menu->addAction(
-		(livestream
-			? tr::lng_menu_start_group_call_channel
-			: tr::lng_menu_start_group_call)(tr::now),
-		[=] { callback({}); },
-		&st::menuIconStartStream);
-	_menu->addAction(
-		(livestream
-			? tr::lng_menu_start_group_call_scheduled_channel
-			: tr::lng_menu_start_group_call_scheduled)(tr::now),
-		[=] { callback({ .scheduleNeeded = true }); },
-		&st::menuIconReschedule);
-	_menu->addAction(
-		(livestream
-			? tr::lng_menu_start_group_call_with_channel
-			: tr::lng_menu_start_group_call_with)(tr::now),
-		rtmpCallback,
-		&st::menuIconStartStreamWith);
+	const auto addAction = Ui::Menu::CreateAddActionCallback(_menu);
+	Window::FillVideoChatMenu(_controller, _activeChat, addAction);
 	_menu->setForcedOrigin(Ui::PanelAnimation::Origin::TopRight);
 	_menu->popup(mapToGlobal(QPoint(
 		_groupCall->x() + _groupCall->width() + st::topBarMenuGroupCallSkip,
@@ -376,7 +364,10 @@ void TopBarWidget::toggleInfoSection() {
 			Core::App().saveSettingsDelayed();
 			if (isThreeColumn) {
 				_controller->showSection(
-					Info::Memento::Default(_activeChat.key.peer()),
+					(_activeChat.key.topic()
+						? std::make_shared<Info::Memento>(
+							_activeChat.key.topic())
+						: Info::Memento::Default(_activeChat.key.peer())),
 					Window::SectionShow().withThirdColumn());
 			} else {
 				_controller->resizeForThirdSection();
@@ -419,18 +410,22 @@ void TopBarWidget::paintEvent(QPaintEvent *e) {
 	}
 	Painter p(this);
 
-	auto selectedButtonsTop = countSelectedButtonsTop(
+	const auto selectedButtonsTop = countSelectedButtonsTop(
 		_selectedShown.value(showSelectedActions() ? 1. : 0.));
+	const auto searchFieldTop = _searchField
+		? countSelectedButtonsTop(_searchShown.value(_searchMode ? 1. : 0.))
+		: -st::topBarHeight;
+	const auto slidingTop = std::max(selectedButtonsTop, searchFieldTop);
 
 	p.fillRect(QRect(0, 0, width(), st::topBarHeight), st::topBarBg);
-	if (selectedButtonsTop < 0) {
-		p.translate(0, selectedButtonsTop + st::topBarHeight);
+	if (slidingTop < 0) {
+		p.translate(0, slidingTop + st::topBarHeight);
 		paintTopBar(p);
 	}
 }
 
 void TopBarWidget::paintTopBar(Painter &p) {
-	if (!_activeChat.key) {
+	if (!_activeChat.key || _narrowRatio == 1.) {
 		return;
 	}
 	auto nameleft = _leftTaken;
@@ -472,13 +467,34 @@ void TopBarWidget::paintTopBar(Painter &p) {
 	}
 
 	const auto now = crl::now();
-	const auto history = _activeChat.key.history();
+	const auto history = _activeChat.key.owningHistory();
 	const auto folder = _activeChat.key.folder();
-	if (folder
+	const auto topic = _activeChat.key.topic();
+	if (topic && _activeChat.section == Section::Replies) {
+		p.setPen(st::dialogsNameFg);
+		topic->chatListNameText().drawElided(
+			p,
+			nameleft,
+			nametop,
+			availableWidth);
+
+		p.setFont(st::dialogsTextFont);
+		if (!paintConnectingState(p, nameleft, statustop, width())
+			&& !paintSendAction(
+				p,
+				nameleft,
+				statustop,
+				availableWidth,
+				width(),
+				st::historyStatusFgTyping,
+				now)) {
+			p.setPen(st::historyStatusFg);
+			p.drawTextLeft(nameleft, statustop, width(), _customTitleText);
+		}
+	} else if (folder
 		|| history->peer->sharedMediaInfo()
 		|| (_activeChat.section == Section::Scheduled)
 		|| (_activeChat.section == Section::Pinned)) {
-		// #TODO feed name emoji.
 		auto text = (_activeChat.section == Section::Scheduled)
 			? ((history && history->peer->isSelf())
 				? tr::lng_reminder_messages(tr::now)
@@ -489,7 +505,9 @@ void TopBarWidget::paintTopBar(Painter &p) {
 			? folder->chatListName()
 			: history->peer->isSelf()
 			? tr::lng_saved_messages(tr::now)
-			: tr::lng_replies_messages(tr::now);
+			: history->peer->isRepliesChat()
+			? tr::lng_replies_messages(tr::now)
+			: history->peer->name();
 		const auto textWidth = st::historySavedFont->width(text);
 		if (availableWidth < textWidth) {
 			text = st::historySavedFont->elided(text, availableWidth);
@@ -544,10 +562,9 @@ void TopBarWidget::paintTopBar(Painter &p) {
 			{
 				.peer = peer,
 				.verified = &st::dialogsVerifiedIcon,
-				.premium = &st::dialogsPremiumIcon,
+				.premium = &st::dialogsPremiumIcon.icon,
 				.scam = &st::attentionButtonFg,
 				.premiumFg = &st::dialogsVerifiedIconBg,
-				.preview = st::windowBgOver->c,
 				.customEmojiRepaint = [=] { update(); },
 				.now = now,
 				.paused = _controller->isGifPausedAtLeastFor(
@@ -656,9 +673,11 @@ void TopBarWidget::mousePressEvent(QMouseEvent *e) {
 		&& !showSelectedState()
 		&& !_chooseForReportReason;
 	if (handleClick) {
-		if (_animatingMode && _back->rect().contains(e->pos())) {
+		if ((_animatingMode && _back->rect().contains(e->pos()))
+			|| (_activeChat.section == Section::ChatsList
+				&& _activeChat.key.folder())) {
 			backClicked();
-		} else  {
+		} else {
 			infoClicked();
 		}
 	}
@@ -668,8 +687,8 @@ void TopBarWidget::infoClicked() {
 	const auto key = _activeChat.key;
 	if (!key) {
 		return;
-	} else if (key.folder()) {
-		_controller->closeFolder();
+	} else if (const auto topic = key.topic()) {
+		_controller->showSection(std::make_shared<Info::Memento>(topic));
 	} else if (key.peer()->isSelf()) {
 		_controller->showSection(std::make_shared<Info::Memento>(
 			key.peer(),
@@ -686,6 +705,10 @@ void TopBarWidget::infoClicked() {
 void TopBarWidget::backClicked() {
 	if (_activeChat.key.folder()) {
 		_controller->closeFolder();
+	} else if (_activeChat.section == Section::ChatsList
+		&& _activeChat.key.history()
+		&& _activeChat.key.history()->isForum()) {
+		_controller->closeForum();
 	} else {
 		_controller->showBackFromStack();
 	}
@@ -694,31 +717,33 @@ void TopBarWidget::backClicked() {
 void TopBarWidget::setActiveChat(
 		ActiveChat activeChat,
 		SendActionPainter *sendAction) {
+	_sendAction = sendAction;
 	if (_activeChat.key == activeChat.key
 		&& _activeChat.section == activeChat.section) {
 		_activeChat = activeChat;
 		return;
 	}
+	const auto topicChanged = (_activeChat.key.topic()
+		!= activeChat.key.topic());
 	const auto peerChanged = (_activeChat.key.history()
 		!= activeChat.key.history());
 
 	_activeChat = activeChat;
-	_sendAction = sendAction;
 	_titlePeerText.clear();
 	_back->clearState();
 	update();
 
-	if (peerChanged) {
+	if (peerChanged || topicChanged) {
 		_titleBadge.unload();
 		_titleNameVersion = 0;
 		_emojiInteractionSeen = nullptr;
 		_activeChatLifetime.destroy();
-		if (const auto history = _activeChat.key.history()) {
+		if (const auto peer = _activeChat.key.peer()) {
 			session().changes().peerFlagsValue(
-				history->peer,
+				peer,
 				Data::PeerUpdate::Flag::GroupCall
 			) | rpl::map([=] {
-				return history->peer->groupCall();
+				return peer->groupCall();
 			}) | rpl::distinct_until_changed(
 			) | rpl::map([](Data::GroupCall *call) {
 				return call ? call->fullCountValue() : rpl::single(-1);
@@ -731,13 +756,40 @@ void TopBarWidget::setActiveChat(
 				updateControlsGeometry();
 			}, _activeChatLifetime);
 
+			if (const auto channel = peer->asChannel()) {
+				if (channel->canEditStories()
+					&& !channel->owner().stories().archiveCountKnown(
+						channel->id)) {
+					channel->owner().stories().archiveLoadMore(channel->id);
+				}
+			}
+		}
+
+		if (const auto history = _activeChat.key.history()) {
 			using InteractionSeen = ChatHelpers::EmojiInteractionSeen;
 			_controller->emojiInteractions().seen(
 			) | rpl::filter([=](const InteractionSeen &seen) {
 				return (seen.peer == history->peer);
 			}) | rpl::start_with_next([=](const InteractionSeen &seen) {
 				handleEmojiInteractionSeen(seen.emoticon);
-			}, lifetime());
+			}, _activeChatLifetime);
+		}
+
+		if (const auto topic = _activeChat.key.topic()) {
+			Info::Profile::NameValue(
+				topic->channel()
+			) | rpl::start_with_next([=](const QString &name) {
+				_titlePeerText.setText(st::dialogsTextStyle, name);
+				_titlePeerTextOnline = false;
+				update();
+			}, _activeChatLifetime);
+
+			// _menuToggle visibility depends on "View topic info",
+			// "View topic info" visibility depends on activeChatCurrent.
+			_controller->activeChatChanges(
+			) | rpl::start_with_next([=] {
+				updateControlsVisibility();
+			}, _activeChatLifetime);
 		}
 	}
 	updateUnreadBadge();
@@ -748,6 +800,7 @@ void TopBarWidget::setActiveChat(
 	updateOnlineDisplay();
 	updateControlsVisibility();
 	refreshUnreadBadge();
+	setupDragOnBackButton();
 }
 
 void TopBarWidget::handleEmojiInteractionSeen(const QString &emoticon) {
@@ -793,12 +846,13 @@ void TopBarWidget::setCustomTitle(const QString &title) {
 }
 
 void TopBarWidget::refreshInfoButton() {
-	if (const auto peer = _activeChat.key.peer()) {
+	if (_activeChat.key.topic()
+		|| _activeChat.section == Section::ChatsList) {
+		_info.destroy();
+	} else if (const auto peer = _activeChat.key.peer()) {
 		auto info = object_ptr<Ui::UserpicButton>(
 			this,
-			_controller,
 			peer,
-			Ui::UserpicButton::Role::Custom,
 			st::topBarInfoButton);
 		info->showSavedMessagesOnSelf(true);
 		_info.destroy();
@@ -819,21 +873,31 @@ int TopBarWidget::countSelectedButtonsTop(float64 selectedShown) {
 }
 
 void TopBarWidget::updateSearchVisibility() {
-	const auto historyMode = (_activeChat.section == Section::History);
-	const auto smallDialogsColumn = _activeChat.key.folder()
-		&& (width() < _back->width() + _search->width());
-	_search->setVisible(historyMode
-		&& !smallDialogsColumn
-		&& !_chooseForReportReason);
+	const auto searchAllowedMode = (_activeChat.section == Section::History)
+		|| (_activeChat.section == Section::Replies
+			&& _activeChat.key.topic());
+	_search->setVisible(searchAllowedMode && !_chooseForReportReason);
 }
 
 void TopBarWidget::updateControlsGeometry() {
 	if (!_activeChat.key) {
 		return;
 	}
-	auto hasSelected = showSelectedActions();
-	auto selectedButtonsTop = countSelectedButtonsTop(_selectedShown.value(hasSelected ? 1. : 0.));
-	auto otherButtonsTop = selectedButtonsTop + st::topBarHeight;
+	const auto hasSelected = showSelectedActions();
+	auto selectedButtonsTop = countSelectedButtonsTop(
+		_selectedShown.value(hasSelected ? 1. : 0.));
+	if (!_searchMode && !_searchShown.animating() && _searchField) {
+		_searchField.destroy();
+		_searchCancel.destroy();
+		_jumpToDate.destroy();
+		_chooseFromUser.destroy();
+	}
+	auto searchFieldTop = _searchField
+		? countSelectedButtonsTop(_searchShown.value(_searchMode ? 1. : 0.))
+		: -st::topBarHeight;
+	const auto otherButtonsTop = std::max(selectedButtonsTop, searchFieldTop)
+		+ st::topBarHeight;
+	const auto backButtonTop = selectedButtonsTop + st::topBarHeight;
 	auto buttonsLeft = st::topBarActionSkip
 		+ (_controller->adaptive().isOneColumn() ? 0 : st::lineWidth);
 	auto buttonsWidth = (_forward->isHidden() ? 0 : _forward->contentWidth())
@@ -870,15 +934,46 @@ void TopBarWidget::updateControlsGeometry() {
 	} else if (_back->isHidden()) {
 		_leftTaken = st::topBarArrowPadding.right();
 	} else {
-		const auto smallDialogsColumn = _activeChat.key.folder()
-			&& (width() < _back->width() + _search->width());
-		_leftTaken = smallDialogsColumn ? (width() - _back->width()) / 2 : 0;
-		_back->moveToLeft(_leftTaken, otherButtonsTop);
+		_leftTaken = anim::interpolate(
+			0,
+			(_narrowWidth - _back->width()) / 2,
+			_narrowRatio);
+		_back->moveToLeft(_leftTaken, backButtonTop);
 		_leftTaken += _back->width();
 	}
 	if (_info && !_info->isHidden()) {
 		_info->moveToLeft(_leftTaken, otherButtonsTop);
 		_leftTaken += _info->width();
+	} else if (_activeChat.key.topic()
+		|| _activeChat.section == Section::ChatsList) {
+		_leftTaken += st::normalFont->spacew;
+	}
+
+	if (_searchField) {
+		const auto fieldLeft = _leftTaken;
+		const auto fieldTop = searchFieldTop
+			+ (height() - _searchField->height()) / 2;
+		const auto fieldRight = st::dialogsFilterSkip
+			+ st::dialogsFilterPadding.x();
+		const auto fieldWidth = width() - fieldLeft - fieldRight;
+		_searchField->setGeometryToLeft(
+			fieldLeft,
+			fieldTop,
+			fieldWidth,
+			_searchField->height());
+
+		auto right = fieldLeft + fieldWidth;
+		_searchCancel->moveToLeft(
+			right - _searchCancel->width(),
+			_searchField->y());
+		right -= st::dialogsCalendar.width;
+		if (_jumpToDate) {
+			_jumpToDate->moveToLeft(right, _searchField->y());
+		}
+		right -= st::dialogsSearchFrom.width;
+		if (_chooseFromUser) {
+			_chooseFromUser->moveToLeft(right, _searchField->y());
+		}
 	}
 
 	_rightTaken = 0;
@@ -899,7 +994,9 @@ void TopBarWidget::updateControlsGeometry() {
 		_rightTaken += _call->width();
 	}
 	_search->moveToRight(_rightTaken, otherButtonsTop);
-	_rightTaken += _search->width() + st::topBarCallSkip;
+	if (!_search->isHidden()) {
+		_rightTaken += _search->width() + st::topBarCallSkip;
+	}
 
 	updateMembersShowArea();
 }
@@ -933,28 +1030,71 @@ void TopBarWidget::updateControlsVisibility() {
 	const auto isOneColumn = _controller->adaptive().isOneColumn();
 	auto backVisible = isOneColumn
 		|| !_controller->content()->stackIsEmpty()
-		|| _activeChat.key.folder();
+		|| (_activeChat.section == Section::ChatsList);
 	_back->setVisible(backVisible && !_chooseForReportReason);
 	_cancelChoose->setVisible(_chooseForReportReason.has_value());
 	if (_info) {
-		_info->setVisible((isOneColumn || !_primaryWindow)
-			&& !_chooseForReportReason);
+		_info->setVisible(!_chooseForReportReason
+			&& (isOneColumn || !_primaryWindow));
 	}
 	if (_unreadBadge) {
 		_unreadBadge->setVisible(!_chooseForReportReason);
 	}
+	const auto topic = _activeChat.key.topic();
 	const auto section = _activeChat.section;
 	const auto historyMode = (section == Section::History);
-	const auto hasPollsMenu = _activeChat.key.peer()
-		&& _activeChat.key.peer()->canSendPolls();
+	const auto hasPollsMenu = (_activeChat.key.peer()
+		&& _activeChat.key.peer()->canCreatePolls())
+		|| (topic && Data::CanSend(topic, ChatRestriction::SendPolls));
+	const auto hasTopicMenu = [&] {
+		if (!topic || section != Section::Replies) {
+			return false;
+		}
+		auto empty = true;
+		const auto callback = [&](const Ui::Menu::MenuCallback::Args&) {
+			empty = false;
+			return (QAction*)nullptr;
+		};
+		Window::FillDialogsEntryMenu(
+			_controller,
+			_activeChat,
+			Ui::Menu::MenuCallback(callback));
+		return !empty;
+	}();
 	const auto hasMenu = !_activeChat.key.folder()
-		&& ((section == Section::Scheduled || section == Section::Replies)
+		&& (section == Section::History
+			? true
+			: (section == Section::Scheduled)
 			? hasPollsMenu
-			: historyMode);
+			: (section == Section::Replies)
+			? (hasPollsMenu || hasTopicMenu)
+			: (section == Section::ChatsList)
+			? (_activeChat.key.peer() && _activeChat.key.peer()->isForum())
+			: false);
+	const auto hasInfo = !_activeChat.key.folder()
+		&& (section == Section::History
+			? true
+			: (section == Section::Replies)
+			? (_activeChat.key.topic() != nullptr)
+			: false);
 	updateSearchVisibility();
-	_menuToggle->setVisible(hasMenu && !_chooseForReportReason);
-	_infoToggle->setVisible(historyMode
-		&& !_activeChat.key.folder()
+	if (_searchMode) {
+		const auto hasSearchQuery = _searchField
+			&& !_searchField->getLastText().isEmpty();
+		if (!_jumpToDate || hasSearchQuery) {
+			_searchCancel->show(anim::type::normal);
+			if (_jumpToDate) {
+				_jumpToDate->hide(anim::type::normal);
+			}
+		} else {
+			_searchCancel->hide(anim::type::normal);
+			_jumpToDate->show(anim::type::normal);
+		}
+	}
+	_menuToggle->setVisible(hasMenu
+		&& !_chooseForReportReason
+		&& (_narrowRatio < 1.));
+	_infoToggle->setVisible(hasInfo
 		&& !isOneColumn
 		&& _controller->canShowThirdSection()
 		&& !_chooseForReportReason);
@@ -993,7 +1133,10 @@ void TopBarWidget::updateControlsVisibility() {
 void TopBarWidget::updateMembersShowArea() {
 	const auto membersShowAreaNeeded = [&] {
 		const auto peer = _activeChat.key.peer();
-		if (showSelectedState() || !peer) {
+		if (showSelectedState()
+			|| !peer
+			|| _activeChat.section == Section::ChatsList
+			|| _activeChat.key.topic()) {
 			return false;
 		} else if (const auto chat = peer->asChat()) {
 			return chat->amIn();
@@ -1075,20 +1218,193 @@ void TopBarWidget::showSelected(SelectedState state) {
 	}
 }
 
+bool TopBarWidget::toggleSearch(bool shown, anim::type animated) {
+	if (_searchMode == shown) {
+		if (animated == anim::type::instant) {
+			_searchShown.stop();
+		}
+		return false;
+	}
+	_searchMode = shown;
+	if (shown && !_searchField) {
+		_searchField.create(this, st::dialogsFilter, tr::lng_dlg_filter());
+		_searchField->setFocusPolicy(Qt::StrongFocus);
+		_searchField->customUpDown(true);
+		_searchField->show();
+		_searchCancel.create(this, st::dialogsCancelSearch);
+		_searchCancel->show(anim::type::instant);
+		_searchCancel->setClickedCallback([=] { _searchCancelled.fire({}); });
+		_searchField->submits(
+		) | rpl::start_with_next([=] {
+			_searchSubmitted.fire({});
+		}, _searchField->lifetime());
+		_searchField->changes(
+		) | rpl::start_with_next([=] {
+			const auto was = _searchQuery.current();
+			const auto now = _searchField->getLastText();
+			if (_jumpToDate && was.isEmpty() != now.isEmpty()) {
+				updateControlsVisibility();
+			}
+			if (_chooseFromUser) {
+				auto switchToChooseFrom = SwitchToChooseFromQuery();
+				if (was != switchToChooseFrom
+					&& switchToChooseFrom.startsWith(was)
+					&& now == switchToChooseFrom) {
+					_chooseFromUserRequests.fire({});
+				}
+			}
+			_searchQuery = now;
+		}, _searchField->lifetime());
+	} else {
+		Assert(_searchField != nullptr);
+	}
+	_searchQuery = shown ? _searchField->getLastText() : QString();
+	if (animated == anim::type::normal) {
+		_searchShown.start(
+			[=] { slideAnimationCallback(); },
+			shown ? 0. : 1.,
+			shown ? 1. : 0.,
+			st::slideWrapDuration,
+			anim::easeOutCirc);
+	} else {
+		_searchShown.stop();
+		slideAnimationCallback();
+	}
+	if (shown) {
+		_searchField->setFocusFast();
+	}
+	return true;
+}
+
+void TopBarWidget::searchEnableJumpToDate(bool enable) {
+	if (!_searchMode) {
+		return;
+	} else if (!enable) {
+		_jumpToDate.destroy();
+	} else if (!_jumpToDate) {
+		_jumpToDate.create(
+			this,
+			object_ptr<Ui::IconButton>(this, st::dialogsCalendar));
+		_jumpToDate->toggle(
+			_searchField->getLastText().isEmpty(),
+			anim::type::instant);
+		_jumpToDate->entity()->clicks(
+		) | rpl::to_empty | rpl::start_to_stream(
+			_jumpToDateRequests,
+			_jumpToDate->lifetime());
+	}
+	updateControlsVisibility();
+	updateControlsGeometry();
+}
+
+void TopBarWidget::searchEnableChooseFromUser(bool enable, bool visible) {
+	if (!_searchMode) {
+		return;
+	} else if (!enable) {
+		_chooseFromUser.destroy();
+	} else if (!_chooseFromUser) {
+		_chooseFromUser.create(
+			this,
+			object_ptr<Ui::IconButton>(this, st::dialogsSearchFrom));
+		_chooseFromUser->toggle(visible, anim::type::instant);
+		_chooseFromUser->entity()->clicks(
+		) | rpl::to_empty | rpl::start_to_stream(
+			_chooseFromUserRequests,
+			_chooseFromUser->lifetime());
+	} else {
+		_chooseFromUser->toggle(visible, anim::type::normal);
+	}
+	auto additional = QMargins();
+	if (_chooseFromUser && _chooseFromUser->toggled()) {
+		additional.setRight(_chooseFromUser->width());
+	}
+	_searchField->setAdditionalMargins(additional);
+	updateControlsVisibility();
+	updateControlsGeometry();
+}
+
+bool TopBarWidget::searchSetFocus() {
+	if (!_searchMode) {
+		return false;
+	}
+	_searchField->setFocus();
+	return true;
+}
+
+bool TopBarWidget::searchMode() const {
+	return _searchMode;
+}
+
+bool TopBarWidget::searchHasFocus() const {
+	return _searchMode && _searchField->hasFocus();
+}
+
+rpl::producer<> TopBarWidget::searchCancelled() const {
+	return _searchCancelled.events();
+}
+
+rpl::producer<> TopBarWidget::searchSubmitted() const {
+	return _searchSubmitted.events();
+}
+
+rpl::producer<QString> TopBarWidget::searchQuery() const {
+	return _searchQuery.value();
+}
+
+QString TopBarWidget::searchQueryCurrent() const {
+	return _searchQuery.current();
+}
+
+void TopBarWidget::searchClear() {
+	if (_searchMode) {
+		_searchField->clear();
+	}
+}
+
+void TopBarWidget::searchSetText(const QString &query) {
+	if (_searchMode) {
+		_searchField->setText(query);
+	}
+}
+
 void TopBarWidget::toggleSelectedControls(bool shown) {
 	_selectedShown.start(
-		[this] { selectedShowCallback(); },
+		[this] { slideAnimationCallback(); },
 		shown ? 0. : 1.,
 		shown ? 1. : 0.,
 		st::slideWrapDuration,
 		anim::easeOutCirc);
 }
 
+void TopBarWidget::setGeometryWithNarrowRatio(
+		QRect geometry,
+		int narrowWidth,
+		float64 narrowRatio) {
+	if (_activeChat.section != Section::ChatsList) {
+		narrowRatio = 0.;
+		narrowWidth = 0;
+	}
+	const auto changed = (_narrowRatio != narrowRatio);
+	const auto started = (_narrowRatio == 0.) != (narrowRatio == 0.);
+	const auto finished = (_narrowRatio == 1.) != (narrowRatio == 1.);
+	const auto resized = (size() != geometry.size());
+	_narrowRatio = narrowRatio;
+	_narrowWidth = narrowWidth;
+	if (started || finished) {
+		updateControlsVisibility();
+	}
+	setGeometry(geometry);
+	if (changed && !resized) {
+		updateSearchVisibility();
+		updateControlsGeometry();
+	}
+}
+
 bool TopBarWidget::showSelectedActions() const {
 	return showSelectedState() && !_chooseForReportReason;
 }
 
-void TopBarWidget::selectedShowCallback() {
+void TopBarWidget::slideAnimationCallback() {
 	updateControlsGeometry();
 	update();
 }
@@ -1137,7 +1453,7 @@ void TopBarWidget::updateUnreadBadge() {
 			return QString();
 		}
 		return (counter > 999)
-			? qsl("..%1").arg(counter % 100, 2, 10, QChar('0'))
+			? u"..%1"_q.arg(counter % 100, 2, 10, QChar('0'))
 			: QString::number(counter);
 	}();
 	_unreadBadge->setText(text, !muted);
@@ -1157,9 +1473,65 @@ void TopBarWidget::updateInfoToggleActive() {
 	_infoToggle->setRippleColorOverride(rippleOverride);
 }
 
+void TopBarWidget::setupDragOnBackButton() {
+	_backLifetime.destroy();
+	if (_activeChat.section != Section::ChatsList) {
+		_back->setAcceptDrops(false);
+		return;
+	}
+	const auto lifetime = _backLifetime.make_state<rpl::lifetime>();
+	_back->setAcceptDrops(true);
+	_back->events(
+	) | rpl::filter([=](not_null<QEvent*> e) {
+		return e->type() == QEvent::DragEnter;
+	}) | rpl::start_with_next([=](not_null<QEvent*> e) {
+		using namespace Storage;
+		const auto d = static_cast<QDragEnterEvent*>(e.get());
+		const auto data = d->mimeData();
+		if (ComputeMimeDataState(data) == MimeDataState::None) {
+			return;
+		}
+		const auto timer = _backLifetime.make_state<base::Timer>([=] {
+			backClicked();
+		});
+		timer->callOnce(ChoosePeerByDragTimeout);
+		d->setDropAction(Qt::CopyAction);
+		d->accept();
+		_back->events(
+		) | rpl::filter([=](not_null<QEvent*> e) {
+			return e->type() == QEvent::DragMove
+				|| e->type() == QEvent::DragLeave;
+		}) | rpl::start_with_next([=](not_null<QEvent*> e) {
+			if (e->type() == QEvent::DragMove) {
+				timer->callOnce(ChoosePeerByDragTimeout);
+			} else if (e->type() == QEvent::DragLeave) {
+				timer->cancel();
+				lifetime->destroy();
+			}
+		}, *lifetime);
+	}, _backLifetime);
+}
+
+bool TopBarWidget::trackOnlineOf(not_null<PeerData*> user) const {
+	const auto peer = _activeChat.key.peer();
+	if (!peer || _activeChat.key.topic() || !user->isUser()) {
+		return false;
+	} else if (peer->isUser()) {
+		return (peer == user);
+	} else if (const auto chat = peer->asChat()) {
+		return chat->participants.contains(user->asUser());
+	} else if (const auto channel = peer->asMegagroup()) {
+		return channel->canViewMembers()
+			&& ranges::contains(
+				channel->mgInfo->lastParticipants,
+				not_null{ user->asUser() });
+	}
+	return false;
+}
+
 void TopBarWidget::updateOnlineDisplay() {
 	const auto peer = _activeChat.key.peer();
-	if (!peer) {
+	if (!peer || _activeChat.key.topic()) {
 		return;
 	}
 
@@ -1208,6 +1580,7 @@ void TopBarWidget::updateOnlineDisplay() {
 		}
 	} else if (const auto channel = peer->asChannel()) {
 		if (channel->isMegagroup()
+			&& channel->canViewMembers()
 			&& (channel->membersCount() > 0)
 			&& (channel->membersCount()
 				<= channel->session().serverConfig().chatSizeMax)) {
@@ -1259,9 +1632,9 @@ void TopBarWidget::updateOnlineDisplayTimer() {
 	}
 
 	const auto now = base::unixtime::now();
-	auto minTimeout = crl::time(86400);
+	auto minTimeout = 86400 * crl::time(1000);
 	const auto handleUser = [&](not_null<UserData*> user) {
-		auto hisTimeout = Data::OnlineChangeTimeout(user, now);
+		const auto hisTimeout = Data::OnlineChangeTimeout(user, now);
 		accumulate_min(minTimeout, hisTimeout);
 	};
 	if (const auto user = peer->asUser()) {
