@@ -10,9 +10,11 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lang/lang_keys.h"
 #include "history/history.h"
 #include "history/history_item.h"
-#include "history/view/history_view_element.h"
+#include "history/view/history_view_message.h"
 #include "history/view/history_view_cursor_state.h"
 #include "calls/calls_instance.h"
+#include "ui/chat/message_bubble.h"
+#include "ui/chat/chat_style.h"
 #include "ui/text/text_options.h"
 #include "ui/text/text_utilities.h"
 #include "ui/text/format_values.h"
@@ -21,6 +23,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/effects/ripple_animation.h"
 #include "ui/effects/fireworks_animation.h"
 #include "ui/toast/toast.h"
+#include "ui/painter.h"
 #include "data/data_media_types.h"
 #include "data/data_poll.h"
 #include "data/data_user.h"
@@ -28,8 +31,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/unixtime.h"
 #include "base/timer.h"
 #include "main/main_session.h"
-#include "layout.h" // FullSelection
 #include "apiwrap.h"
+#include "api/api_polls.h"
 #include "styles/style_chat.h"
 #include "styles/style_widgets.h"
 #include "styles/style_window.h"
@@ -105,10 +108,10 @@ void CountNicePercent(
 	PercentCounterItem ItemsStorage[PollData::kMaxOptions];
 	const auto items = gsl::make_span(ItemsStorage).subspan(0, count);
 	auto left = 100;
-	auto &&zipped = ranges::view::zip(
+	auto &&zipped = ranges::views::zip(
 		votes,
 		items,
-		ranges::view::ints(0, int(items.size())));
+		ranges::views::ints(0, int(items.size())));
 	for (auto &&[votes, item, index] : zipped) {
 		item.index = index;
 		item.percent = (votes * 100) / total;
@@ -178,6 +181,11 @@ struct Poll::CloseInformation {
 	Ui::Animations::Basic radial;
 };
 
+struct Poll::RecentVoter {
+	not_null<PeerData*> peer;
+	mutable Ui::PeerUserpicView userpic;
+};
+
 template <typename Callback>
 Poll::SendingAnimation::SendingAnimation(
 	const QByteArray &option,
@@ -248,9 +256,9 @@ QSize Poll::countOptimalSize() {
 			+ st::historyPollAnswerPadding.right());
 	}
 
-	const auto answersHeight = ranges::accumulate(ranges::view::all(
+	const auto answersHeight = ranges::accumulate(ranges::views::all(
 		_answers
-	) | ranges::view::transform([](const Answer &answer) {
+	) | ranges::views::transform([](const Answer &answer) {
 		return st::historyPollAnswerPadding.top()
 			+ answer.text.minHeight()
 			+ st::historyPollAnswerPadding.bottom();
@@ -280,7 +288,7 @@ bool Poll::showVotes() const {
 }
 
 bool Poll::canVote() const {
-	return !showVotes() && IsServerMsgId(_parent->data()->id);
+	return !showVotes() && _parent->data()->isRegular();
 }
 
 bool Poll::canSendVotes() const {
@@ -307,9 +315,6 @@ int Poll::countAnswerTop(
 	}
 	tshift += _question.countHeight(innerWidth) + st::historyPollSubtitleSkip;
 	tshift += st::msgDateFont->height + st::historyPollAnswersSkip;
-	auto &&answers = ranges::view::zip(
-		_answers,
-		ranges::view::ints(0, int(_answers.size())));
 	const auto i = ranges::find(
 		_answers,
 		&answer,
@@ -338,16 +343,14 @@ int Poll::countAnswerHeight(
 }
 
 QSize Poll::countCurrentSize(int newWidth) {
-	const auto paddings = st::msgPadding.left() + st::msgPadding.right();
-
 	accumulate_min(newWidth, maxWidth());
 	const auto innerWidth = newWidth
 		- st::msgPadding.left()
 		- st::msgPadding.right();
 
-	const auto answersHeight = ranges::accumulate(ranges::view::all(
+	const auto answersHeight = ranges::accumulate(ranges::views::all(
 		_answers
-	) | ranges::view::transform([&](const Answer &answer) {
+	) | ranges::views::transform([&](const Answer &answer) {
 		return countAnswerHeight(answer, innerWidth);
 	}), 0);
 
@@ -428,10 +431,10 @@ void Poll::checkQuizAnswered() {
 	}
 	if (i->correct) {
 		_fireworksAnimation = std::make_unique<Ui::FireworksAnimation>(
-			[=] { history()->owner().requestViewRepaint(_parent); });
+			[=] { repaint(); });
 	} else {
 		_wrongAnswerAnimation.start(
-			[=] { history()->owner().requestViewRepaint(_parent); },
+			[=] { repaint(); },
 			0.,
 			1.,
 			kRollDuration,
@@ -458,7 +461,7 @@ void Poll::solutionToggled(
 		if (animated == anim::type::instant
 			&& _solutionButtonAnimation.animating()) {
 			_solutionButtonAnimation.stop();
-			history()->owner().requestViewRepaint(_parent);
+			repaint();
 		}
 		return;
 	}
@@ -466,10 +469,10 @@ void Poll::solutionToggled(
 	history()->owner().notifyViewLayoutChange(_parent);
 	if (animated == anim::type::instant) {
 		_solutionButtonAnimation.stop();
-		history()->owner().requestViewRepaint(_parent);
+		repaint();
 	} else {
 		_solutionButtonAnimation.start(
-			[=] { history()->owner().requestViewRepaint(_parent); },
+			[=] { repaint(); },
 			visible ? 0. : 1.,
 			visible ? 1. : 0.,
 			st::fadeWrapDuration);
@@ -477,27 +480,27 @@ void Poll::solutionToggled(
 }
 
 void Poll::updateRecentVoters() {
-	auto &&sliced = ranges::view::all(
+	auto &&sliced = ranges::views::all(
 		_poll->recentVoters
-	) | ranges::view::take(kShowRecentVotersCount);
+	) | ranges::views::take(kShowRecentVotersCount);
 	const auto changed = !ranges::equal(
 		_recentVoters,
 		sliced,
 		ranges::equal_to(),
-		&RecentVoter::user);
+		&RecentVoter::peer);
 	if (changed) {
-		auto updated = ranges::view::all(
+		auto updated = ranges::views::all(
 			sliced
-		) | ranges::views::transform([](not_null<UserData*> user) {
-			return RecentVoter{ user };
+		) | ranges::views::transform([](not_null<PeerData*> peer) {
+			return RecentVoter{ peer };
 		}) | ranges::to_vector;
 		const auto has = hasHeavyPart();
 		if (has) {
 			for (auto &voter : updated) {
 				const auto i = ranges::find(
 					_recentVoters,
-					voter.user,
-					&RecentVoter::user);
+					voter.peer,
+					&RecentVoter::peer);
 				if (i != end(_recentVoters)) {
 					voter.userpic = std::move(i->userpic);
 				}
@@ -518,15 +521,15 @@ void Poll::updateAnswers() {
 		&Answer::option,
 		&PollAnswer::option);
 	if (!changed) {
-		auto &&answers = ranges::view::zip(_answers, _poll->answers);
+		auto &&answers = ranges::views::zip(_answers, _poll->answers);
 		for (auto &&[answer, original] : answers) {
 			answer.fillData(_poll, original);
 		}
 		return;
 	}
-	_answers = ranges::view::all(
+	_answers = ranges::views::all(
 		_poll->answers
-	) | ranges::view::transform([&](const PollAnswer &answer) {
+	) | ranges::views::transform([&](const PollAnswer &answer) {
 		auto result = Answer();
 		result.option = answer.option;
 		result.fillData(_poll, answer);
@@ -550,7 +553,7 @@ ClickHandlerPtr Poll::createAnswerClickHandler(
 	}
 	return std::make_shared<LambdaClickHandler>(crl::guard(this, [=] {
 		_votedFromHere = true;
-		history()->session().api().sendPollVotes(
+		history()->session().api().polls().sendVotes(
 			_parent->data()->fullId(),
 			{ option });
 	}));
@@ -565,7 +568,7 @@ void Poll::toggleMultiOption(const QByteArray &option) {
 		const auto selected = i->selected;
 		i->selected = !selected;
 		i->selectedAnimation.start(
-			[=] { history()->owner().requestViewRepaint(_parent); },
+			[=] { repaint(); },
 			selected ? 1. : 0.,
 			selected ? 0. : 1.,
 			st::defaultCheck.duration);
@@ -578,19 +581,19 @@ void Poll::toggleMultiOption(const QByteArray &option) {
 		} else {
 			_hasSelected = true;
 		}
-		history()->owner().requestViewRepaint(_parent);
+		repaint();
 	}
 }
 
 void Poll::sendMultiOptions() {
-	auto chosen = _answers | ranges::view::filter(
+	auto chosen = _answers | ranges::views::filter(
 		&Answer::selected
-	) | ranges::view::transform(
+	) | ranges::views::transform(
 		&Answer::option
 	) | ranges::to_vector;
 	if (!chosen.empty()) {
 		_votedFromHere = true;
-		history()->session().api().sendPollVotes(
+		history()->session().api().polls().sendVotes(
 			_parent->data()->fullId(),
 			std::move(chosen));
 	}
@@ -697,9 +700,9 @@ void Poll::updateAnswerVotes() {
 	int VotesStorage[kMaxCount] = { 0 };
 
 	ranges::copy(
-		ranges::view::all(
+		ranges::views::all(
 			_poll->answers
-		) | ranges::view::transform(&PollAnswer::votes),
+		) | ranges::views::transform(&PollAnswer::votes),
 		ranges::begin(VotesStorage));
 
 	CountNicePercent(
@@ -707,7 +710,7 @@ void Poll::updateAnswerVotes() {
 		totalVotes,
 		gsl::make_span(PercentsStorage).subspan(0, count));
 
-	auto &&answers = ranges::view::zip(
+	auto &&answers = ranges::views::zip(
 		_answers,
 		_poll->answers,
 		PercentsStorage);
@@ -720,17 +723,16 @@ void Poll::updateAnswerVotes() {
 	}
 }
 
-void Poll::draw(Painter &p, const QRect &r, TextSelection selection, crl::time ms) const {
+void Poll::draw(Painter &p, const PaintContext &context) const {
 	if (width() < st::msgPadding.left() + st::msgPadding.right() + 1) return;
-	auto paintx = 0, painty = 0, paintw = width(), painth = height();
+	auto paintw = width();
 
 	checkSendingAnimation();
-	_poll->checkResultsReload(_parent->data(), ms);
-
-	const auto outbg = _parent->hasOutLayout();
-	const auto selected = (selection == FullSelection);
-	const auto &regular = selected ? (outbg ? st::msgOutDateFgSelected : st::msgInDateFgSelected) : (outbg ? st::msgOutDateFg : st::msgInDateFg);
+	if (_poll->checkResultsReload(context.now)) {
+		history()->session().api().polls().reloadResults(_parent->data());
+	}
 
+	const auto stm = context.messageStyle();
 	const auto padding = st::msgPadding;
 	auto tshift = st::historyPollQuestionTop;
 	if (!isBubbleTop()) {
@@ -738,15 +740,15 @@ void Poll::draw(Painter &p, const QRect &r, TextSelection selection, crl::time m
 	}
 	paintw -= padding.left() + padding.right();
 
-	p.setPen(outbg ? st::webPageTitleOutFg : st::webPageTitleInFg);
-	_question.drawLeft(p, padding.left(), tshift, paintw, width(), style::al_left, 0, -1, selection);
+	p.setPen(stm->historyTextFg);
+	_question.drawLeft(p, padding.left(), tshift, paintw, width(), style::al_left, 0, -1, context.selection);
 	tshift += _question.countHeight(paintw) + st::historyPollSubtitleSkip;
 
-	p.setPen(regular);
+	p.setPen(stm->msgDateFg);
 	_subtitle.drawLeftElided(p, padding.left(), tshift, paintw, width());
-	paintRecentVoters(p, padding.left() + _subtitle.maxWidth(), tshift, selection);
-	paintCloseByTimer(p, padding.left() + paintw, tshift, selection);
-	paintShowSolution(p, padding.left() + paintw, tshift, selection);
+	paintRecentVoters(p, padding.left() + _subtitle.maxWidth(), tshift, context);
+	paintCloseByTimer(p, padding.left() + paintw, tshift, context);
+	paintShowSolution(p, padding.left() + paintw, tshift, context);
 	tshift += st::msgDateFont->height + st::historyPollAnswersSkip;
 
 	const auto progress = _answersAnimation
@@ -756,10 +758,10 @@ void Poll::draw(Painter &p, const QRect &r, TextSelection selection, crl::time m
 		resetAnswersAnimation();
 	}
 
-	auto &&answers = ranges::view::zip(
+	auto &&answers = ranges::views::zip(
 		_answers,
-		ranges::view::ints(0, int(_answers.size())));
-	for (const auto &[answer, index] : answers) {
+		ranges::views::ints(0, int(_answers.size())));
+	for (const auto [answer, index] : answers) {
 		const auto animation = _answersAnimation
 			? &_answersAnimation->data[index]
 			: nullptr;
@@ -778,14 +780,14 @@ void Poll::draw(Painter &p, const QRect &r, TextSelection selection, crl::time m
 			tshift,
 			paintw,
 			width(),
-			selection);
+			context);
 		tshift += height;
 	}
 	if (!inlineFooter()) {
-		paintBottom(p, padding.left(), tshift, paintw, selection);
+		paintBottom(p, padding.left(), tshift, paintw, context);
 	} else if (!_totalVotesLabel.isEmpty()) {
 		tshift += st::msgPadding.bottom();
-		paintInlineFooter(p, padding.left(), tshift, paintw, selection);
+		paintInlineFooter(p, padding.left(), tshift, paintw, context);
 	}
 }
 
@@ -794,18 +796,16 @@ void Poll::paintInlineFooter(
 		int left,
 		int top,
 		int paintw,
-		TextSelection selection) const {
-	const auto selected = (selection == FullSelection);
-	const auto outbg = _parent->hasOutLayout();
-	const auto &regular = selected ? (outbg ? st::msgOutDateFgSelected : st::msgInDateFgSelected) : (outbg ? st::msgOutDateFg : st::msgInDateFg);
-	p.setPen(regular);
+		const PaintContext &context) const {
+	const auto stm = context.messageStyle();
+	p.setPen(stm->msgDateFg);
 	_totalVotesLabel.drawLeftElided(
 		p,
 		left,
 		top,
 		std::min(
 			_totalVotesLabel.maxWidth(),
-			paintw - _parent->infoWidth()),
+			paintw - _parent->bottomInfoFirstLineWidth()),
 		width());
 }
 
@@ -814,13 +814,13 @@ void Poll::paintBottom(
 		int left,
 		int top,
 		int paintw,
-		TextSelection selection) const {
-	const auto stringtop = top + st::msgPadding.bottom() + st::historyPollBottomButtonTop;
-	const auto selected = (selection == FullSelection);
-	const auto outbg = _parent->hasOutLayout();
-	const auto &regular = selected ? (outbg ? st::msgOutDateFgSelected : st::msgInDateFgSelected) : (outbg ? st::msgOutDateFg : st::msgInDateFg);
+		const PaintContext &context) const {
+	const auto stringtop = top
+		+ st::msgPadding.bottom()
+		+ st::historyPollBottomButtonTop;
+	const auto stm = context.messageStyle();
 	if (showVotersCount()) {
-		p.setPen(regular);
+		p.setPen(stm->msgDateFg);
 		_totalVotesLabel.draw(p, left, stringtop, paintw, style::al_top);
 	} else {
 		const auto link = showVotes()
@@ -831,23 +831,29 @@ void Poll::paintBottom(
 		if (_linkRipple) {
 			const auto linkHeight = bottomButtonHeight();
 			p.setOpacity(st::historyPollRippleOpacity);
-			_linkRipple->paint(p, left - st::msgPadding.left(), height() - linkHeight, width());
+			_linkRipple->paint(
+				p,
+				left - st::msgPadding.left(),
+				height() - linkHeight,
+				width(),
+				&stm->msgWaveformInactive->c);
 			if (_linkRipple->empty()) {
 				_linkRipple.reset();
 			}
 			p.setOpacity(1.);
 		}
 		p.setFont(st::semiboldFont);
-		if (!link) {
-			p.setPen(regular);
-		} else {
-			p.setPen(outbg ? (selected ? st::msgFileThumbLinkOutFgSelected : st::msgFileThumbLinkOutFg) : (selected ? st::msgFileThumbLinkInFgSelected : st::msgFileThumbLinkInFg));
-		}
+		p.setPen(link ? stm->msgFileThumbLinkFg : stm->msgDateFg);
 		const auto string = showVotes()
 			? tr::lng_polls_view_results(tr::now, Ui::Text::Upper)
 			: tr::lng_polls_submit_votes(tr::now, Ui::Text::Upper);
 		const auto stringw = st::semiboldFont->width(string);
-		p.drawTextLeft(left + (paintw - stringw) / 2, stringtop, width(), string, stringw);
+		p.drawTextLeft(
+			left + (paintw - stringw) / 2,
+			stringtop,
+			width(),
+			string,
+			stringw);
 	}
 }
 
@@ -861,7 +867,7 @@ void Poll::resetAnswersAnimation() const {
 
 void Poll::radialAnimationCallback() const {
 	if (!anim::Disabled()) {
-		history()->owner().requestViewRepaint(_parent);
+		repaint();
 	}
 }
 
@@ -869,7 +875,7 @@ void Poll::paintRecentVoters(
 		Painter &p,
 		int left,
 		int top,
-		TextSelection selection) const {
+		const PaintContext &context) const {
 	const auto count = int(_recentVoters.size());
 	if (!count) {
 		return;
@@ -879,24 +885,37 @@ void Poll::paintRecentVoters(
 		+ (count - 1) * st::historyPollRecentVoterSkip;
 	auto y = top;
 	const auto size = st::historyPollRecentVoterSize;
-	const auto outbg = _parent->hasOutLayout();
-	const auto selected = (selection == FullSelection);
-	auto pen = (selected
-		? (outbg ? st::msgOutBgSelected : st::msgInBgSelected)
-		: (outbg ? st::msgOutBg : st::msgInBg))->p;
+	const auto stm = context.messageStyle();
+	auto pen = stm->msgBg->p;
 	pen.setWidth(st::lineWidth);
 
 	auto created = false;
 	for (auto &recent : _recentVoters) {
-		const auto was = (recent.userpic != nullptr);
-		recent.user->paintUserpic(p, recent.userpic, x, y, size);
-		if (!was && recent.userpic) {
+		const auto was = !recent.userpic.null();
+		recent.peer->paintUserpic(p, recent.userpic, x, y, size);
+		if (!was && !recent.userpic.null()) {
 			created = true;
 		}
-		p.setPen(pen);
-		p.setBrush(Qt::NoBrush);
-		PainterHighQualityEnabler hq(p);
-		p.drawEllipse(x, y, size, size);
+		const auto paintContent = [&](QPainter &p) {
+			p.setPen(pen);
+			p.setBrush(Qt::NoBrush);
+			PainterHighQualityEnabler hq(p);
+			p.drawEllipse(x, y, size, size);
+		};
+		if (usesBubblePattern(context)) {
+			const auto add = st::lineWidth * 2;
+			const auto target = QRect(x, y, size, size).marginsAdded(
+				{ add, add, add, add });
+			Ui::PaintPatternBubblePart(
+				p,
+				context.viewport,
+				context.bubblesPattern->pixmap,
+				target,
+				paintContent,
+				_userpicCircleCache);
+		} else {
+			paintContent(p);
+		}
 		x -= st::historyPollRecentVoterSkip;
 	}
 	if (created) {
@@ -908,7 +927,7 @@ void Poll::paintCloseByTimer(
 		Painter &p,
 		int right,
 		int top,
-		TextSelection selection) const {
+		const PaintContext &context) const {
 	if (!canVote() || _poll->closeDate <= 0 || _poll->closePeriod <= 0) {
 		_close = nullptr;
 		return;
@@ -917,7 +936,7 @@ void Poll::paintCloseByTimer(
 		_close = std::make_unique<CloseInformation>(
 			_poll->closeDate,
 			_poll->closePeriod,
-			[=] { history()->owner().requestViewRepaint(_parent); });
+			[=] { repaint(); });
 	}
 	const auto now = crl::now();
 	const auto left = std::max(_close->finish - now, crl::time(0));
@@ -927,7 +946,7 @@ void Poll::paintCloseByTimer(
 	} else if (left < radial && !anim::Disabled()) {
 		if (!_close->radial.animating()) {
 			_close->radial.init([=] {
-				history()->owner().requestViewRepaint(_parent);
+				repaint();
 			});
 			_close->radial.start();
 		}
@@ -935,22 +954,16 @@ void Poll::paintCloseByTimer(
 		_close->radial.stop();
 	}
 	const auto time = Ui::FormatDurationText(int(std::ceil(left / 1000.)));
-	const auto outbg = _parent->hasOutLayout();
-	const auto selected = (selection == FullSelection);
-	const auto &icon = selected
-		? (outbg
-			? st::historyQuizTimerOutSelected
-			: st::historyQuizTimerInSelected)
-		: (outbg ? st::historyQuizTimerOut : st::historyQuizTimerIn);
+	const auto st = context.st;
+	const auto stm = context.messageStyle();
+	const auto &icon = stm->historyQuizTimer;
 	const auto x = right - icon.width();
 	const auto y = top
 		+ (st::normalFont->height - icon.height()) / 2
 		- st::lineWidth;
 	const auto &regular = (left < kCriticalCloseDuration)
-		? st::boxTextFgError
-		: selected
-		? (outbg ? st::msgOutDateFgSelected : st::msgInDateFgSelected)
-		: (outbg ? st::msgOutDateFg : st::msgInDateFg);
+		? st->boxTextFgError()
+		: stm->msgDateFg;
 	p.setPen(regular);
 	const auto timeWidth = st::normalFont->width(time);
 	p.drawTextLeft(x - timeWidth, top, width(), time, timeWidth);
@@ -958,8 +971,8 @@ void Poll::paintCloseByTimer(
 		auto hq = PainterHighQualityEnabler(p);
 		const auto part = std::max(
 			left / float64(radial),
-			1. / FullArcLength);
-		const auto length = int(std::round(FullArcLength * part));
+			1. / arc::kFullLength);
+		const auto length = int(base::SafeRound(arc::kFullLength * part));
 		auto pen = regular->p;
 		pen.setWidth(st::historyPollRadio.thickness);
 		pen.setCapStyle(Qt::RoundCap);
@@ -967,7 +980,7 @@ void Poll::paintCloseByTimer(
 		const auto size = icon.width() / 2;
 		const auto left = (x + (icon.width() - size) / 2);
 		const auto top = (y + (icon.height() - size) / 2) + st::lineWidth;
-		p.drawArc(left, top, size, size, (FullArcLength / 4), length);
+		p.drawArc(left, top, size, size, (arc::kFullLength / 4), length);
 	} else {
 		icon.paint(p, x, y, width());
 	}
@@ -982,7 +995,7 @@ void Poll::paintShowSolution(
 		Painter &p,
 		int right,
 		int top,
-		TextSelection selection) const {
+		const PaintContext &context) const {
 	const auto shown = _solutionButtonAnimation.value(
 		_solutionButtonVisible ? 1. : 0.);
 	if (!shown) {
@@ -992,12 +1005,8 @@ void Poll::paintShowSolution(
 		_showSolutionLink = std::make_shared<LambdaClickHandler>(
 			crl::guard(this, [=] { showSolution(); }));
 	}
-	const auto outbg = _parent->hasOutLayout();
-	const auto &icon = (selection == FullSelection)
-		? (outbg
-			? st::historyQuizExplainOutSelected
-			: st::historyQuizExplainInSelected)
-		: (outbg ? st::historyQuizExplainOut : st::historyQuizExplainIn);
+	const auto stm = context.messageStyle();
+	const auto &icon = stm->historyQuizExplain;
 	const auto x = right - icon.width();
 	const auto y = top + (st::normalFont->height - icon.height()) / 2;
 	if (shown == 1.) {
@@ -1020,9 +1029,9 @@ int Poll::paintAnswer(
 		int top,
 		int width,
 		int outerWidth,
-		TextSelection selection) const {
+		const PaintContext &context) const {
 	const auto height = countAnswerHeight(answer, width);
-	const auto outbg = _parent->hasOutLayout();
+	const auto stm = context.messageStyle();
 	const auto aleft = left + st::historyPollAnswerPadding.left();
 	const auto awidth = width
 		- st::historyPollAnswerPadding.left()
@@ -1030,7 +1039,12 @@ int Poll::paintAnswer(
 
 	if (answer.ripple) {
 		p.setOpacity(st::historyPollRippleOpacity);
-		answer.ripple->paint(p, left - st::msgPadding.left(), top, outerWidth);
+		answer.ripple->paint(
+			p,
+			left - st::msgPadding.left(),
+			top,
+			outerWidth,
+			&stm->msgWaveformInactive->c);
 		if (answer.ripple->empty()) {
 			answer.ripple.reset();
 		}
@@ -1041,11 +1055,11 @@ int Poll::paintAnswer(
 		const auto opacity = animation->opacity.current();
 		if (opacity < 1.) {
 			p.setOpacity(1. - opacity);
-			paintRadio(p, answer, left, top, selection);
+			paintRadio(p, answer, left, top, context);
 		}
 		if (opacity > 0.) {
 			const auto percent = QString::number(
-				int(std::round(animation->percent.current()))) + '%';
+				int(base::SafeRound(animation->percent.current()))) + '%';
 			const auto percentWidth = st::historyPollPercentFont->width(
 				percent);
 			p.setOpacity(opacity);
@@ -1056,7 +1070,7 @@ int Poll::paintAnswer(
 				left,
 				top,
 				outerWidth,
-				selection);
+				context);
 			p.setOpacity(sqrt(opacity));
 			paintFilling(
 				p,
@@ -1067,11 +1081,11 @@ int Poll::paintAnswer(
 				top,
 				width,
 				height,
-				selection);
+				context);
 			p.setOpacity(1.);
 		}
 	} else if (!showVotes()) {
-		paintRadio(p, answer, left, top, selection);
+		paintRadio(p, answer, left, top, context);
 	} else {
 		paintPercent(
 			p,
@@ -1080,7 +1094,7 @@ int Poll::paintAnswer(
 			left,
 			top,
 			outerWidth,
-			selection);
+			context);
 		paintFilling(
 			p,
 			answer.chosen,
@@ -1090,11 +1104,11 @@ int Poll::paintAnswer(
 			top,
 			width,
 			height,
-			selection);
+			context);
 	}
 
 	top += st::historyPollAnswerPadding.top();
-	p.setPen(outbg ? st::webPageDescriptionOutFg : st::webPageDescriptionInFg);
+	p.setPen(stm->historyTextFg);
 	answer.text.drawLeft(p, aleft, top, awidth, outerWidth);
 
 	return height;
@@ -1105,16 +1119,15 @@ void Poll::paintRadio(
 		const Answer &answer,
 		int left,
 		int top,
-		TextSelection selection) const {
+		const PaintContext &context) const {
 	top += st::historyPollAnswerPadding.top();
 
-	const auto outbg = _parent->hasOutLayout();
-	const auto selected = (selection == FullSelection);
+	const auto stm = context.messageStyle();
 
 	PainterHighQualityEnabler hq(p);
-	const auto &st = st::historyPollRadio;
+	const auto &radio = st::historyPollRadio;
 	const auto over = ClickHandler::showAsActive(answer.handler);
-	const auto &regular = selected ? (outbg ? st::msgOutDateFgSelected : st::msgInDateFgSelected) : (outbg ? st::msgOutDateFg : st::msgInDateFg);
+	const auto &regular = stm->msgDateFg;
 
 	const auto checkmark = answer.selectedAnimation.value(answer.selected ? 1. : 0.);
 
@@ -1124,15 +1137,15 @@ void Poll::paintRadio(
 		p.setOpacity(o * (over ? st::historyPollRadioOpacityOver : st::historyPollRadioOpacity));
 	}
 
-	const auto rect = QRectF(left, top, st.diameter, st.diameter).marginsRemoved(QMarginsF(st.thickness / 2., st.thickness / 2., st.thickness / 2., st.thickness / 2.));
+	const auto rect = QRectF(left, top, radio.diameter, radio.diameter).marginsRemoved(QMarginsF(radio.thickness / 2., radio.thickness / 2., radio.thickness / 2., radio.thickness / 2.));
 	if (_sendingAnimation && _sendingAnimation->option == answer.option) {
-		const auto &active = selected ? (outbg ? st::msgOutServiceFgSelected : st::msgInServiceFgSelected) : (outbg ? st::msgOutServiceFg : st::msgInServiceFg);
+		const auto &active = stm->msgServiceFg;
 		if (anim::Disabled()) {
-			anim::DrawStaticLoading(p, rect, st.thickness, active);
+			anim::DrawStaticLoading(p, rect, radio.thickness, active);
 		} else {
 			const auto state = _sendingAnimation->animation.computeState();
 			auto pen = anim::pen(regular, active, state.shown);
-			pen.setWidth(st.thickness);
+			pen.setWidth(radio.thickness);
 			pen.setCapStyle(Qt::RoundCap);
 			p.setPen(pen);
 			p.drawArc(
@@ -1143,21 +1156,21 @@ void Poll::paintRadio(
 	} else {
 		if (checkmark < 1.) {
 			auto pen = regular->p;
-			pen.setWidth(st.thickness);
+			pen.setWidth(radio.thickness);
 			p.setPen(pen);
 			p.drawEllipse(rect);
 		}
 		if (checkmark > 0.) {
-			const auto removeFull = (st.diameter / 2 - st.thickness);
+			const auto removeFull = (radio.diameter / 2 - radio.thickness);
 			const auto removeNow = removeFull * (1. - checkmark);
-			const auto color = outbg ? (selected ? st::msgFileThumbLinkOutFgSelected : st::msgFileThumbLinkOutFg) : (selected ? st::msgFileThumbLinkInFgSelected : st::msgFileThumbLinkInFg);
+			const auto color = stm->msgFileThumbLinkFg;
 			auto pen = color->p;
-			pen.setWidth(st.thickness);
+			pen.setWidth(radio.thickness);
 			p.setPen(pen);
 			p.setBrush(color);
 			p.drawEllipse(rect.marginsRemoved({ removeNow, removeNow, removeNow, removeNow }));
-			const auto &icon = outbg ? (selected ? st::historyPollOutChosenSelected : st::historyPollOutChosen) : (selected ? st::historyPollInChosenSelected : st::historyPollInChosen);
-			icon.paint(p, left + (st.diameter - icon.width()) / 2, top + (st.diameter - icon.height()) / 2, width());
+			const auto &icon = stm->historyPollChosen;
+			icon.paint(p, left + (radio.diameter - icon.width()) / 2, top + (radio.diameter - icon.height()) / 2, width());
 		}
 	}
 
@@ -1171,15 +1184,14 @@ void Poll::paintPercent(
 		int left,
 		int top,
 		int outerWidth,
-		TextSelection selection) const {
-	const auto outbg = _parent->hasOutLayout();
-	const auto selected = (selection == FullSelection);
+		const PaintContext &context) const {
+	const auto stm = context.messageStyle();
 	const auto aleft = left + st::historyPollAnswerPadding.left();
 
 	top += st::historyPollAnswerPadding.top();
 
 	p.setFont(st::historyPollPercentFont);
-	p.setPen(outbg ? st::webPageDescriptionOutFg : st::webPageDescriptionInFg);
+	p.setPen(stm->historyTextFg);
 	const auto pleft = aleft - percentWidth - st::historyPollPercentSkip;
 	p.drawTextLeft(pleft, top + st::historyPollPercentTop, outerWidth, percent, percentWidth);
 }
@@ -1193,10 +1205,10 @@ void Poll::paintFilling(
 		int top,
 		int width,
 		int height,
-		TextSelection selection) const {
+		const PaintContext &context) const {
 	const auto bottom = top + height;
-	const auto outbg = _parent->hasOutLayout();
-	const auto selected = (selection == FullSelection);
+	const auto st = context.st;
+	const auto stm = context.messageStyle();
 	const auto aleft = left + st::historyPollAnswerPadding.left();
 	const auto awidth = width
 		- st::historyPollAnswerPadding.left()
@@ -1204,32 +1216,67 @@ void Poll::paintFilling(
 
 	top += st::historyPollAnswerPadding.top();
 
-	PainterHighQualityEnabler hq(p);
-	p.setPen(Qt::NoPen);
 	const auto thickness = st::historyPollFillingHeight;
 	const auto max = awidth - st::historyPollFillingRight;
 	const auto size = anim::interpolate(st::historyPollFillingMin, max, filling);
 	const auto radius = st::historyPollFillingRadius;
 	const auto ftop = bottom - st::historyPollFillingBottom - thickness;
 
-	if (chosen && !correct) {
-		p.setBrush(st::boxTextFgError);
-	} else if (chosen && correct && _poll->quiz() && !outbg) {
-		p.setBrush(st::boxTextFgGood);
-	} else {
-		const auto bar = outbg ? (selected ? st::msgWaveformOutActiveSelected : st::msgWaveformOutActive) : (selected ? st::msgWaveformInActiveSelected : st::msgWaveformInActive);
-		p.setBrush(bar);
-	}
+	enum class Style {
+		Incorrect,
+		Correct,
+		Default,
+	};
+	const auto style = [&] {
+		if (chosen && !correct) {
+			return Style::Incorrect;
+		} else if (chosen && correct && _poll->quiz() && !context.outbg) {
+			return Style::Correct;
+		} else {
+			return Style::Default;
+		}
+	}();
 	auto barleft = aleft;
 	auto barwidth = size;
+	const auto &color = (style == Style::Incorrect)
+		? st->boxTextFgError()
+		: (style == Style::Correct)
+		? st->boxTextFgGood()
+		: stm->msgFileBg;
+	p.setPen(Qt::NoPen);
+	p.setBrush(color);
+	PainterHighQualityEnabler hq(p);
 	if (chosen || correct) {
-		const auto &icon = (chosen && !correct)
-			? st::historyPollChoiceWrong
-			: st::historyPollChoiceRight;
+		const auto &icon = (style == Style::Incorrect)
+			? st->historyPollChoiceWrong()
+			: (style == Style::Correct)
+			? st->historyPollChoiceRight()
+			: stm->historyPollChoiceRight;
 		const auto cleft = aleft - st::historyPollPercentSkip - icon.width();
 		const auto ctop = ftop - (icon.height() - thickness) / 2;
 		p.drawEllipse(cleft, ctop, icon.width(), icon.height());
-		icon.paint(p, cleft, ctop, width);
+
+		const auto paintContent = [&](QPainter &p) {
+			icon.paint(p, cleft, ctop, width);
+		};
+		if (style == Style::Default && usesBubblePattern(context)) {
+			const auto add = st::lineWidth * 2;
+			const auto target = QRect(
+				cleft,
+				ctop,
+				icon.width(),
+				icon.height()
+			).marginsAdded({ add, add, add, add });
+			Ui::PaintPatternBubblePart(
+				p,
+				context.viewport,
+				context.bubblesPattern->pixmap,
+				target,
+				paintContent,
+				_fillingIconCache);
+		} else {
+			paintContent(p);
+		}
 		//barleft += icon.width() - radius;
 		//barwidth -= icon.width() - radius;
 	}
@@ -1292,7 +1339,7 @@ void Poll::startAnswersAnimation() const {
 	}
 
 	const auto show = showVotes();
-	auto &&both = ranges::view::zip(_answers, _answersAnimation->data);
+	auto &&both = ranges::views::zip(_answers, _answersAnimation->data);
 	for (auto &&[answer, data] : both) {
 		data.percent.start(show ? float64(answer.votesPercent) : 0.);
 		data.filling.start(show ? answer.filling : 0.);
@@ -1301,12 +1348,26 @@ void Poll::startAnswersAnimation() const {
 		data.correct = data.correct || answer.correct;
 	}
 	_answersAnimation->progress.start(
-		[=] { history()->owner().requestViewRepaint(_parent); },
+		[=] { repaint(); },
 		0.,
 		1.,
 		st::historyPollDuration);
 }
 
+TextSelection Poll::adjustSelection(
+		TextSelection selection,
+		TextSelectType type) const {
+	return _question.adjustSelection(selection, type);
+}
+
+uint16 Poll::fullSelectionLength() const {
+	return _question.length();
+}
+
+TextForMimeData Poll::selectedText(TextSelection selection) const {
+	return _question.toTextForMimeData(selection);
+}
+
 TextState Poll::textState(QPoint point, StateRequest request) const {
 	auto result = TextState(_parent);
 	if (!_poll->sendingVotes.empty()) {
@@ -1323,15 +1384,20 @@ TextState Poll::textState(QPoint point, StateRequest request) const {
 	}
 	paintw -= padding.left() + padding.right();
 
-	tshift += _question.countHeight(paintw) + st::historyPollSubtitleSkip;
+	const auto questionH = _question.countHeight(paintw);
+	if (QRect(padding.left(), tshift, paintw, questionH).contains(point)) {
+		result = TextState(_parent, _question.getState(
+			point - QPoint(padding.left(), tshift),
+			paintw,
+			request.forText()));
+		return result;
+	}
+	tshift += questionH + st::historyPollSubtitleSkip;
 	if (inShowSolution(point, padding.left() + paintw, tshift)) {
 		result.link = _showSolutionLink;
 		return result;
 	}
 	tshift += st::msgDateFont->height + st::historyPollAnswersSkip;
-	const auto awidth = paintw
-		- st::historyPollAnswerPadding.left()
-		- st::historyPollAnswerPadding.right();
 	for (const auto &answer : _answers) {
 		const auto height = countAnswerHeight(answer, paintw);
 		if (point.y() >= tshift && point.y() < tshift + height) {
@@ -1384,7 +1450,6 @@ auto Poll::bubbleRoll() const -> BubbleRoll {
 	if (!_wrongAnswerAnimated) {
 		return BubbleRoll();
 	}
-	const auto rotateFull = value * kRotateSegments;
 	const auto progress = [](float64 full) {
 		const auto lower = std::floor(full);
 		const auto shift = (full - lower);
@@ -1439,13 +1504,13 @@ void Poll::clickHandlerPressedChanged(
 
 void Poll::unloadHeavyPart() {
 	for (auto &recent : _recentVoters) {
-		recent.userpic = nullptr;
+		recent.userpic = {};
 	}
 }
 
 bool Poll::hasHeavyPart() const {
 	for (auto &recent : _recentVoters) {
-		if (recent.userpic) {
+		if (!recent.userpic.null()) {
 			return true;
 		}
 	}
@@ -1459,15 +1524,13 @@ void Poll::toggleRipple(Answer &answer, bool pressed) {
 			- st::msgPadding.left()
 			- st::msgPadding.right();
 		if (!answer.ripple) {
-			auto mask = Ui::RippleAnimation::rectMask(QSize(
+			auto mask = Ui::RippleAnimation::RectMask(QSize(
 				outerWidth,
 				countAnswerHeight(answer, innerWidth)));
 			answer.ripple = std::make_unique<Ui::RippleAnimation>(
-				(_parent->hasOutLayout()
-					? st::historyPollRippleOut
-					: st::historyPollRippleIn),
+				st::defaultRippleAnimation,
 				std::move(mask),
-				[=] { history()->owner().requestViewRepaint(_parent); });
+				[=] { repaint(); });
 		}
 		const auto top = countAnswerTop(answer, innerWidth);
 		answer.ripple->add(_lastLinkPoint - QPoint(0, top));
@@ -1510,31 +1573,20 @@ void Poll::toggleLinkRipple(bool pressed) {
 		const auto linkWidth = width();
 		const auto linkHeight = bottomButtonHeight();
 		if (!_linkRipple) {
-			const auto drawMask = [&](QPainter &p) {
-				const auto radius = st::historyMessageRadius;
-				p.drawRoundedRect(
-					0,
-					0,
-					linkWidth,
-					linkHeight,
-					radius,
-					radius);
-				p.fillRect(0, 0, linkWidth, radius * 2, Qt::white);
-			};
 			auto mask = isRoundedInBubbleBottom()
-				? Ui::RippleAnimation::maskByDrawer(
-					QSize(linkWidth, linkHeight),
-					false,
-					drawMask)
-				: Ui::RippleAnimation::rectMask({ linkWidth, linkHeight });
+				? static_cast<Message*>(_parent.get())->bottomRippleMask(
+					bottomButtonHeight())
+				: BottomRippleMask{
+					Ui::RippleAnimation::RectMask({ linkWidth, linkHeight }),
+				};
 			_linkRipple = std::make_unique<Ui::RippleAnimation>(
-				(_parent->hasOutLayout()
-					? st::historyPollRippleOut
-					: st::historyPollRippleIn),
-				std::move(mask),
-				[=] { history()->owner().requestViewRepaint(_parent); });
+				st::defaultRippleAnimation,
+				std::move(mask.image),
+				[=] { repaint(); });
+			_linkRippleShift = mask.shift;
 		}
-		_linkRipple->add(_lastLinkPoint - QPoint(0, height() - linkHeight));
+		_linkRipple->add(_lastLinkPoint
+			+ QPoint(_linkRippleShift, linkHeight - height()));
 	} else if (_linkRipple) {
 		_linkRipple->lastStop();
 	}
