@@ -26,8 +26,9 @@ public:
 	int top() override;
 	QSize size() override;
 	QString title() override;
-	QString subtitle() override;
+	TextWithEntities subtitle() override;
 	QString button() override;
+	int buttonSkip() override;
 	void draw(
 		Painter &p,
 		const PaintContext &context,
