@@ -7,9 +7,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "media/view/media_view_overlay_opengl.h"
 
+#include "data/data_peer_values.h" // AmPremiumValue.
 #include "ui/gl/gl_shader.h"
 #include "ui/painter.h"
+#include "media/stories/media_stories_view.h"
 #include "media/streaming/media_streaming_common.h"
+#include "platform/platform_overlay_widget.h"
 #include "base/platform/base_platform_info.h"
 #include "core/crash_reports.h"
 #include "styles/style_media_view.h"
@@ -19,7 +22,8 @@ namespace {
 
 using namespace Ui::GL;
 
-constexpr auto kRadialLoadingOffset = 4;
+constexpr auto kNotchOffset = 4;
+constexpr auto kRadialLoadingOffset = kNotchOffset + 4;
 constexpr auto kThemePreviewOffset = kRadialLoadingOffset + 4;
 constexpr auto kDocumentBubbleOffset = kThemePreviewOffset + 4;
 constexpr auto kSaveMsgOffset = kDocumentBubbleOffset + 4;
@@ -27,7 +31,36 @@ constexpr auto kFooterOffset = kSaveMsgOffset + 4;
 constexpr auto kCaptionOffset = kFooterOffset + 4;
 constexpr auto kGroupThumbsOffset = kCaptionOffset + 4;
 constexpr auto kControlsOffset = kGroupThumbsOffset + 4;
-constexpr auto kControlValues = 2 * 4 + 4 * 4;
+constexpr auto kControlValues = 4 * 4 + 4 * 4; // over + icon
+
+[[nodiscard]] ShaderPart FragmentApplyControlsFade() {
+	return {
+		.header = R"(
+uniform sampler2D f_texture;
+uniform vec4 shadowTopRect;
+uniform vec4 shadowBottomSkipOpacityFullFade;
+)",
+		.body = R"(
+	float topHeight = shadowTopRect.w;
+	float bottomHeight = shadowBottomSkipOpacityFullFade.x;
+	float bottomSkip = shadowBottomSkipOpacityFullFade.y;
+	float opacity = shadowBottomSkipOpacityFullFade.z;
+	float fullFade = shadowBottomSkipOpacityFullFade.w;
+	float viewportHeight = shadowTopRect.y + topHeight;
+	float fullHeight = topHeight + bottomHeight;
+	float topY = min(
+		(viewportHeight - gl_FragCoord.y) / fullHeight,
+		topHeight / fullHeight);
+	float topX = (gl_FragCoord.x - shadowTopRect.x) / shadowTopRect.z;
+	vec4 fadeTop = texture2D(f_texture, vec2(topX, topY)) * opacity;
+	float bottomY = max(bottomSkip + fullHeight - gl_FragCoord.y, topHeight)
+		/ fullHeight;
+	vec4 fadeBottom = texture2D(f_texture, vec2(0.5, bottomY)) * opacity;
+	float fade = min((1. - fadeTop.a) * (1. - fadeBottom.a), fullFade);
+	result.rgb = result.rgb * fade;
+)",
+	};
+}
 
 [[nodiscard]] ShaderPart FragmentPlaceOnTransparentBackground() {
 	return {
@@ -66,7 +99,7 @@ float roundedCorner() {
 }
 )",
 		.body = R"(
-	result = vec4(roundedCorner());
+	result *= roundedCorner();
 )",
 	};
 }
@@ -77,6 +110,7 @@ OverlayWidget::RendererGL::RendererGL(not_null<OverlayWidget*> owner)
 : _owner(owner) {
 	style::PaletteChanged(
 	) | rpl::start_with_next([=] {
+		_controlsFadeImage.invalidate();
 		_radialImage.invalidate();
 		_documentBubbleImage.invalidate();
 		_themePreviewImage.invalidate();
@@ -85,19 +119,39 @@ OverlayWidget::RendererGL::RendererGL(not_null<OverlayWidget*> owner)
 		_captionImage.invalidate();
 		invalidateControls();
 	}, _lifetime);
+
+	crl::on_main(this, [=] {
+		_owner->_storiesChanged.events(
+		) | rpl::start_with_next([=] {
+			if (_owner->_storiesSession) {
+				Data::AmPremiumValue(
+					_owner->_storiesSession
+				) | rpl::start_with_next([=] {
+					invalidateControls();
+				}, _storiesLifetime);
+			} else {
+				_storiesLifetime.destroy();
+				invalidateControls();
+			}
+		}, _lifetime);
+	});
 }
 
 void OverlayWidget::RendererGL::init(
 		not_null<QOpenGLWidget*> widget,
 		QOpenGLFunctions &f) {
-	constexpr auto kQuads = 8;
+	constexpr auto kQuads = 9;
 	constexpr auto kQuadVertices = kQuads * 4;
 	constexpr auto kQuadValues = kQuadVertices * 4;
 	constexpr auto kControlsValues = kControlsCount * kControlValues;
 	constexpr auto kRoundingQuads = 4;
 	constexpr auto kRoundingVertices = kRoundingQuads * 6;
 	constexpr auto kRoundingValues = kRoundingVertices * 2;
-	constexpr auto kValues = kQuadValues + kControlsValues + kRoundingValues;
+	constexpr auto kStoriesSiblingValues = kStoriesSiblingPartsCount * 16;
+	constexpr auto kValues = kQuadValues
+		+ kControlsValues
+		+ kRoundingValues
+		+ kStoriesSiblingValues;
 
 	_contentBuffer.emplace();
 	_contentBuffer->setUsagePattern(QOpenGLBuffer::DynamicDraw);
@@ -118,6 +172,16 @@ void OverlayWidget::RendererGL::init(
 			FragmentSampleARGB32Texture(),
 		})).vertex;
 
+	_staticContentProgram.emplace();
+	LinkProgram(
+		&*_staticContentProgram,
+		_texturedVertexShader,
+		FragmentShader({
+			FragmentSampleARGB32Texture(),
+			FragmentApplyControlsFade(),
+			FragmentRoundedCorners()
+		}));
+
 	_withTransparencyProgram.emplace();
 	LinkProgram(
 		&*_withTransparencyProgram,
@@ -125,6 +189,7 @@ void OverlayWidget::RendererGL::init(
 		FragmentShader({
 			FragmentSampleARGB32Texture(),
 			FragmentPlaceOnTransparentBackground(),
+			FragmentApplyControlsFade()
 		}));
 
 	_yuv420Program.emplace();
@@ -133,6 +198,8 @@ void OverlayWidget::RendererGL::init(
 		_texturedVertexShader,
 		FragmentShader({
 			FragmentSampleYUV420Texture(),
+			FragmentApplyControlsFade(),
+			FragmentRoundedCorners()
 		}));
 
 	_nv12Program.emplace();
@@ -141,6 +208,8 @@ void OverlayWidget::RendererGL::init(
 		_texturedVertexShader,
 		FragmentShader({
 			FragmentSampleNV12Texture(),
+			FragmentApplyControlsFade(),
+			FragmentRoundedCorners()
 		}));
 
 	_fillProgram.emplace();
@@ -162,7 +231,10 @@ void OverlayWidget::RendererGL::init(
 	LinkProgram(
 		&*_roundedCornersProgram,
 		VertexShader({ VertexViewportTransform() }),
-		FragmentShader({ FragmentRoundedCorners() }));
+		FragmentShader({
+			{ .body = "result = vec4(1.);" },
+			FragmentRoundedCorners(),
+		}));
 
 	const auto renderer = reinterpret_cast<const char*>(
 		f.glGetString(GL_RENDERER));
@@ -183,6 +255,18 @@ void OverlayWidget::RendererGL::deinit(
 	_fillProgram = std::nullopt;
 	_controlsProgram = std::nullopt;
 	_contentBuffer = std::nullopt;
+	_controlsFadeImage.destroy(f);
+	_radialImage.destroy(f);
+	_documentBubbleImage.destroy(f);
+	_themePreviewImage.destroy(f);
+	_saveMsgImage.destroy(f);
+	_footerImage.destroy(f);
+	_captionImage.destroy(f);
+	_groupThumbsImage.destroy(f);
+	_controlsImage.destroy(f);
+	for (auto &part : _storiesSiblingParts) {
+		part.destroy(f);
+	}
 }
 
 void OverlayWidget::RendererGL::paint(
@@ -191,10 +275,17 @@ void OverlayWidget::RendererGL::paint(
 	if (handleHideWorkaround(f)) {
 		return;
 	}
-	const auto factor = widget->devicePixelRatio();
+	const auto factor = widget->devicePixelRatioF();
 	if (_factor != factor) {
 		_factor = factor;
+		_ifactor = int(std::ceil(factor));
 		_controlsImage.invalidate();
+
+		// We use the fact that fade texture atlas
+		// takes exactly full texture size. In case we
+		// just invalidate it we may get larger image
+		// in case of moving from greater _factor to lesser.
+		_controlsFadeImage.destroy(&f);
 	}
 	_blendingEnabled = false;
 	_viewport = widget->size();
@@ -224,6 +315,28 @@ bool OverlayWidget::RendererGL::handleHideWorkaround(QOpenGLFunctions &f) {
 
 void OverlayWidget::RendererGL::paintBackground() {
 	_contentBuffer->bind();
+	if (const auto notch = _owner->topNotchSkip()) {
+		const auto top = transformRect(QRect(0, 0, _owner->width(), notch));
+        const GLfloat coords[] = {
+			top.left(), top.top(),
+			top.right(), top.top(),
+			top.right(), top.bottom(),
+			top.left(), top.bottom(),
+		};
+		const auto offset = kNotchOffset;
+		_contentBuffer->write(
+			offset * 4 * sizeof(GLfloat),
+			coords,
+			sizeof(coords));
+
+		_fillProgram->bind();
+		_fillProgram->setUniformValue("viewport", _uniformViewport);
+		FillRectangle(
+			*_f,
+			&*_fillProgram,
+			offset,
+			QColor(0, 0, 0));
+	}
 }
 
 void OverlayWidget::RendererGL::paintTransformedVideoFrame(
@@ -255,7 +368,7 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 	_streamedIndex = _owner->streamedIndex();
 
 	_f->glActiveTexture(GL_TEXTURE0);
-	_textures.bind(*_f, 1);
+	_textures.bind(*_f, 3);
 	if (upload) {
 		_f->glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 		uploadTexture(
@@ -268,7 +381,7 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 		_lumaSize = yuv->size;
 	}
 	_f->glActiveTexture(GL_TEXTURE1);
-	_textures.bind(*_f, 2);
+	_textures.bind(*_f, 4);
 	if (upload) {
 		uploadTexture(
 			nv12 ? GL_RG : GL_ALPHA,
@@ -283,9 +396,14 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 		}
 		_chromaNV12 = nv12;
 	}
-	if (!nv12) {
+
+	validateControlsFade();
+	if (nv12) {
 		_f->glActiveTexture(GL_TEXTURE2);
-		_textures.bind(*_f, 3);
+		_controlsFadeImage.bind(*_f);
+	} else {
+		_f->glActiveTexture(GL_TEXTURE2);
+		_textures.bind(*_f, 5);
 		if (upload) {
 			uploadTexture(
 				GL_ALPHA,
@@ -297,6 +415,9 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 			_chromaSize = yuv->chromaSize;
 			_f->glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
 		}
+
+		_f->glActiveTexture(GL_TEXTURE3);
+		_controlsFadeImage.bind(*_f);
 	}
 	program->setUniformValue("y_texture", GLint(0));
 	if (nv12) {
@@ -305,16 +426,19 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 		program->setUniformValue("u_texture", GLint(1));
 		program->setUniformValue("v_texture", GLint(2));
 	}
+	program->setUniformValue("f_texture", GLint(nv12 ? 2 : 3));
 
-	toggleBlending(false);
-	paintTransformedContent(program, geometry);
+	toggleBlending(geometry.roundRadius > 0.);
+	paintTransformedContent(program, geometry, false);
 }
 
 void OverlayWidget::RendererGL::paintTransformedStaticContent(
 		const QImage &image,
 		ContentGeometry geometry,
 		bool semiTransparent,
-		bool fillTransparentBackground) {
+		bool fillTransparentBackground,
+		int index) {
+	Expects(index >= 0 && index < 3);
 	Expects(image.isNull()
 		|| image.format() == QImage::Format_RGB32
 		|| image.format() == QImage::Format_ARGB32_Premultiplied);
@@ -325,7 +449,7 @@ void OverlayWidget::RendererGL::paintTransformedStaticContent(
 
 	auto &program = fillTransparentBackground
 		? _withTransparencyProgram
-		: _imageProgram;
+		: _staticContentProgram;
 	program->bind();
 	if (fillTransparentBackground) {
 		program->setUniformValue(
@@ -340,11 +464,11 @@ void OverlayWidget::RendererGL::paintTransformedStaticContent(
 	}
 
 	_f->glActiveTexture(GL_TEXTURE0);
-	_textures.bind(*_f, 0);
+	_textures.bind(*_f, index);
 	const auto cacheKey = image.isNull() ? qint64(-1) : image.cacheKey();
-	const auto upload = (_cacheKey != cacheKey);
+	const auto upload = (_cacheKeys[index] != cacheKey);
 	if (upload) {
-		_cacheKey = cacheKey;
+		_cacheKeys[index] = cacheKey;
 		if (image.isNull()) {
 			// Upload transparent 2x2 texture.
 			const auto stride = 2;
@@ -353,9 +477,10 @@ void OverlayWidget::RendererGL::paintTransformedStaticContent(
 				Ui::GL::kFormatRGBA,
 				Ui::GL::kFormatRGBA,
 				QSize(2, 2),
-				_rgbaSize,
+				_rgbaSize[index],
 				stride,
 				data);
+			_rgbaSize[index] = QSize(2, 2);
 		} else {
 			const auto stride = image.bytesPerLine() / 4;
 			const auto data = image.constBits();
@@ -363,22 +488,32 @@ void OverlayWidget::RendererGL::paintTransformedStaticContent(
 				Ui::GL::kFormatRGBA,
 				Ui::GL::kFormatRGBA,
 				image.size(),
-				_rgbaSize,
+				_rgbaSize[index],
 				stride,
 				data);
-			_rgbaSize = image.size();
+			_rgbaSize[index] = image.size();
 		}
 	}
-	program->setUniformValue("s_texture", GLint(0));
 
-	toggleBlending(semiTransparent && !fillTransparentBackground);
-	paintTransformedContent(&*program, geometry);
+	validateControlsFade();
+	_f->glActiveTexture(GL_TEXTURE1);
+	_controlsFadeImage.bind(*_f);
+
+	program->setUniformValue("s_texture", GLint(0));
+	program->setUniformValue("f_texture", GLint(1));
+
+	toggleBlending((geometry.roundRadius > 0.)
+		|| (semiTransparent && !fillTransparentBackground));
+	paintTransformedContent(&*program, geometry, fillTransparentBackground);
 }
 
 void OverlayWidget::RendererGL::paintTransformedContent(
 		not_null<QOpenGLShaderProgram*> program,
-		ContentGeometry geometry) {
-	const auto rect = transformRect(geometry.rect);
+		ContentGeometry geometry,
+		bool fillTransparentBackground) {
+	const auto rect = scaleRect(
+		transformRect(geometry.rect),
+		geometry.scale);
 	const auto centerx = rect.x() + rect.width() / 2;
 	const auto centery = rect.y() + rect.height() / 2;
 	const auto rsin = float(std::sin(geometry.rotation * M_PI / 180.));
@@ -412,7 +547,44 @@ void OverlayWidget::RendererGL::paintTransformedContent(
 	_contentBuffer->write(0, coords, sizeof(coords));
 
 	program->setUniformValue("viewport", _uniformViewport);
-
+	if (_owner->_stories) {
+		const auto &top = st::storiesShadowTop.size();
+		const auto shadowTop = geometry.topShadowShown
+			? geometry.rect.y()
+			: geometry.rect.y() - top.height();
+		program->setUniformValue(
+			"shadowTopRect",
+			Uniform(transformRect(
+				QRect(QPoint(geometry.rect.x(), shadowTop), top))));
+	} else {
+		const auto &top = st::mediaviewShadowTop.size();
+		const auto point = QPoint(
+			_shadowTopFlip ? 0 : (_viewport.width() - top.width()),
+			0);
+		program->setUniformValue(
+			"shadowTopRect",
+			Uniform(transformRect(QRect(point, top))));
+	}
+	const auto &bottom = _owner->_stories
+		? st::storiesShadowBottom
+		: st::mediaviewShadowBottom;
+	program->setUniformValue("shadowBottomSkipOpacityFullFade", QVector4D(
+		bottom.height() * _factor,
+		geometry.bottomShadowSkip * _factor,
+		geometry.controlsOpacity,
+		1.f - float(geometry.fade)));
+	if (!fillTransparentBackground) {
+		program->setUniformValue(
+			"roundRect",
+			geometry.roundRadius ? Uniform(rect) : QVector4D(
+				0,
+				0,
+				_uniformViewport.x(),
+				_uniformViewport.y()));
+		program->setUniformValue(
+			"roundRadius",
+			GLfloat(geometry.roundRadius * _factor));
+	}
 	FillTexturedRectangle(*_f, &*program);
 }
 
@@ -493,29 +665,38 @@ void OverlayWidget::RendererGL::paintControlsStart() {
 }
 
 void OverlayWidget::RendererGL::paintControl(
-		OverState control,
-		QRect outer,
-		float64 outerOpacity,
+		Over control,
+		QRect over,
+		float64 overOpacity,
 		QRect inner,
 		float64 innerOpacity,
 		const style::icon &icon) {
-	const auto meta = ControlMeta(control);
+	const auto meta = controlMeta(control);
 	Assert(meta.icon == &icon);
 
-	const auto &bg = st::mediaviewControlBg->c;
-	const auto bgAlpha = int(base::SafeRound(bg.alpha() * outerOpacity));
+	const auto overAlpha = overOpacity * kOverBackgroundOpacity;
 	const auto offset = kControlsOffset + (meta.index * kControlValues) / 4;
-	const auto fgOffset = offset + 2;
-	const auto bgRect = transformRect(outer);
+	const auto fgOffset = offset + 4;
+	const auto overRect = _controlsImage.texturedRect(
+		over,
+		_controlsTextures[kControlsCount]);
+	const auto overGeometry = transformRect(over);
 	const auto iconRect = _controlsImage.texturedRect(
 		inner,
 		_controlsTextures[meta.index]);
 	const auto iconGeometry = transformRect(iconRect.geometry);
 	const GLfloat coords[] = {
-		bgRect.left(), bgRect.top(),
-		bgRect.right(), bgRect.top(),
-		bgRect.right(), bgRect.bottom(),
-		bgRect.left(), bgRect.bottom(),
+		overGeometry.left(), overGeometry.top(),
+		overRect.texture.left(), overRect.texture.bottom(),
+
+		overGeometry.right(), overGeometry.top(),
+		overRect.texture.right(), overRect.texture.bottom(),
+
+		overGeometry.right(), overGeometry.bottom(),
+		overRect.texture.right(), overRect.texture.top(),
+
+		overGeometry.left(), overGeometry.bottom(),
+		overRect.texture.left(), overRect.texture.top(),
 
 		iconGeometry.left(), iconGeometry.top(),
 		iconRect.texture.left(), iconRect.texture.bottom(),
@@ -529,39 +710,47 @@ void OverlayWidget::RendererGL::paintControl(
 		iconGeometry.left(), iconGeometry.bottom(),
 		iconRect.texture.left(), iconRect.texture.top(),
 	};
-	if (!outer.isEmpty() && bgAlpha > 0) {
+	_controlsProgram->bind();
+	_controlsProgram->setUniformValue("viewport", _uniformViewport);
+	if (!over.isEmpty() && overOpacity > 0) {
 		_contentBuffer->write(
 			offset * 4 * sizeof(GLfloat),
 			coords,
 			sizeof(coords));
-		_fillProgram->bind();
-		_fillProgram->setUniformValue("viewport", _uniformViewport);
-		FillRectangle(
-			*_f,
-			&*_fillProgram,
-			offset,
-			QColor(bg.red(), bg.green(), bg.blue(), bgAlpha));
+		_controlsProgram->setUniformValue("g_opacity", GLfloat(overAlpha));
+		FillTexturedRectangle(*_f, &*_controlsProgram, offset);
 	} else {
 		_contentBuffer->write(
 			fgOffset * 4 * sizeof(GLfloat),
 			coords + (fgOffset - offset) * 4,
 			sizeof(coords) - (fgOffset - offset) * 4 * sizeof(GLfloat));
 	}
-	_controlsProgram->bind();
 	_controlsProgram->setUniformValue("g_opacity", GLfloat(innerOpacity));
-	_controlsProgram->setUniformValue("viewport", _uniformViewport);
 	FillTexturedRectangle(*_f, &*_controlsProgram, fgOffset);
 }
 
-auto OverlayWidget::RendererGL::ControlMeta(OverState control)
--> Control {
+auto OverlayWidget::RendererGL::controlMeta(Over control) const -> Control {
+	const auto stories = [&] {
+		return (_owner->_stories != nullptr);
+	};
 	switch (control) {
-	case OverLeftNav: return { 0, &st::mediaviewLeft };
-	case OverRightNav: return { 1, &st::mediaviewRight };
-	case OverClose: return { 2, &st::mediaviewClose };
-	case OverSave: return { 3, &st::mediaviewSave };
-	case OverRotate: return { 4, &st::mediaviewRotate };
-	case OverMore: return { 5, &st::mediaviewMore };
+	case Over::Left: return {
+		0,
+		stories() ? &st::storiesLeft : &st::mediaviewLeft
+	};
+	case Over::Right: return {
+		1,
+		stories() ? &st::storiesRight : &st::mediaviewRight
+	};
+	case Over::Save: return {
+		2,
+		(_owner->saveControlLocked()
+			? &st::mediaviewSaveLocked
+			: &st::mediaviewSave)
+	};
+	case Over::Share: return { 3, &st::mediaviewShare };
+	case Over::Rotate: return { 4, &st::mediaviewRotate };
+	case Over::More: return { 5, &st::mediaviewMore };
 	}
 	Unexpected("Control value in OverlayWidget::RendererGL::ControlIndex.");
 }
@@ -571,12 +760,12 @@ void OverlayWidget::RendererGL::validateControls() {
 		return;
 	}
 	const auto metas = {
-		ControlMeta(OverLeftNav),
-		ControlMeta(OverRightNav),
-		ControlMeta(OverClose),
-		ControlMeta(OverSave),
-		ControlMeta(OverRotate),
-		ControlMeta(OverMore),
+		controlMeta(Over::Left),
+		controlMeta(Over::Right),
+		controlMeta(Over::Save),
+		controlMeta(Over::Share),
+		controlMeta(Over::Rotate),
+		controlMeta(Over::More),
 	};
 	auto maxWidth = 0;
 	auto fullHeight = 0;
@@ -584,11 +773,13 @@ void OverlayWidget::RendererGL::validateControls() {
 		maxWidth = std::max(meta.icon->width(), maxWidth);
 		fullHeight += meta.icon->height();
 	}
+	maxWidth = std::max(st::mediaviewIconOver, maxWidth);
+	fullHeight += st::mediaviewIconOver;
 	auto image = QImage(
-		QSize(maxWidth, fullHeight) * _factor,
+		QSize(maxWidth, fullHeight) * _ifactor,
 		QImage::Format_ARGB32_Premultiplied);
 	image.fill(Qt::transparent);
-	image.setDevicePixelRatio(_factor);
+	image.setDevicePixelRatio(_ifactor);
 	{
 		auto p = QPainter(&image);
 		auto index = 0;
@@ -596,10 +787,19 @@ void OverlayWidget::RendererGL::validateControls() {
 		for (const auto &meta : metas) {
 			meta.icon->paint(p, 0, height, maxWidth);
 			_controlsTextures[index++] = QRect(
-				QPoint(0, height) * _factor,
-				meta.icon->size() * _factor);
+				QPoint(0, height) * _ifactor,
+				meta.icon->size() * _ifactor);
 			height += meta.icon->height();
 		}
+		auto hq = PainterHighQualityEnabler(p);
+		p.setPen(Qt::NoPen);
+		p.setBrush(OverBackgroundColor());
+		p.drawEllipse(
+			QRect(0, height, st::mediaviewIconOver, st::mediaviewIconOver));
+		_controlsTextures[index++] = QRect(
+			QPoint(0, height) * _ifactor,
+			QSize(st::mediaviewIconOver, st::mediaviewIconOver) * _ifactor);
+		height += st::mediaviewIconOver;
 	}
 	_controlsImage.setImage(std::move(image));
 }
@@ -609,6 +809,46 @@ void OverlayWidget::RendererGL::invalidateControls() {
 	ranges::fill(_controlsTextures, QRect());
 }
 
+void OverlayWidget::RendererGL::validateControlsFade() {
+	const auto forStories = (_owner->_stories != nullptr);
+	const auto flip = !forStories && !_owner->topShadowOnTheRight();
+	if (!_controlsFadeImage.image().isNull()
+		&& _shadowTopFlip == flip
+		&& _shadowsForStories == forStories) {
+		return;
+	}
+	_shadowTopFlip = flip;
+	_shadowsForStories = forStories;
+	const auto &top = _shadowsForStories
+		? st::storiesShadowTop
+		: st::mediaviewShadowTop;
+	const auto &bottom = _shadowsForStories
+		? st::storiesShadowBottom
+		: st::mediaviewShadowBottom;
+	const auto width = top.width();
+	const auto bottomTop = top.height();
+	const auto height = bottomTop + bottom.height();
+
+	auto image = QImage(
+		QSize(width, height) * _ifactor,
+		QImage::Format_ARGB32_Premultiplied);
+	image.fill(Qt::transparent);
+	image.setDevicePixelRatio(_ifactor);
+
+	auto p = QPainter(&image);
+	top.paint(p, 0, 0, width);
+	bottom.fill(
+		p,
+		QRect(0, bottomTop, width, bottom.height()));
+	p.end();
+
+	if (flip) {
+		image = std::move(image).mirrored(true, false);
+	}
+
+	_controlsFadeImage.setImage(std::move(image));
+}
+
 void OverlayWidget::RendererGL::paintFooter(QRect outer, float64 opacity) {
 	paintUsingRaster(_footerImage, outer, [&](Painter &&p) {
 		const auto newOuter = QRect(QPoint(), outer.size());
@@ -702,25 +942,77 @@ void OverlayWidget::RendererGL::paintRoundedCorners(int radius) {
 	_f->glDisableVertexAttribArray(position);
 }
 
-void OverlayWidget::RendererGL::invalidate() {
-	_trackFrameIndex = -1;
-	_streamedIndex = -1;
-	const auto images = {
-		&_radialImage,
-		&_documentBubbleImage,
-		&_themePreviewImage,
-		&_saveMsgImage,
-		&_footerImage,
-		&_captionImage,
-		&_groupThumbsImage,
-		&_controlsImage,
-	};
-	for (const auto image : images) {
-		image->setImage(QImage());
+void OverlayWidget::RendererGL::paintStoriesSiblingPart(
+		int index,
+		const QImage &image,
+		QRect rect,
+		float64 opacity) {
+	Expects(index >= 0 && index < kStoriesSiblingPartsCount);
+
+	if (image.isNull() || rect.isEmpty()) {
+		return;
 	}
-	invalidateControls();
+
+	_f->glActiveTexture(GL_TEXTURE0);
+
+	auto &part = _storiesSiblingParts[index];
+	part.setImage(image);
+	part.bind(*_f);
+
+	const auto textured = part.texturedRect(
+		rect,
+		QRect(QPoint(), image.size()));
+	const auto geometry = transformRect(textured.geometry);
+	const GLfloat coords[] = {
+		geometry.left(), geometry.top(),
+		textured.texture.left(), textured.texture.bottom(),
+
+		geometry.right(), geometry.top(),
+		textured.texture.right(), textured.texture.bottom(),
+
+		geometry.right(), geometry.bottom(),
+		textured.texture.right(), textured.texture.top(),
+
+		geometry.left(), geometry.bottom(),
+		textured.texture.left(), textured.texture.top(),
+	};
+	const auto offset = kControlsOffset
+		+ (kControlsCount * kControlValues) / 4
+		+ (6 * 2 * 4) / 4 // rounding
+		+ (index * 4);
+	const auto byteOffset = offset * 4 * sizeof(GLfloat);
+	_contentBuffer->write(byteOffset, coords, sizeof(coords));
+
+	_controlsProgram->bind();
+	_controlsProgram->setUniformValue("viewport", _uniformViewport);
+	_contentBuffer->write(
+		offset * 4 * sizeof(GLfloat),
+		coords,
+		sizeof(coords));
+	_controlsProgram->setUniformValue("g_opacity", GLfloat(opacity));
+	FillTexturedRectangle(*_f, &*_controlsProgram, offset);
 }
 
+//
+//void OverlayWidget::RendererGL::invalidate() {
+//	_trackFrameIndex = -1;
+//	_streamedIndex = -1;
+//	const auto images = {
+//		&_radialImage,
+//		&_documentBubbleImage,
+//		&_themePreviewImage,
+//		&_saveMsgImage,
+//		&_footerImage,
+//		&_captionImage,
+//		&_groupThumbsImage,
+//		&_controlsImage,
+//	};
+//	for (const auto image : images) {
+//		image->setImage(QImage());
+//	}
+//	invalidateControls();
+//}
+
 void OverlayWidget::RendererGL::paintUsingRaster(
 		Ui::GL::Image &image,
 		QRect rect,
@@ -728,17 +1020,18 @@ void OverlayWidget::RendererGL::paintUsingRaster(
 		int bufferOffset,
 		bool transparent) {
 	auto raster = image.takeImage();
-	const auto size = rect.size() * _factor;
+	const auto size = rect.size() * _ifactor;
 	if (raster.width() < size.width() || raster.height() < size.height()) {
 		raster = QImage(size, QImage::Format_ARGB32_Premultiplied);
-		raster.setDevicePixelRatio(_factor);
+		Assert(!raster.isNull());
+		raster.setDevicePixelRatio(_ifactor);
 		if (!transparent
 			&& (raster.width() > size.width()
 				|| raster.height() > size.height())) {
 			raster.fill(Qt::transparent);
 		}
-	} else if (raster.devicePixelRatio() != _factor) {
-		raster.setDevicePixelRatio(_factor);
+	} else if (raster.devicePixelRatio() != _ifactor) {
+		raster.setDevicePixelRatio(_ifactor);
 	}
 
 	if (transparent) {
@@ -803,4 +1096,17 @@ Rect OverlayWidget::RendererGL::transformRect(const QRect &raster) const {
 	return TransformRect(Rect(raster), _viewport, _factor);
 }
 
+Rect OverlayWidget::RendererGL::scaleRect(
+		const Rect &unscaled,
+		float64 scale) const {
+	const auto added = scale - 1.;
+	const auto addw = unscaled.width() * added;
+	const auto addh = unscaled.height() * added;
+	return Rect(
+		unscaled.x() - addw / 2,
+		unscaled.y() - addh / 2,
+		unscaled.width() + addw,
+		unscaled.height() + addh);
+}
+
 } // namespace Media::View
