@@ -15,16 +15,12 @@ class Launcher : public Core::Launcher {
 public:
 	Launcher(int argc, char *argv[]);
 
-	static Launcher &Instance();
-
 	int exec() override;
 
 private:
 	void initHook() override;
 	bool launchUpdater(UpdaterLaunch action) override;
 
-	std::vector<std::string> _arguments;
-
 };
 
 } // namespace Platform
