@@ -12,6 +12,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/window_session_controller.h"
 #include "window/window_controller.h"
 #include "ui/chat/chat_theme.h"
+#include "ui/controls/userpic_button.h"
+#include "ui/effects/snowflakes.h"
+#include "ui/effects/toggle_arrow.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/labels.h"
 #include "ui/widgets/popup_menu.h"
@@ -23,19 +26,23 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/text/format_values.h" // Ui::FormatPhone
 #include "ui/text/text_utilities.h"
 #include "ui/text/text_options.h"
-#include "ui/special_buttons.h"
+#include "ui/painter.h"
 #include "ui/empty_userpic.h"
-#include "dialogs/ui/dialogs_layout.h"
+#include "ui/unread_badge_paint.h"
 #include "base/call_delayed.h"
+#include "inline_bots/bot_attach_web_view.h"
 #include "mainwindow.h"
 #include "storage/localstorage.h"
 #include "storage/storage_account.h"
 #include "support/support_templates.h"
+#include "settings/settings_advanced.h"
 #include "settings/settings_common.h"
 #include "settings/settings_calls.h"
 #include "settings/settings_information.h"
 #include "info/profile/info_profile_badge.h"
 #include "info/profile/info_profile_emoji_status_panel.h"
+#include "info/stories/info_stories_widget.h"
+#include "info/info_memento.h"
 #include "base/qt_signal_producer.h"
 #include "boxes/about_box.h"
 #include "ui/boxes/confirm_box.h"
@@ -56,7 +63,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_session.h"
 #include "data/data_user.h"
 #include "data/data_changes.h"
+#include "data/data_stories.h"
 #include "mainwidget.h"
+#include "styles/style_chat.h" // popupMenuExpandedSeparator
 #include "styles/style_window.h"
 #include "styles/style_widgets.h"
 #include "styles/style_dialogs.h"
@@ -77,20 +86,75 @@ namespace {
 
 constexpr auto kPlayStatusLimit = 2;
 
+[[nodiscard]] bool CanCheckSpecialEvent() {
+	static const auto result = [] {
+		const auto now = QDate::currentDate();
+		return (now.month() == 12) || (now.month() == 1 && now.day() == 1);
+	}();
+	return result;
+}
+
+[[nodiscard]] bool CheckSpecialEvent() {
+	const auto now = QDate::currentDate();
+	return (now.month() == 12 && now.day() >= 24)
+		|| (now.month() == 1 && now.day() == 1);
+}
+
 void ShowCallsBox(not_null<Window::SessionController*> window) {
-	auto controller = std::make_unique<Calls::BoxController>(window);
-	const auto initBox = [
-		window,
-		controller = controller.get()
-	](not_null<PeerListBox*> box) {
+	struct State {
+		State(not_null<Window::SessionController*> window)
+		: callsController(window)
+		, groupCallsController(window) {
+		}
+		Calls::BoxController callsController;
+		PeerListContentDelegateSimple callsDelegate;
+
+		Calls::GroupCalls::ListController groupCallsController;
+		PeerListContentDelegateSimple groupCallsDelegate;
+
+		base::unique_qptr<Ui::PopupMenu> menu;
+	};
+
+	window->show(Box([=](not_null<Ui::GenericBox*> box) {
+		const auto state = box->lifetime().make_state<State>(window);
+
+		const auto groupCalls = box->addRow(
+			object_ptr<Ui::SlideWrap<Ui::VerticalLayout>>(
+				box,
+				object_ptr<Ui::VerticalLayout>(box)),
+			{});
+		groupCalls->hide(anim::type::instant);
+		groupCalls->toggleOn(state->groupCallsController.shownValue());
+
+		Settings::AddSubsectionTitle(
+			groupCalls->entity(),
+			tr::lng_call_box_groupcalls_subtitle());
+		state->groupCallsDelegate.setContent(groupCalls->entity()->add(
+			object_ptr<PeerListContent>(box, &state->groupCallsController),
+			{}));
+		state->groupCallsController.setDelegate(&state->groupCallsDelegate);
+		Settings::AddSkip(groupCalls->entity());
+		Settings::AddDivider(groupCalls->entity());
+		Settings::AddSkip(groupCalls->entity());
+
+		const auto content = box->addRow(
+			object_ptr<PeerListContent>(box, &state->callsController),
+			{});
+		state->callsDelegate.setContent(content);
+		state->callsController.setDelegate(&state->callsDelegate);
+
+		box->setWidth(state->callsController.contentWidth());
+		state->callsController.boxHeightValue(
+		) | rpl::start_with_next([=](int height) {
+			box->setMinHeight(height);
+		}, box->lifetime());
+		box->setTitle(tr::lng_call_box_title());
 		box->addButton(tr::lng_close(), [=] {
 			box->closeBox();
 		});
-		using MenuPointer = base::unique_qptr<Ui::PopupMenu>;
-		const auto menu = std::make_shared<MenuPointer>();
 		const auto menuButton = box->addTopButton(st::infoTopBarMenu);
 		menuButton->setClickedCallback([=] {
-			*menu = base::make_unique_q<Ui::PopupMenu>(
+			state->menu = base::make_unique_q<Ui::PopupMenu>(
 				menuButton,
 				st::popupMenuWithIcons);
 			const auto showSettings = [=] {
@@ -99,23 +163,22 @@ void ShowCallsBox(not_null<Window::SessionController*> window) {
 					Window::SectionShow(anim::type::instant));
 			};
 			const auto clearAll = crl::guard(box, [=] {
-				box->getDelegate()->show(Box(Calls::ClearCallsBox, window));
+				box->uiShow()->showBox(Box(Calls::ClearCallsBox, window));
 			});
-			(*menu)->addAction(
+			state->menu->addAction(
 				tr::lng_settings_section_call_settings(tr::now),
 				showSettings,
 				&st::menuIconSettings);
-			if (controller->delegate()->peerListFullRowsCount() > 0) {
-				(*menu)->addAction(
+			if (state->callsDelegate.peerListFullRowsCount() > 0) {
+				state->menu->addAction(
 					tr::lng_call_box_clear_all(tr::now),
 					clearAll,
 					&st::menuIconDelete);
 			}
-			(*menu)->popup(QCursor::pos());
+			state->menu->popup(QCursor::pos());
 			return true;
 		});
-	};
-	window->show(Box<PeerListBox>(std::move(controller), initBox));
+	}));
 }
 
 [[nodiscard]] rpl::producer<TextWithEntities> SetStatusLabel(
@@ -136,6 +199,98 @@ void ShowCallsBox(not_null<Window::SessionController*> window) {
 	}) | rpl::flatten_latest();
 }
 
+void SetupMenuBots(
+		not_null<Ui::VerticalLayout*> container,
+		not_null<Window::SessionController*> controller) {
+	const auto wrap = container->add(
+		object_ptr<Ui::VerticalLayout>(container));
+	const auto bots = &controller->session().attachWebView();
+
+	rpl::single(
+		rpl::empty
+	) | rpl::then(
+		bots->attachBotsUpdates()
+	) | rpl::start_with_next([=] {
+		const auto width = container->widthNoMargins();
+		wrap->clear();
+		for (const auto &bot : bots->attachBots()) {
+			if (!bot.inMainMenu) {
+				continue;
+			}
+			const auto button = Settings::AddButton(
+				wrap,
+				rpl::single(bot.name),
+				st::mainMenuButton);
+			const auto menu = button->lifetime().make_state<
+				base::unique_qptr<Ui::PopupMenu>
+			>();
+			const auto icon = Ui::CreateChild<InlineBots::MenuBotIcon>(
+				button.get(),
+				bot.media);
+			button->heightValue(
+			) | rpl::start_with_next([=](int height) {
+				icon->move(
+					st::mainMenuButton.iconLeft,
+					(height - icon->height()) / 2);
+			}, button->lifetime());
+			const auto user = bot.user;
+			const auto weak = Ui::MakeWeak(container);
+			button->setAcceptBoth(true);
+			button->clicks(
+			) | rpl::start_with_next([=](Qt::MouseButton which) {
+				if (which == Qt::LeftButton) {
+					bots->requestSimple(controller, user, {
+						.fromMainMenu = true,
+					});
+					if (weak) {
+						controller->window().hideSettingsAndLayer();
+					}
+				} else {
+					(*menu) = nullptr;
+					(*menu) = base::make_unique_q<Ui::PopupMenu>(
+						button,
+						st::popupMenuWithIcons);
+					(*menu)->addAction(
+						tr::lng_bot_remove_from_menu(tr::now),
+						[=] { bots->removeFromMenu(user); },
+						&st::menuIconDelete);
+					(*menu)->popup(QCursor::pos());
+				}
+			}, button->lifetime());
+
+			const auto badge = bots->showMainMenuNewBadge(bot)
+				? Ui::CreateChild<Ui::PaddingWrap<Ui::FlatLabel>>(
+					button.get(),
+					object_ptr<Ui::FlatLabel>(
+						button,
+						tr::lng_bot_side_menu_new(),
+						st::settingsPremiumNewBadge),
+					st::settingsPremiumNewBadgePadding)
+				: nullptr;
+			if (badge) {
+				badge->setAttribute(Qt::WA_TransparentForMouseEvents);
+				badge->paintRequest() | rpl::start_with_next([=] {
+					auto p = QPainter(badge);
+					auto hq = PainterHighQualityEnabler(p);
+					p.setPen(Qt::NoPen);
+					p.setBrush(st::windowBgActive);
+					const auto r = st::settingsPremiumNewBadgePadding.left();
+					p.drawRoundedRect(badge->rect(), r, r);
+				}, badge->lifetime());
+
+				button->sizeValue(
+				) | rpl::start_with_next([=](QSize size) {
+					badge->moveToRight(
+						st::mainMenuButton.padding.right(),
+						(size.height() - badge->height()) / 2,
+						size.width());
+				}, badge->lifetime());
+			}
+		}
+		wrap->resizeToWidth(width);
+	}, wrap->lifetime());
+}
+
 } // namespace
 
 class MainMenu::ToggleAccountsButton final : public Ui::AbstractButton {
@@ -211,40 +366,12 @@ MainMenu::ToggleAccountsButton::ToggleAccountsButton(QWidget *parent)
 void MainMenu::ToggleAccountsButton::paintEvent(QPaintEvent *e) {
 	auto p = Painter(this);
 
-	const auto toggled = _toggledAnimation.value(_toggled ? 1. : 0.);
-	const auto x = 0. + width() - st::mainMenuTogglePosition.x();
-	const auto y = 0. + height() - st::mainMenuTogglePosition.y();
-	const auto size = st::mainMenuToggleSize;
-	const auto size2 = size / 2.;
-	const auto sqrt2 = sqrt(2.);
-	const auto stroke = (st::mainMenuToggleFourStrokes / 4.) / sqrt2;
-	const auto left = x - size;
-	const auto right = x + size;
-	const auto bottom = y + size2;
-	constexpr auto kPointCount = 6;
-	std::array<QPointF, kPointCount> points = { {
-		{ left - stroke, bottom - stroke },
-		{ x, bottom - stroke - size - stroke },
-		{ right + stroke, bottom - stroke },
-		{ right - stroke, bottom + stroke },
-		{ x, bottom + stroke - size + stroke },
-		{ left + stroke, bottom + stroke }
-	} };
-	const auto alpha = (toggled - 1.) * M_PI;
-	const auto cosalpha = cos(alpha);
-	const auto sinalpha = sin(alpha);
-	for (auto &point : points) {
-		auto px = point.x() - x;
-		auto py = point.y() - y;
-		point.setX(x + px * cosalpha - py * sinalpha);
-		point.setY(y + py * cosalpha + px * sinalpha);
-	}
-	QPainterPath path;
-	path.moveTo(points[0]);
-	for (int i = 1; i != kPointCount; ++i) {
-		path.lineTo(points[i]);
-	}
-	path.lineTo(points[0]);
+	const auto path = Ui::ToggleUpDownArrowPath(
+		0. + width() - st::mainMenuTogglePosition.x(),
+		0. + height() - st::mainMenuTogglePosition.y(),
+		st::mainMenuToggleSize,
+		st::mainMenuToggleFourStrokes,
+		_toggledAnimation.value(_toggled ? 1. : 0.));
 
 	auto hq = PainterHighQualityEnabler(p);
 	p.fillPath(path, st::windowSubTextFg);
@@ -270,7 +397,7 @@ void MainMenu::ToggleAccountsButton::paintUnreadBadge(Painter &p) {
 		- st::mainMenuTogglePosition.y()
 		- st::mainMenuBadgeSize / 2;
 	p.setOpacity(progress);
-	Dialogs::Ui::PaintUnreadBadge(p, _unreadBadge, right, top, st);
+	Ui::PaintUnreadBadge(p, _unreadBadge, right, top, st);
 }
 
 void MainMenu::ToggleAccountsButton::validateUnreadBadge() {
@@ -288,7 +415,7 @@ void MainMenu::ToggleAccountsButton::validateUnreadBadge() {
 	if (!_unreadBadge.isEmpty()) {
 		const auto st = Settings::Badge::Style();
 		skip += 2 * st::mainMenuToggleSize
-			+ Dialogs::Ui::CountUnreadBadgeSize(_unreadBadge, st).width();
+			+ Ui::CountUnreadBadgeSize(_unreadBadge, st).width();
 	}
 	_rightSkip = skip;
 }
@@ -353,9 +480,7 @@ MainMenu::MainMenu(
 , _controller(controller)
 , _userpicButton(
 	this,
-	_controller,
 	_controller->session().user(),
-	Ui::UserpicButton::Role::Custom,
 	st::mainMenuUserpic)
 , _toggleAccounts(this)
 , _setEmojiStatus(this, SetStatusLabel(&controller->session()))
@@ -431,8 +556,8 @@ MainMenu::MainMenu(
 	parentResized();
 
 	_telegram->setMarkedText(Ui::Text::Link(
-		qsl("Telegram Desktop"),
-		qsl("https://desktop.telegram.org")));
+		u"Telegram Desktop"_q,
+		u"https://desktop.telegram.org"_q));
 	_telegram->setLinksTrusted();
 	_version->setMarkedText(
 		Ui::Text::Link(
@@ -470,6 +595,42 @@ MainMenu::MainMenu(
 	}, lifetime());
 
 	initResetScaleButton();
+
+	if (CanCheckSpecialEvent() && CheckSpecialEvent()) {
+		const auto snowLifetime = lifetime().make_state<rpl::lifetime>();
+		const auto rebuild = [=] {
+			const auto snowRaw = Ui::CreateChild<Ui::RpWidget>(this);
+			const auto snow = snowLifetime->make_state<Ui::Snowflakes>(
+				[=](const QRect &r) { snowRaw->update(r); });
+			snow->setBrush(QColor(230, 230, 230));
+			_showFinished.value(
+			) | rpl::start_with_next([=](bool shown) {
+				snow->setPaused(!shown);
+			}, snowRaw->lifetime());
+			snowRaw->paintRequest(
+			) | rpl::start_with_next([=](const QRect &r) {
+				auto p = Painter(snowRaw);
+				p.fillRect(r, st::mainMenuBg);
+				drawName(p);
+				snow->paint(p, snowRaw->rect());
+			}, snowRaw->lifetime());
+			widthValue(
+			) | rpl::start_with_next([=](int width) {
+				snowRaw->setGeometry(0, 0, width, st::mainMenuCoverHeight);
+			}, snowRaw->lifetime());
+			snowRaw->show();
+			snowRaw->lower();
+			snowRaw->setAttribute(Qt::WA_TransparentForMouseEvents);
+			snowLifetime->add([=] { base::unique_qptr{ snowRaw }; });
+		};
+		Window::Theme::IsNightModeValue(
+		) | rpl::start_with_next([=](bool isNightMode) {
+			snowLifetime->destroy();
+			if (isNightMode) {
+				rebuild();
+			}
+		}, lifetime());
+	}
 }
 
 MainMenu::~MainMenu() = default;
@@ -487,7 +648,7 @@ void MainMenu::moveBadge() {
 	_badge->move(
 		left,
 		st::mainMenuCoverNameTop,
-		st::mainMenuCoverNameTop + st::msgNameStyle.font->height);
+		st::mainMenuCoverNameTop + st::semiboldFont->height);
 }
 
 void MainMenu::setupArchive() {
@@ -506,7 +667,7 @@ void MainMenu::setupArchive() {
 	const auto checkArchive = [=] {
 		const auto f = folder();
 		return f
-			&& !f->chatsList()->empty()
+			&& (!f->chatsList()->empty() || f->storiesCount() > 0)
 			&& controller->session().settings().archiveInMainMenu();
 	};
 
@@ -521,7 +682,7 @@ void MainMenu::setupArchive() {
 		inner,
 		tr::lng_archived_name(),
 		st::mainMenuButton,
-		{ &st::settingsIconArchive, kIconGray });
+		{ &st::menuIconArchiveOpen });
 	inner->add(
 		object_ptr<Ui::PlainShadow>(inner),
 		{ 0, st::mainMenuSkip, 0, st::mainMenuSkip });
@@ -536,7 +697,7 @@ void MainMenu::setupArchive() {
 		}
 		_contextMenu = base::make_unique_q<Ui::PopupMenu>(
 			this,
-			st::popupMenuWithIcons);
+			st::popupMenuExpandedSeparator);
 		const auto addAction = PeerMenuCallback([&](
 				PeerMenuCallback::Args a) {
 			return _contextMenu->addAction(
@@ -560,6 +721,12 @@ void MainMenu::setupArchive() {
 			[f = folder()] { return f->chatsList(); },
 			addAction);
 
+		_contextMenu->addSeparator();
+		Settings::PreloadArchiveSettings(&controller->session());
+		addAction(tr::lng_context_archive_settings(tr::now), [=] {
+			controller->show(Box(Settings::ArchiveSettingsBox, controller));
+		}, &st::menuIconManage);
+
 		_contextMenu->popup(QCursor::pos());
 	}, button->lifetime());
 
@@ -578,16 +745,21 @@ void MainMenu::setupArchive() {
 		return folder->owner().chatsList(folder)->unreadStateChanges();
 	}) | rpl::flatten_latest() | rpl::to_empty) | rpl::map([=] {
 		const auto loaded = folder();
-		return Badge::UnreadBadge{
-			loaded ? loaded->chatListUnreadCount() : 0,
-			true,
-		};
+		const auto state = loaded
+			? loaded->chatListBadgesState()
+			: Dialogs::BadgesState();
+		return Badge::UnreadBadge{ state.unreadCounter, true };
 	}));
 
-	controller->session().data().chatsListChanges(
-	) | rpl::filter([](Data::Folder *folder) {
-		return folder && (folder->id() == Data::Folder::kId);
-	}) | rpl::start_with_next([=] {
+	rpl::merge(
+		controller->session().data().chatsListChanges(
+		) | rpl::filter([](Data::Folder *folder) {
+			return folder && (folder->id() == Data::Folder::kId);
+		}) | rpl::to_empty,
+		controller->session().data().stories().sourcesChanged(
+			Data::StorySourcesList::Hidden
+		)
+	) | rpl::start_with_next([=] {
 		const auto isArchiveVisible = checkArchive();
 		wrap->toggle(isArchiveVisible, anim::type::normal);
 		if (!isArchiveVisible) {
@@ -617,7 +789,7 @@ void MainMenu::setupAccounts() {
 	inner->add(object_ptr<Ui::FixedHeightWidget>(inner, st::mainMenuSkip));
 
 	std::move(
-		events.currentAccountActivations
+		events.closeRequests
 	) | rpl::start_with_next([=] {
 		closeLayer();
 	}, inner->lifetime());
@@ -648,6 +820,10 @@ void MainMenu::parentResized() {
 	resize(st::mainMenuWidth, parentWidget()->height());
 }
 
+void MainMenu::showFinished() {
+	_showFinished = true;
+}
+
 void MainMenu::setupMenu() {
 	using namespace Settings;
 
@@ -664,46 +840,76 @@ void MainMenu::setupMenu() {
 	if (!_controller->session().supportMode()) {
 		addAction(
 			tr::lng_create_group_title(),
-			{ &st::settingsIconGroup, kIconLightBlue }
+			{ &st::menuIconGroups }
 		)->setClickedCallback([=] {
 			controller->showNewGroup();
 		});
 		addAction(
 			tr::lng_create_channel_title(),
-			{ &st::settingsIconChannel, kIconLightOrange }
+			{ &st::menuIconChannel }
 		)->setClickedCallback([=] {
 			controller->showNewChannel();
 		});
+
+		const auto wrap = _menu->add(
+			object_ptr<Ui::SlideWrap<Ui::SettingsButton>>(
+				_menu,
+				CreateButton(
+					_menu,
+					tr::lng_menu_my_stories(),
+					st::mainMenuButton,
+					IconDescriptor{ &st::menuIconStoriesSavedSection })));
+		const auto selfId = controller->session().userPeerId();
+		const auto stories = &controller->session().data().stories();
+		if (stories->archiveCount(selfId) > 0) {
+			wrap->toggle(true, anim::type::instant);
+		} else {
+			wrap->toggle(false, anim::type::instant);
+			if (!stories->archiveCountKnown(selfId)) {
+				stories->archiveLoadMore(selfId);
+				wrap->toggleOn(stories->archiveChanged(
+				) | rpl::filter(
+					rpl::mappers::_1 == selfId
+				) | rpl::map([=] {
+					return stories->archiveCount(selfId) > 0;
+				}) | rpl::filter(rpl::mappers::_1) | rpl::take(1));
+			}
+		}
+		wrap->entity()->setClickedCallback([=] {
+			controller->showSection(
+				Info::Stories::Make(controller->session().user()));
+		});
+
+		SetupMenuBots(_menu, controller);
+
 		addAction(
 			tr::lng_menu_contacts(),
-			{ &st::settingsIconUser, kIconRed }
+			{ &st::menuIconProfile }
 		)->setClickedCallback([=] {
 			controller->show(PrepareContactsBox(controller));
 		});
 		addAction(
 			tr::lng_menu_calls(),
-			{ &st::settingsIconCalls, kIconGreen }
+			{ &st::menuIconPhone }
 		)->setClickedCallback([=] {
 			ShowCallsBox(controller);
 		});
 		addAction(
 			tr::lng_saved_messages(),
-			{ &st::settingsIconSavedMessages, kIconLightBlue }
+			{ &st::menuIconSavedMessages }
 		)->setClickedCallback([=] {
-			controller->content()->choosePeer(
-				controller->session().userPeerId(),
-				ShowAtUnreadMsgId);
+			controller->showPeerHistory(controller->session().user());
 		});
 	} else {
 		addAction(
 			tr::lng_profile_add_contact(),
-			{ &st::settingsIconUser, kIconRed }
+			{ &st::menuIconProfile }
 		)->setClickedCallback([=] {
 			controller->showAddContact();
 		});
 		addAction(
 			rpl::single(u"Fix chats order"_q),
-			{ &st::settingsIconPin, kIconGreen }
+			{ &st::menuIconPin }
 		)->toggleOn(rpl::single(
 			_controller->session().settings().supportFixChatsOrder()
 		))->toggledChanges(
@@ -713,21 +919,21 @@ void MainMenu::setupMenu() {
 		}, _menu->lifetime());
 		addAction(
 			rpl::single(u"Reload templates"_q),
-			{ &st::settingsIconReload, kIconLightBlue }
+			{ &st::menuIconRestore }
 		)->setClickedCallback([=] {
 			_controller->session().supportTemplates().reload();
 		});
 	}
 	addAction(
 		tr::lng_menu_settings(),
-		{ &st::settingsIconSettings, kIconPurple }
+		{ &st::menuIconSettings }
 	)->setClickedCallback([=] {
 		controller->showSettings();
 	});
 
 	_nightThemeToggle = addAction(
 		tr::lng_menu_night_mode(),
-		{ &st::settingsIconNight, kIconDarkBlue }
+		{ &st::menuIconNightMode }
 	)->toggleOn(_nightThemeSwitches.events_starting_with(
 		Window::Theme::IsNightMode()
 	));
@@ -815,39 +1021,43 @@ void MainMenu::chooseEmojiStatus() {
 }
 
 void MainMenu::paintEvent(QPaintEvent *e) {
-	Painter p(this);
+	auto p = Painter(this);
 	const auto clip = e->rect();
 	const auto cover = QRect(0, 0, width(), st::mainMenuCoverHeight);
 
 	p.fillRect(clip, st::mainMenuBg);
 	if (cover.intersects(clip)) {
-		const auto widthText = width()
-			- st::mainMenuCoverNameLeft
-			- _toggleAccounts->rightSkip();
-
-		const auto user = _controller->session().user();
-		if (_nameVersion < user->nameVersion()) {
-			_nameVersion = user->nameVersion();
-			_name.setText(
-				st::msgNameStyle,
-				user->name(),
-				Ui::NameTextOptions());
-			moveBadge();
-		}
-		p.setFont(st::semiboldFont);
-		p.setPen(st::windowBoldFg);
-		_name.drawLeftElided(
-			p,
-			st::mainMenuCoverNameLeft,
-			st::mainMenuCoverNameTop,
-			(widthText
-				- (_badge->widget()
-					? (st::semiboldFont->spacew + _badge->widget()->width())
-					: 0)),
-			width());
+		drawName(p);
 	}
 }
 
+void MainMenu::drawName(Painter &p) {
+	const auto widthText = width()
+		- st::mainMenuCoverNameLeft
+		- _toggleAccounts->rightSkip();
+
+	const auto user = _controller->session().user();
+	if (_nameVersion < user->nameVersion()) {
+		_nameVersion = user->nameVersion();
+		_name.setText(
+			st::semiboldTextStyle,
+			user->name(),
+			Ui::NameTextOptions());
+		moveBadge();
+	}
+	p.setFont(st::semiboldFont);
+	p.setPen(st::windowBoldFg);
+	_name.drawLeftElided(
+		p,
+		st::mainMenuCoverNameLeft,
+		st::mainMenuCoverNameTop,
+		(widthText
+			- (_badge->widget()
+				? (st::semiboldFont->spacew + _badge->widget()->width())
+				: 0)),
+		width());
+}
+
 void MainMenu::initResetScaleButton() {
 	if (!window() || !window()->windowHandle()) {
 		return;
@@ -887,21 +1097,22 @@ void MainMenu::initResetScaleButton() {
 }
 
 OthersUnreadState OtherAccountsUnreadStateCurrent() {
-	auto &app = Core::App();
-	const auto active = &app.activeAccount();
+	auto &domain = Core::App().domain();
+	const auto active = &domain.active();
+	auto counter = 0;
 	auto allMuted = true;
-	for (const auto &[index, account] : app.domain().accounts()) {
+	for (const auto &[index, account] : domain.accounts()) {
 		if (account.get() == active) {
 			continue;
 		} else if (const auto session = account->maybeSession()) {
+			counter += session->data().unreadBadge();
 			if (!session->data().unreadBadgeMuted()) {
 				allMuted = false;
-				break;
 			}
 		}
 	}
 	return {
-		.count = (app.unreadBadge() - active->session().data().unreadBadge()),
+		.count = counter,
 		.allMuted = allMuted,
 	};
 }
