@@ -9,13 +9,16 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "base/flags.h"
 #include "base/object_ptr.h"
-#include "base/observer.h"
 #include "base/weak_ptr.h"
 #include "base/timer.h"
+#include "boxes/gift_premium_box.h" // GiftPremiumValidator.
+#include "chat_helpers/compose/compose_show.h"
+#include "data/data_chat_participant_status.h"
 #include "dialogs/dialogs_key.h"
 #include "ui/layers/layer_widget.h"
-#include "ui/layers/show.h"
+#include "settings/settings_type.h"
 #include "window/window_adaptive.h"
+#include "mtproto/sender.h"
 
 class PhotoData;
 class MainWidget;
@@ -25,18 +28,26 @@ namespace Adaptive {
 enum class WindowLayout;
 } // namespace Adaptive
 
+namespace Data {
+struct StoriesContext;
+enum class StorySourcesList : uchar;
+} // namespace Data
+
 namespace ChatHelpers {
 class TabbedSelector;
 class EmojiInteractions;
+struct FileChosen;
 } // namespace ChatHelpers
 
 namespace Main {
 class Session;
 } // namespace Main
 
-namespace Settings {
-enum class Type;
-} // namespace Settings
+namespace InlineBots {
+class AttachWebView;
+enum class PeerType : uint8;
+using PeerTypes = base::flags<PeerType>;
+} // namespace InlineBots
 
 namespace Calls {
 struct StartGroupCallArgs;
@@ -62,6 +73,10 @@ class MessageSendingAnimationController;
 namespace Data {
 struct CloudTheme;
 enum class CloudThemeType;
+class Thread;
+class Forum;
+class ForumTopic;
+class WallPaper;
 } // namespace Data
 
 namespace HistoryView::Reactions {
@@ -70,25 +85,28 @@ class CachedIconFactory;
 
 namespace Window {
 
-class MainWindow;
+using GifPauseReason = ChatHelpers::PauseReason;
+using GifPauseReasons = ChatHelpers::PauseReasons;
+
 class SectionMemento;
 class Controller;
 class FiltersMenu;
 
-enum class GifPauseReason {
-	Any           = 0,
-	InlineResults = (1 << 0),
-	SavedGifs     = (1 << 1),
-	Layer         = (1 << 2),
-	RoundPlaying  = (1 << 3),
-	MediaPreview  = (1 << 4),
+enum class ResolveType {
+	Default,
+	BotApp,
+	BotStart,
+	AddToGroup,
+	AddToChannel,
+	ShareGame,
+	Mention,
+	Boost,
 };
-using GifPauseReasons = base::flags<GifPauseReason>;
-inline constexpr bool is_flag_type(GifPauseReason) { return true; };
 
 struct PeerThemeOverride {
 	PeerData *peer = nullptr;
 	std::shared_ptr<Ui::ChatTheme> theme;
+	EmojiPtr emoji = nullptr;
 };
 bool operator==(const PeerThemeOverride &a, const PeerThemeOverride &b);
 bool operator!=(const PeerThemeOverride &a, const PeerThemeOverride &b);
@@ -102,6 +120,7 @@ public:
 
 private:
 	Dialogs::Key _chat;
+	base::weak_ptr<Data::ForumTopic> _weak;
 	QDate _date;
 
 };
@@ -133,19 +152,26 @@ struct SectionShow {
 	, activation(activation) {
 	}
 
-	SectionShow withWay(Way newWay) const {
+	[[nodiscard]] SectionShow withWay(Way newWay) const {
 		return SectionShow(newWay, animated, activation);
 	}
-	SectionShow withThirdColumn() const {
+	[[nodiscard]] SectionShow withThirdColumn() const {
 		auto copy = *this;
 		copy.thirdColumn = true;
 		return copy;
 	}
+	[[nodiscard]] SectionShow withChildColumn() const {
+		auto copy = *this;
+		copy.childColumn = true;
+		return copy;
+	}
 
 	Way way = Way::Forward;
 	anim::type animated = anim::type::normal;
 	anim::activation activation = anim::activation::normal;
 	bool thirdColumn = false;
+	bool childColumn = false;
+	bool reapplyLocalDraft = false;
 	Origin origin;
 
 };
@@ -157,7 +183,7 @@ public:
 	explicit SessionNavigation(not_null<Main::Session*> session);
 	virtual ~SessionNavigation();
 
-	Main::Session &session() const;
+	[[nodiscard]] Main::Session &session() const;
 
 	virtual void showSection(
 		std::shared_ptr<SectionMemento> memento,
@@ -177,10 +203,21 @@ public:
 		std::variant<QString, ChannelId> usernameOrId;
 		QString phone;
 		MsgId messageId = ShowAtUnreadMsgId;
+		StoryId storyId = 0;
 		RepliesByLinkInfo repliesInfo;
+		ResolveType resolveType = ResolveType::Default;
 		QString startToken;
+		ChatAdminRights startAdminRights;
+		bool startAutoSubmit = false;
+		QString botAppName;
+		bool botAppForceConfirmation = false;
+		QString attachBotUsername;
+		std::optional<QString> attachBotToggleCommand;
+		bool attachBotMenuOpen = false;
+		InlineBots::PeerTypes attachBotChooseTypes;
 		std::optional<QString> voicechatHash;
 		FullMsgId clickFromMessageId;
+		QString clickFromAttachBotWebviewUrl;
 	};
 	void showPeerByLink(const PeerByLinkInfo &info);
 
@@ -189,6 +226,14 @@ public:
 		MsgId rootId,
 		MsgId commentId = 0,
 		const SectionShow &params = SectionShow());
+	void showTopic(
+		not_null<Data::ForumTopic*> topic,
+		MsgId itemId = 0,
+		const SectionShow &params = SectionShow());
+	void showThread(
+		not_null<Data::Thread*> thread,
+		MsgId itemId = 0,
+		const SectionShow &params = SectionShow());
 
 	void showPeerInfo(
 		PeerId peerId,
@@ -197,7 +242,7 @@ public:
 		not_null<PeerData*> peer,
 		const SectionShow &params = SectionShow());
 	void showPeerInfo(
-		not_null<History*> history,
+		not_null<Data::Thread*> thread,
 		const SectionShow &params = SectionShow());
 
 	virtual void showPeerHistory(
@@ -231,6 +276,16 @@ public:
 		FullMsgId contextId,
 		const SectionShow &params = SectionShow());
 
+	base::weak_ptr<Ui::Toast::Instance> showToast(
+		Ui::Toast::Config &&config);
+	base::weak_ptr<Ui::Toast::Instance> showToast(
+		TextWithEntities &&text,
+		crl::time duration = 0);
+	base::weak_ptr<Ui::Toast::Instance> showToast(
+		const QString &text,
+		crl::time duration = 0);
+
+	[[nodiscard]] virtual std::shared_ptr<ChatHelpers::Show> uiShow();
 
 private:
 	void resolvePhone(
@@ -247,18 +302,38 @@ private:
 		const MTPcontacts_ResolvedPeer &result,
 		Fn<void(not_null<PeerData*>)> done);
 
+	void showMessageByLinkResolved(
+		not_null<HistoryItem*> item,
+		const PeerByLinkInfo &info);
 	void showPeerByLinkResolved(
 		not_null<PeerData*> peer,
 		const PeerByLinkInfo &info);
+	void joinVoiceChatFromLink(
+		not_null<PeerData*> peer,
+		const PeerByLinkInfo &info);
+
+	void resolveBoostState(not_null<ChannelData*> channel);
+	void applyBoost(not_null<ChannelData*> channel, Fn<void(bool)> done);
+	void replaceBoostConfirm(
+		not_null<PeerData*> from,
+		not_null<ChannelData*> channel,
+		Fn<void(bool)> done);
+	void applyBoostChecked(
+		not_null<ChannelData*> channel,
+		Fn<void(bool)> done);
 
 	const not_null<Main::Session*> _session;
 
+	MTP::Sender _api;
+
 	mtpRequestId _resolveRequestId = 0;
 
 	History *_showingRepliesHistory = nullptr;
 	MsgId _showingRepliesRootId = 0;
 	mtpRequestId _showingRepliesRequestId = 0;
 
+	ChannelData *_boostStateResolving = nullptr;
+
 };
 
 class SessionController : public SessionNavigation {
@@ -280,23 +355,17 @@ public:
 		return *_emojiInteractions;
 	}
 
-	// We need access to this from MainWidget::MainWidget, where
-	// we can't call content() yet.
-	void setSelectingPeer(bool selecting) {
-		_selectingPeer = selecting;
-	}
-	[[nodiscard]] bool selectingPeer() const {
-		return _selectingPeer;
-	}
-
 	void setConnectingBottomSkip(int skip);
 	rpl::producer<int> connectingBottomSkipValue() const;
 
+	using FileChosen = ChatHelpers::FileChosen;
+	void stickerOrEmojiChosen(FileChosen chosen);
+	[[nodiscard]] rpl::producer<FileChosen> stickerOrEmojiChosen() const;
+
 	QPointer<Ui::BoxContent> show(
 		object_ptr<Ui::BoxContent> content,
 		Ui::LayerOptions options = Ui::LayerOption::KeepOther,
 		anim::type animated = anim::type::normal);
-
 	void hideLayer(anim::type animated = anim::type::normal);
 
 	[[nodiscard]] auto sendingAnimation() const
@@ -310,10 +379,17 @@ public:
 	// is changed in the Dialogs::Widget of the current window.
 	rpl::variable<Dialogs::Key> searchInChat;
 	bool uniqueChatsInSearchResults() const;
+
 	void openFolder(not_null<Data::Folder*> folder);
 	void closeFolder();
 	const rpl::variable<Data::Folder*> &openedFolder() const;
 
+	void showForum(
+		not_null<Data::Forum*> forum,
+		const SectionShow &params = SectionShow::Way::ClearStack);
+	void closeForum();
+	const rpl::variable<Data::Forum*> &shownForum() const;
+
 	void setActiveChatEntry(Dialogs::RowDescriptor row);
 	void setActiveChatEntry(Dialogs::Key key);
 	Dialogs::RowDescriptor activeChatEntryCurrent() const;
@@ -324,12 +400,24 @@ public:
 	rpl::producer<Dialogs::Key> activeChatValue() const;
 	bool jumpToChatListEntry(Dialogs::RowDescriptor row);
 
+	void setCurrentDialogsEntryState(Dialogs::EntryState state);
+	[[nodiscard]] Dialogs::EntryState currentDialogsEntryState() const;
+	bool switchInlineQuery(
+		Dialogs::EntryState to,
+		not_null<UserData*> bot,
+		const QString &query);
+	bool switchInlineQuery(
+		not_null<Data::Thread*> thread,
+		not_null<UserData*> bot,
+		const QString &query);
+
 	[[nodiscard]] Dialogs::RowDescriptor resolveChatNext(
 		Dialogs::RowDescriptor from = {}) const;
 	[[nodiscard]] Dialogs::RowDescriptor resolveChatPrevious(
 		Dialogs::RowDescriptor from = {}) const;
 
 	void showEditPeerBox(PeerData *peer);
+	void showGiftPremiumBox(UserData *user);
 
 	void enableGifPauseReason(GifPauseReason reason);
 	void disableGifPauseReason(GifPauseReason reason);
@@ -356,11 +444,13 @@ public:
 	void resizeForThirdSection();
 	void closeThirdSection();
 
+	[[nodiscard]] bool canShowSeparateWindow(not_null<PeerData*> peer) const;
 	void showPeer(not_null<PeerData*> peer, MsgId msgId = ShowAtUnreadMsgId);
 
+	void startOrJoinGroupCall(not_null<PeerData*> peer);
 	void startOrJoinGroupCall(
 		not_null<PeerData*> peer,
-		const Calls::StartGroupCallArgs &args);
+		Calls::StartGroupCallArgs args);
 
 	void showSection(
 		std::shared_ptr<SectionMemento> memento,
@@ -374,7 +464,9 @@ public:
 		const SectionShow &params = SectionShow::Way::ClearStack,
 		MsgId msgId = ShowAtUnreadMsgId) override;
 
-	void showPeerHistoryAtItem(not_null<const HistoryItem*> item);
+	void showMessage(
+		not_null<const HistoryItem*> item,
+		const SectionShow &params = SectionShow::Way::ClearStack);
 	void cancelUploadLayer(not_null<HistoryItem*> item);
 
 	void showLayer(
@@ -390,6 +482,7 @@ public:
 		showSpecialLayer(nullptr, animated);
 	}
 	void removeLayerBlackout();
+	[[nodiscard]] bool isLayerShown() const;
 
 	void showCalendar(
 		Dialogs::Key chat,
@@ -402,12 +495,24 @@ public:
 	void showPassportForm(const Passport::FormRequest &request);
 	void clearPassportForm();
 
-	void openPhoto(not_null<PhotoData*> photo, FullMsgId contextId);
+	struct MessageContext {
+		FullMsgId id;
+		MsgId topicRootId;
+	};
+	void openPhoto(
+		not_null<PhotoData*> photo,
+		MessageContext message,
+		const Data::StoriesContext *stories = nullptr);
 	void openPhoto(not_null<PhotoData*> photo, not_null<PeerData*> peer);
 	void openDocument(
 		not_null<DocumentData*> document,
-		FullMsgId contextId,
-		bool showInMediaView = false);
+		bool showInMediaView,
+		MessageContext message,
+		const Data::StoriesContext *stories = nullptr);
+	bool openSharedStory(HistoryItem *item);
+	bool openFakeItemStory(
+		FullMsgId fakeItemId,
+		const Data::StoriesContext *stories = nullptr);
 
 	void showChooseReportMessages(
 		not_null<PeerData*> peer,
@@ -415,19 +520,33 @@ public:
 		Fn<void(MessageIdsList)> done);
 	void clearChooseReportMessages();
 
-	void toggleChooseChatTheme(not_null<PeerData*> peer);
+	void showInNewWindow(
+		not_null<PeerData*> peer,
+		MsgId msgId = ShowAtUnreadMsgId);
 
-	base::Variable<bool> &dialogsListFocused() {
-		return _dialogsListFocused;
+	void toggleChooseChatTheme(
+		not_null<PeerData*> peer,
+		std::optional<bool> show = std::nullopt);
+	void finishChatThemeEdit(not_null<PeerData*> peer);
+
+	[[nodiscard]] bool dialogsListFocused() const {
+		return _dialogsListFocused.current();
 	}
-	const base::Variable<bool> &dialogsListFocused() const {
-		return _dialogsListFocused;
+	[[nodiscard]] rpl::producer<bool> dialogsListFocusedChanges() const {
+		return _dialogsListFocused.changes();
 	}
-	base::Variable<bool> &dialogsListDisplayForced() {
-		return _dialogsListDisplayForced;
+	void setDialogsListFocused(bool value) {
+		_dialogsListFocused = value;
 	}
-	const base::Variable<bool> &dialogsListDisplayForced() const {
-		return _dialogsListDisplayForced;
+	[[nodiscard]] bool dialogsListDisplayForced() const {
+		return _dialogsListDisplayForced.current();
+	}
+	[[nodiscard]] auto dialogsListDisplayForcedChanges() const
+	-> rpl::producer<bool> {
+		return _dialogsListDisplayForced.changes();
+	}
+	void setDialogsListDisplayForced(bool value) {
+		_dialogsListDisplayForced = value;
 	}
 
 	not_null<SessionController*> parentController() override {
@@ -437,7 +556,9 @@ public:
 	[[nodiscard]] int filtersWidth() const;
 	[[nodiscard]] rpl::producer<FilterId> activeChatsFilter() const;
 	[[nodiscard]] FilterId activeChatsFilterCurrent() const;
-	void setActiveChatsFilter(FilterId id);
+	void setActiveChatsFilter(
+		FilterId id,
+		const SectionShow &params = SectionShow::Way::ClearStack);
 
 	void toggleFiltersMenu(bool enabled);
 	[[nodiscard]] rpl::producer<> filtersMenuChanged() const;
@@ -448,21 +569,36 @@ public:
 	}
 	[[nodiscard]] auto cachedChatThemeValue(
 		const Data::CloudTheme &data,
+		const Data::WallPaper &paper,
 		Data::CloudThemeType type)
 	-> rpl::producer<std::shared_ptr<Ui::ChatTheme>>;
+	[[nodiscard]] bool chatThemeAlreadyCached(
+		const Data::CloudTheme &data,
+		const Data::WallPaper &paper,
+		Data::CloudThemeType type);
 	void setChatStyleTheme(const std::shared_ptr<Ui::ChatTheme> &theme);
 	void clearCachedChatThemes();
 	void pushLastUsedChatTheme(const std::shared_ptr<Ui::ChatTheme> &theme);
+	[[nodiscard]] not_null<Ui::ChatTheme*> currentChatTheme() const;
 
 	void overridePeerTheme(
 		not_null<PeerData*> peer,
-		std::shared_ptr<Ui::ChatTheme> theme);
+		std::shared_ptr<Ui::ChatTheme> theme,
+		EmojiPtr emoji);
 	void clearPeerThemeOverride(not_null<PeerData*> peer);
 	[[nodiscard]] auto peerThemeOverrideValue() const
 		-> rpl::producer<PeerThemeOverride> {
 		return _peerThemeOverride.value();
 	}
 
+	void openPeerStory(
+		not_null<PeerData*> peer,
+		StoryId storyId,
+		Data::StoriesContext context);
+	void openPeerStories(
+		PeerId peerId,
+		std::optional<Data::StorySourcesList> list = std::nullopt);
+
 	struct PaintContextArgs {
 		not_null<Ui::ChatTheme*> theme;
 		int visibleAreaTop = 0;
@@ -481,11 +617,23 @@ public:
 		return *_cachedReactionIconFactory;
 	}
 
-	rpl::lifetime &lifetime() {
+	[[nodiscard]] QString authedName() const {
+		return _authedName;
+	}
+
+	void setPremiumRef(const QString &ref);
+	[[nodiscard]] QString premiumRef() const;
+
+	[[nodiscard]] bool contentOverlapped(QWidget *w, QPaintEvent *e);
+
+	[[nodiscard]] std::shared_ptr<ChatHelpers::Show> uiShow() override;
+
+	[[nodiscard]] rpl::lifetime &lifetime() {
 		return _lifetime;
 	}
 
 private:
+	struct CachedThemeKey;
 	struct CachedTheme;
 
 	void init();
@@ -493,6 +641,7 @@ private:
 	void refreshFiltersMenu();
 	void checkOpenedFilter();
 	void suggestArchiveAndMute();
+	void activateFirstChatsFilter();
 
 	int minimalThreeColumnWidth() const;
 	int countDialogsWidthFromRatio(int bodyWidth) const;
@@ -511,10 +660,13 @@ private:
 	void resetFakeUnreadWhileOpened();
 
 	void checkInvitePeek();
+	void setupPremiumToast();
 
 	void pushDefaultChatBackground();
 	void cacheChatTheme(
+		CachedThemeKey key,
 		const Data::CloudTheme &data,
+		const Data::WallPaper &paper,
 		Data::CloudThemeType type);
 	void cacheChatThemeDone(std::shared_ptr<Ui::ChatTheme> result);
 	void updateCustomThemeBackground(CachedTheme &theme);
@@ -524,6 +676,11 @@ private:
 
 	const not_null<Controller*> _window;
 	const std::unique_ptr<ChatHelpers::EmojiInteractions> _emojiInteractions;
+	const bool _isPrimary = false;
+
+	mutable std::shared_ptr<ChatHelpers::Show> _cachedShow;
+
+	QString _authedName;
 
 	using SendingAnimation = Ui::MessageSendingAnimationController;
 	const std::unique_ptr<SendingAnimation> _sendingAnimation;
@@ -538,11 +695,14 @@ private:
 	const std::unique_ptr<ChatHelpers::TabbedSelector> _tabbedSelector;
 
 	rpl::variable<Dialogs::RowDescriptor> _activeChatEntry;
-	base::Variable<bool> _dialogsListFocused = { false };
-	base::Variable<bool> _dialogsListDisplayForced = { false };
+	rpl::lifetime _activeHistoryLifetime;
+	rpl::variable<bool> _dialogsListFocused = false;
+	rpl::variable<bool> _dialogsListDisplayForced = false;
 	std::deque<Dialogs::RowDescriptor> _chatEntryHistory;
 	int _chatEntryHistoryPosition = -1;
-	bool _selectingPeer = false;
+	bool _filtersActivated = false;
+
+	Dialogs::EntryState _currentDialogsEntryState;
 
 	base::Timer _invitePeekTimer;
 
@@ -550,13 +710,17 @@ private:
 
 	rpl::variable<int> _connectingBottomSkip;
 
+	rpl::event_stream<ChatHelpers::FileChosen> _stickerOrEmojiChosen;
+
 	PeerData *_showEditPeer = nullptr;
 	rpl::variable<Data::Folder*> _openedFolder;
+	rpl::variable<Data::Forum*> _shownForum;
+	rpl::lifetime _shownForumLifetime;
 
 	rpl::event_stream<> _filtersMenuChanged;
 
 	std::shared_ptr<Ui::ChatTheme> _defaultChatTheme;
-	base::flat_map<Ui::ChatThemeKey, CachedTheme> _customChatThemes;
+	base::flat_map<CachedThemeKey, CachedTheme> _customChatThemes;
 	rpl::event_stream<std::shared_ptr<Ui::ChatTheme>> _cachedThemesStream;
 	const std::unique_ptr<Ui::ChatStyle> _chatStyle;
 	std::weak_ptr<Ui::ChatTheme> _chatStyleTheme;
@@ -566,26 +730,23 @@ private:
 	using ReactionIconFactory = HistoryView::Reactions::CachedIconFactory;
 	std::unique_ptr<ReactionIconFactory> _cachedReactionIconFactory;
 
+	base::has_weak_ptr _storyOpenGuard;
+
+	GiftPremiumValidator _giftPremiumValidator;
+
+	QString _premiumRef;
+
 	rpl::lifetime _lifetime;
 
 };
 
 void ActivateWindow(not_null<SessionController*> controller);
 
-class Show : public Ui::Show {
-public:
-	explicit Show(not_null<SessionNavigation*> navigation);
-	explicit Show(not_null<Controller*> window);
-	~Show();
-	void showBox(
-		object_ptr<Ui::BoxContent> content,
-		Ui::LayerOptions options = Ui::LayerOption::KeepOther) const override;
-	void hideLayer() const override;
-	[[nodiscard]] not_null<QWidget*> toastParent() const override;
-	[[nodiscard]] bool valid() const override;
-	operator bool() const override;
-private:
-	const base::weak_ptr<Controller> _window;
-};
+[[nodiscard]] bool IsPaused(
+	not_null<SessionController*> controller,
+	GifPauseReason level);
+[[nodiscard]] Fn<bool()> PausedIn(
+	not_null<SessionController*> controller,
+	GifPauseReason level);
 
 } // namespace Window
