@@ -93,6 +93,7 @@ std::string logType(const structure::Type &type) {
 	}
 	static auto builtInTypes = new QMap<structure::TypeTag, std::string> {
 		{ structure::TypeTag::Int       , "int" },
+		{ structure::TypeTag::Bool      , "bool" },
 		{ structure::TypeTag::Double    , "double" },
 		{ structure::TypeTag::Pixels    , "pixels" },
 		{ structure::TypeTag::String    , "string" },
@@ -304,6 +305,8 @@ structure::Value ParsedFile::readValue() {
 		return pointValue;
 	} else if (auto sizeValue = readSizeValue()) {
 		return sizeValue;
+	} else if (auto boolValue = readBoolValue()) {
+		return boolValue;
 	} else if (auto alignValue = readAlignValue()) {
 		return alignValue;
 	} else if (auto marginsValue = readMarginsValue()) {
@@ -568,8 +571,8 @@ structure::Value ParsedFile::readColorValue() {
 }
 
 structure::Value ParsedFile::readPointValue() {
-	if (auto font = file_.getToken(BasicType::Name)) {
-		if (tokenValue(font) == "point") {
+	if (auto name = file_.getToken(BasicType::Name)) {
+		if (tokenValue(name) == "point") {
 			assertNextToken(BasicType::LeftParenthesis);
 
 			auto x = readNumericOrNumericCopyValue(); assertNextToken(BasicType::Comma);
@@ -589,8 +592,8 @@ structure::Value ParsedFile::readPointValue() {
 }
 
 structure::Value ParsedFile::readSizeValue() {
-	if (auto font = file_.getToken(BasicType::Name)) {
-		if (tokenValue(font) == "size") {
+	if (auto name = file_.getToken(BasicType::Name)) {
+		if (tokenValue(name) == "size") {
 			assertNextToken(BasicType::LeftParenthesis);
 
 			auto w = readNumericOrNumericCopyValue(); assertNextToken(BasicType::Comma);
@@ -609,9 +612,21 @@ structure::Value ParsedFile::readSizeValue() {
 	return {};
 }
 
+structure::Value ParsedFile::readBoolValue() {
+	if (auto value = file_.getToken(BasicType::Name)) {
+		if (tokenValue(value) == "true") {
+			return { structure::TypeTag::Bool, true };
+		} else if (tokenValue(value) == "false") {
+			return { structure::TypeTag::Bool, false };
+		}
+		file_.putBack();
+	}
+	return {};
+}
+
 structure::Value ParsedFile::readAlignValue() {
-	if (auto font = file_.getToken(BasicType::Name)) {
-		if (tokenValue(font) == "align") {
+	if (auto name = file_.getToken(BasicType::Name)) {
+		if (tokenValue(name) == "align") {
 			assertNextToken(BasicType::LeftParenthesis);
 
 			auto align = tokenValue(assertNextToken(BasicType::Name));
@@ -630,8 +645,8 @@ structure::Value ParsedFile::readAlignValue() {
 }
 
 structure::Value ParsedFile::readMarginsValue() {
-	if (auto font = file_.getToken(BasicType::Name)) {
-		if (tokenValue(font) == "margins") {
+	if (auto name = file_.getToken(BasicType::Name)) {
+		if (tokenValue(name) == "margins") {
 			assertNextToken(BasicType::LeftParenthesis);
 
 			auto l = readNumericOrNumericCopyValue(); assertNextToken(BasicType::Comma);
@@ -655,8 +670,8 @@ structure::Value ParsedFile::readMarginsValue() {
 }
 
 structure::Value ParsedFile::readFontValue() {
-	if (auto font = file_.getToken(BasicType::Name)) {
-		if (tokenValue(font) == "font") {
+	if (auto name = file_.getToken(BasicType::Name)) {
+		if (tokenValue(name) == "font") {
 			assertNextToken(BasicType::LeftParenthesis);
 
 			int flags = 0;
@@ -701,8 +716,8 @@ structure::Value ParsedFile::readFontValue() {
 }
 
 structure::Value ParsedFile::readIconValue() {
-	if (auto font = file_.getToken(BasicType::Name)) {
-		if (tokenValue(font) == "icon") {
+	if (auto name = file_.getToken(BasicType::Name)) {
+		if (tokenValue(name) == "icon") {
 			std::vector<structure::data::monoicon> parts;
 			if (file_.getToken(BasicType::LeftBrace)) { // complex icon
 				do {
