@@ -7,6 +7,7 @@
 #include "base/platform/linux/base_power_save_blocker_linux.h"
 
 #include "base/platform/base_platform_info.h"
+#include "base/platform/linux/base_linux_xdp_utilities.h"
 #include "base/platform/linux/base_linux_wayland_integration.h"
 #include "base/timer_rpl.h"
 #include "base/random.h"
@@ -15,13 +16,9 @@
 #include "base/platform/linux/base_linux_xcb_utilities.h"
 #endif // !DESKTOP_APP_DISABLE_X11_INTEGRATION
 
-#ifndef DESKTOP_APP_DISABLE_DBUS_INTEGRATION
-#include "base/platform/linux/base_linux_xdp_utilities.h"
+#include <QtGui/QWindow>
 
 #include <giomm.h>
-#endif // !DESKTOP_APP_DISABLE_DBUS_INTEGRATION
-
-#include <QtGui/QWindow>
 
 namespace base::Platform {
 namespace {
@@ -42,9 +39,8 @@ void XCBPreventDisplaySleep(bool prevent) {
 
 	base::timer_each(
 		kResetScreenSaverTimeout
-	) | rpl::start_with_next([] {
-		const auto connection = XCB::GetConnectionFromQt();
-		if (!connection) {
+	) | rpl::start_with_next([connection = XCB::Connection()] {
+		if (!connection || xcb_connection_has_error(connection)) {
 			return;
 		}
 		xcb_force_screen_saver(connection, XCB_SCREEN_SAVER_RESET);
@@ -52,28 +48,30 @@ void XCBPreventDisplaySleep(bool prevent) {
 }
 #endif // !DESKTOP_APP_DISABLE_X11_INTEGRATION
 
-#ifndef DESKTOP_APP_DISABLE_DBUS_INTEGRATION
-void PortalPreventAppSuspension(
-	bool prevent,
-	const QString &description = {},
-	QPointer<QWindow> window = {}) {
+/* https://flatpak.github.io/xdg-desktop-portal/#gdbus-org.freedesktop.portal.Inhibit
+ * 1: Logout
+ * 2: User Switch
+ * 4: Suspend
+ * 8: Idle
+ */
+void PortalInhibit(
+	Glib::ustring &requestPath,
+	uint flags = 0,
+	const QString &description = {}) {
 	try {
 		const auto connection = Gio::DBus::Connection::get_sync(
 			Gio::DBus::BusType::SESSION);
 
-		static Glib::ustring requestPath;
-		if (!prevent && !requestPath.empty()) {
+		if (!requestPath.empty()) {
 			connection->call(
 				requestPath,
-				std::string(XDP::kRequestInterface),
+				XDP::kRequestInterface,
 				"Close",
 				{},
 				{},
-				std::string(XDP::kService));
+				XDP::kService);
 			requestPath = "";
 			return;
-		} else if (!(prevent && requestPath.empty())) {
-			return;
 		}
 
 		const auto handleToken = Glib::ustring("desktop_app")
@@ -90,34 +88,49 @@ void PortalPreventAppSuspension(
 			+ handleToken;
 
 		connection->call(
-			std::string(XDP::kObjectPath),
+			XDP::kObjectPath,
 			"org.freedesktop.portal.Inhibit",
 			"Inhibit",
-			Glib::VariantContainerBase::create_tuple({
-				Glib::Variant<Glib::ustring>::create(
-					XDP::ParentWindowID(window)),
-				Glib::Variant<uint>::create(4), // Suspend
-				Glib::Variant<std::map<
-					Glib::ustring,
-					Glib::VariantBase
-				>>::create({
+			Glib::create_variant(std::tuple{
+				XDP::ParentWindowID(),
+				flags,
+				std::map<Glib::ustring, Glib::VariantBase>{
 					{
 						"handle_token",
-						Glib::Variant<Glib::ustring>::create(handleToken)
+						Glib::create_variant(handleToken)
 					},
 					{
 						"reason",
-						Glib::Variant<Glib::ustring>::create(
-							description.toStdString())
+						Glib::create_variant(
+							Glib::ustring(description.toStdString()))
 					},
-				}),
+				},
 			}),
 			{},
-			std::string(XDP::kService));
+			XDP::kService);
 	} catch (...) {
 	}
 }
-#endif // !DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+
+void PortalPreventDisplaySleep(
+	bool prevent,
+	const QString &description = {}) {
+	static Glib::ustring requestPath;
+	if (prevent && !requestPath.empty()) {
+		return;
+	}
+	PortalInhibit(requestPath, 8 /* Idle */, description);
+}
+
+void PortalPreventAppSuspension(
+	bool prevent,
+	const QString &description = {}) {
+	static Glib::ustring requestPath;
+	if (prevent && !requestPath.empty()) {
+		return;
+	}
+	PortalInhibit(requestPath, 4 /* Suspend */, description);
+}
 
 } // namespace
 
@@ -127,18 +140,16 @@ void BlockPowerSave(
 	QPointer<QWindow> window) {
 	switch (type) {
 	case PowerSaveBlockType::PreventAppSuspension:
-#ifndef DESKTOP_APP_DISABLE_DBUS_INTEGRATION
-		PortalPreventAppSuspension(true, description, window);
-#endif // !DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+		PortalPreventAppSuspension(true, description);
 		break;
 	case PowerSaveBlockType::PreventDisplaySleep:
 		if (const auto integration = WaylandIntegration::Instance()) {
 			integration->preventDisplaySleep(true, window);
-		} else if (::Platform::IsX11()) {
-#ifndef DESKTOP_APP_DISABLE_X11_INTEGRATION
-			XCBPreventDisplaySleep(true);
-#endif // !DESKTOP_APP_DISABLE_X11_INTEGRATION
 		}
+#ifndef DESKTOP_APP_DISABLE_X11_INTEGRATION
+		XCBPreventDisplaySleep(true);
+#endif // !DESKTOP_APP_DISABLE_X11_INTEGRATION
+		PortalPreventDisplaySleep(true, description);
 		break;
 	}
 }
@@ -146,18 +157,16 @@ void BlockPowerSave(
 void UnblockPowerSave(PowerSaveBlockType type, QPointer<QWindow> window) {
 	switch (type) {
 	case PowerSaveBlockType::PreventAppSuspension:
-#ifndef DESKTOP_APP_DISABLE_DBUS_INTEGRATION
 		PortalPreventAppSuspension(false);
-#endif // !DESKTOP_APP_DISABLE_DBUS_INTEGRATION
 		break;
 	case PowerSaveBlockType::PreventDisplaySleep:
 		if (const auto integration = WaylandIntegration::Instance()) {
 			integration->preventDisplaySleep(false, window);
-		} else if (::Platform::IsX11()) {
-#ifndef DESKTOP_APP_DISABLE_X11_INTEGRATION
-			XCBPreventDisplaySleep(false);
-#endif // !DESKTOP_APP_DISABLE_X11_INTEGRATION
 		}
+#ifndef DESKTOP_APP_DISABLE_X11_INTEGRATION
+		XCBPreventDisplaySleep(false);
+#endif // !DESKTOP_APP_DISABLE_X11_INTEGRATION
+		PortalPreventDisplaySleep(false);
 		break;
 	}
 }
