@@ -9,10 +9,11 @@
 #include "base/qthelp_url.h"
 #include "base/qthelp_regex.h"
 #include "base/crc32hash.h"
-#include "base/qt_adapters.h"
 #include "ui/text/text.h"
-#include "ui/widgets/input_fields.h"
+#include "ui/widgets/fields/input_field.h"
 #include "ui/emoji_config.h"
+#include "ui/basic_click_handlers.h"
+#include "base/qt/qt_common_adapters.h"
 
 #include <QtCore/QStack>
 #include <QtCore/QMimeData>
@@ -22,6 +23,8 @@
 namespace TextUtilities {
 namespace {
 
+constexpr auto kTagSeparator = '\\';
+
 using namespace Ui::Text;
 
 QString ExpressionMailNameAtEnd() {
@@ -68,6 +71,10 @@ QString SeparatorsMono() {
 	return Separators(QString::fromUtf8("*~/"));
 }
 
+QString SeparatorsSpoiler() {
+	return Separators(QString::fromUtf8("|*~/"));
+}
+
 QString ExpressionHashtag() {
 	return QString::fromUtf8("(^|[") + ExpressionSeparators(QString::fromUtf8("`\\*/")) + QString::fromUtf8("])#[\\w]{2,64}([\\W]|$)");
 }
@@ -88,9 +95,7 @@ QRegularExpression CreateRegExp(const QString &expression) {
 	auto result = QRegularExpression(
 		expression,
 		QRegularExpression::UseUnicodePropertiesOption);
-#ifndef OS_MAC_OLD
 	result.optimize();
-#endif // OS_MAC_OLD
 	return result;
 }
 
@@ -1146,7 +1151,7 @@ const QRegularExpression &RegExpWordSplit() {
 	auto &&urls = ranges::make_subrange(
 		entities.begin(),
 		entities.end()
-	) | ranges::view::filter([](const EntityInText &entity) {
+	) | ranges::views::filter([](const EntityInText &entity) {
 		return entity.type() == EntityType::CustomUrl;
 	});
 	const auto &original = text.text;
@@ -1158,13 +1163,13 @@ const QRegularExpression &RegExpWordSplit() {
 	for (const auto &entity : urls) {
 		const auto till = entity.offset() + entity.length();
 		if (till > offset) {
-			result.append(original.midRef(offset, till - offset));
+			result.append(base::StringViewMid(original, offset, till - offset));
 		}
 		result.append(qstr(" (")).append(entity.data()).append(')');
 		offset = till;
 	}
 	if (original.size() > offset) {
-		result.append(original.midRef(offset));
+		result.append(base::StringViewMid(original, offset));
 	}
 	return result;
 }
@@ -1231,6 +1236,10 @@ const QRegularExpression &RegExpBotCommand() {
 	return result;
 }
 
+QRegularExpression RegExpDigitsExclude() {
+	return QRegularExpression("[^\\d]");
+}
+
 QString MarkdownBoldGoodBefore() {
 	return SeparatorsBold();
 }
@@ -1271,6 +1280,14 @@ QString MarkdownPreBadAfter() {
 	return QString::fromLatin1("`");
 }
 
+QString MarkdownSpoilerGoodBefore() {
+	return SeparatorsSpoiler();
+}
+
+QString MarkdownSpoilerBadAfter() {
+	return QString::fromLatin1("|");
+}
+
 bool IsValidProtocol(const QString &protocol) {
 	static const auto list = CreateValidProtocols();
 	return list.contains(base::crc32(protocol.constData(), protocol.size() * sizeof(QChar)));
@@ -1281,27 +1298,17 @@ bool IsValidTopDomain(const QString &protocol) {
 	return list.contains(base::crc32(protocol.constData(), protocol.size() * sizeof(QChar)));
 }
 
-QString Clean(const QString &text) {
-	auto result = text;
-	for (auto s = text.unicode(), ch = s, e = text.unicode() + text.size(); ch != e; ++ch) {
-		if (*ch == TextCommand) {
-			result[int(ch - s)] = QChar::Space;
-		}
-	}
-	return result;
-}
-
 QString EscapeForRichParsing(const QString &text) {
 	QString result;
 	result.reserve(text.size());
 	auto s = text.constData(), ch = s;
 	for (const QChar *e = s + text.size(); ch != e; ++ch) {
-		if (*ch == TextCommand) {
-			if (ch > s) result.append(s, ch - s);
-			result.append(QChar::Space);
-			s = ch + 1;
-			continue;
-		}
+		// if (*ch == TextCommand) {
+		// 	if (ch > s) result.append(s, ch - s);
+		// 	result.append(QChar::Space);
+		// 	s = ch + 1;
+		// 	continue;
+		// }
 		if (ch->unicode() == '\\' || ch->unicode() == '[') {
 			if (ch > s) result.append(s, ch - s);
 			result.append('\\');
@@ -1315,7 +1322,7 @@ QString EscapeForRichParsing(const QString &text) {
 
 QString SingleLine(const QString &text) {
 	auto result = text;
-	auto s = text.unicode(), ch = s, e = text.unicode() + text.size();
+	auto s = text.unicode(), e = text.unicode() + text.size();
 
 	// Trim.
 	while (s < e && IsTrimmed(*s)) {
@@ -1329,7 +1336,7 @@ QString SingleLine(const QString &text) {
 	}
 
 	for (auto ch = s; ch != e; ++ch) {
-		if (IsNewline(*ch) || *ch == TextCommand) {
+		if (IsNewline(*ch)/* || *ch == TextCommand*/) {
 			result[int(ch - s)] = QChar::Space;
 		}
 	}
@@ -1404,8 +1411,7 @@ QStringList PrepareSearchWords(
 		auto list = clean.split(SplitterOverride
 			? *SplitterOverride
 			: RegExpWordSplit(),
-			base::QStringSkipEmptyParts);
-		auto size = list.size();
+			Qt::SkipEmptyParts);
 		result.reserve(list.size());
 		for (const auto &word : std::as_const(list)) {
 			auto trimmed = word.trimmed();
@@ -1482,7 +1488,7 @@ bool CutPart(TextWithEntities &sending, TextWithEntities &left, int32 limit) {
 		}
 
 		int elen = 0;
-		if (auto e = Ui::Emoji::Find(ch, end, &elen)) {
+		if (Ui::Emoji::Find(ch, end, &elen)) {
 			for (int i = 0; i < elen; ++i, ++ch, ++s) {
 				if (ch->isHighSurrogate() && i + 1 < elen && (ch + 1)->isLowSurrogate()) {
 					++ch;
@@ -1526,56 +1532,37 @@ bool CutPart(TextWithEntities &sending, TextWithEntities &left, int32 limit) {
 	return true;
 }
 
-bool textcmdStartsLink(const QChar *start, int32 len, int32 commandOffset) {
-	if (commandOffset + 2 < len) {
-		if (*(start + commandOffset + 1) == TextCommandLinkIndex) {
-			return (*(start + commandOffset + 2) != 0);
-		}
-		return (*(start + commandOffset + 1) != TextCommandLinkText);
+MentionNameFields MentionNameDataToFields(QStringView data) {
+	const auto components = data.split('.');
+	if (components.size() != 2) {
+		return {};
 	}
-	return false;
+	const auto parts = components[1].split(':');
+	if (parts.size() != 2) {
+		return {};
+	}
+	return {
+		.selfId = parts[1].toULongLong(),
+		.userId = components[0].toULongLong(),
+		.accessHash = parts[0].toULongLong(),
+	};
 }
 
-bool checkTagStartInCommand(const QChar *start, int32 len, int32 tagStart, int32 &commandOffset, bool &commandIsLink, bool &inLink) {
-	bool inCommand = false;
-	const QChar *commandEnd = start + commandOffset;
-	while (commandOffset < len && tagStart > commandOffset) { // skip commands, evaluating are we in link or not
-		commandEnd = textSkipCommand(start + commandOffset, start + len);
-		if (commandEnd > start + commandOffset) {
-			if (tagStart < (commandEnd - start)) {
-				inCommand = true;
-				break;
-			}
-			for (commandOffset = commandEnd - start; commandOffset < len; ++commandOffset) {
-				if (*(start + commandOffset) == TextCommand) {
-					inLink = commandIsLink;
-					commandIsLink = textcmdStartsLink(start, len, commandOffset);
-					break;
-				}
-			}
-			if (commandOffset >= len) {
-				inLink = commandIsLink;
-				commandIsLink = false;
-			}
-		} else {
-			break;
-		}
-	}
-	if (inCommand) {
-		commandOffset = commandEnd - start;
-	}
-	return inCommand;
+QString MentionNameDataFromFields(const MentionNameFields &fields) {
+	return u"%1.%2:%3"_q
+		.arg(fields.userId)
+		.arg(fields.accessHash)
+		.arg(fields.selfId);
 }
 
 TextWithEntities ParseEntities(const QString &text, int32 flags) {
-	const auto rich = ((flags & TextParseRichText) != 0);
 	auto result = TextWithEntities{ text, EntitiesInText() };
-	ParseEntities(result, flags, rich);
+	ParseEntities(result, flags);
 	return result;
 }
 
 // Some code is duplicated in message_field.cpp!
-void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
+void ParseEntities(TextWithEntities &result, int32 flags) {
 	constexpr auto kNotFound = std::numeric_limits<int>::max();
 
 	auto newEntities = EntitiesInText();
@@ -1586,20 +1573,10 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 	int existingEntityIndex = 0, existingEntitiesCount = result.entities.size();
 	int existingEntityEnd = 0;
 
-	int32 len = result.text.size(), commandOffset = rich ? 0 : len;
-	bool inLink = false, commandIsLink = false;
+	int32 len = result.text.size();
 	const auto start = result.text.constData();
 	const auto end = start + result.text.size();
 	for (int32 offset = 0, matchOffset = offset, mentionSkip = 0; offset < len;) {
-		if (commandOffset <= offset) {
-			for (commandOffset = offset; commandOffset < len; ++commandOffset) {
-				if (*(start + commandOffset) == TextCommand) {
-					inLink = commandIsLink;
-					commandIsLink = textcmdStartsLink(start, len, commandOffset);
-					break;
-				}
-			}
-		}
 		auto mDomain = qthelp::RegExpDomain().match(result.text, matchOffset);
 		auto mExplicitDomain = qthelp::RegExpDomainExplicit().match(result.text, matchOffset);
 		auto mHashtag = withHashtags ? RegExpHashtag().match(result.text, matchOffset) : QRegularExpressionMatch();
@@ -1622,10 +1599,10 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 		auto mentionIgnore = false;
 
 		if (mHashtag.hasMatch()) {
-			if (!mHashtag.capturedRef(1).isEmpty()) {
+			if (!mHashtag.capturedView(1).isEmpty()) {
 				++hashtagStart;
 			}
-			if (!mHashtag.capturedRef(2).isEmpty()) {
+			if (!mHashtag.capturedView(2).isEmpty()) {
 				--hashtagEnd;
 			}
 			if (RegExpHashtagExclude().match(
@@ -1636,10 +1613,10 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			}
 		}
 		while (mMention.hasMatch()) {
-			if (!mMention.capturedRef(1).isEmpty()) {
+			if (!mMention.capturedView(1).isEmpty()) {
 				++mentionStart;
 			}
-			if (!mMention.capturedRef(2).isEmpty()) {
+			if (!mMention.capturedView(2).isEmpty()) {
 				--mentionEnd;
 			}
 			if (!(start + mentionStart + 1)->isLetter() || !(start + mentionEnd - 1)->isLetterOrNumber()) {
@@ -1660,10 +1637,10 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			}
 		}
 		if (mBotCommand.hasMatch()) {
-			if (!mBotCommand.capturedRef(1).isEmpty()) {
+			if (!mBotCommand.capturedView(1).isEmpty()) {
 				++botCommandStart;
 			}
-			if (!mBotCommand.capturedRef(3).isEmpty()) {
+			if (!mBotCommand.capturedView(3).isEmpty()) {
 				--botCommandEnd;
 			}
 		}
@@ -1687,17 +1664,6 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 				offset = matchOffset = mentionEnd;
 				continue;
 			}
-			const auto inCommand = checkTagStartInCommand(
-				start,
-				len,
-				mentionStart,
-				commandOffset,
-				commandIsLink,
-				inLink);
-			if (inCommand || inLink) {
-				offset = matchOffset = commandOffset;
-				continue;
-			}
 
 			lnkType = EntityType::Mention;
 			lnkStart = mentionStart;
@@ -1708,50 +1674,15 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 				offset = matchOffset = hashtagEnd;
 				continue;
 			}
-			const auto inCommand = checkTagStartInCommand(
-				start,
-				len,
-				hashtagStart,
-				commandOffset,
-				commandIsLink,
-				inLink);
-			if (inCommand || inLink) {
-				offset = matchOffset = commandOffset;
-				continue;
-			}
 
 			lnkType = EntityType::Hashtag;
 			lnkStart = hashtagStart;
 			lnkLength = hashtagEnd - hashtagStart;
 		} else if (botCommandStart < domainStart) {
-			const auto inCommand = checkTagStartInCommand(
-				start,
-				len,
-				botCommandStart,
-				commandOffset,
-				commandIsLink,
-				inLink);
-			if (inCommand || inLink) {
-				offset = matchOffset = commandOffset;
-				continue;
-			}
-
 			lnkType = EntityType::BotCommand;
 			lnkStart = botCommandStart;
 			lnkLength = botCommandEnd - botCommandStart;
 		} else {
-			const auto inCommand = checkTagStartInCommand(
-				start,
-				len,
-				domainStart,
-				commandOffset,
-				commandIsLink,
-				inLink);
-			if (inCommand || inLink) {
-				offset = matchOffset = commandOffset;
-				continue;
-			}
-
 			auto protocol = mDomain.captured(1).toLower();
 			auto topDomain = mDomain.captured(3).toLower();
 			auto isProtocolValid = protocol.isEmpty() || IsValidProtocol(protocol);
@@ -2022,63 +1953,258 @@ QString TagsTextMimeType() {
 	return QString::fromLatin1("application/x-td-field-text");
 }
 
-bool IsMentionLink(const QString &link) {
+bool IsMentionLink(QStringView link) {
 	return link.startsWith(kMentionTagStart);
 }
 
-EntitiesInText ConvertTextTagsToEntities(const TextWithTags::Tags &tags) {
-	EntitiesInText result;
-	if (tags.isEmpty()) {
-		return result;
-	}
+QString MentionEntityData(QStringView link) {
+	const auto match = qthelp::regex_match(
+		"^(\\d+\\.\\d+:\\d+)(/|$)",
+		base::StringViewMid(link, kMentionTagStart.size()));
+	return match ? match->captured(1) : QString();
+}
 
-	result.reserve(tags.size());
-	for (const auto &tag : tags) {
-		const auto push = [&](
-				EntityType type,
-				const QString &data = QString()) {
-			result.push_back(
-				EntityInText(type, tag.offset, tag.length, data));
-		};
-		if (IsMentionLink(tag.id)) {
-			if (auto match = qthelp::regex_match("^(\\d+\\.\\d+)(/|$)", tag.id.midRef(kMentionTagStart.size()))) {
-				push(EntityType::MentionName, match->captured(1));
-			}
-		} else if (tag.id == Ui::InputField::kTagBold) {
-			push(EntityType::Bold);
-		//} else if (tag.id == Ui::InputField::kTagSemibold) {
-		//	push(EntityType::Semibold); // Semibold is for UI parts only.
-		} else if (tag.id == Ui::InputField::kTagItalic) {
-			push(EntityType::Italic);
-		} else if (tag.id == Ui::InputField::kTagUnderline) {
-			push(EntityType::Underline);
-		} else if (tag.id == Ui::InputField::kTagStrikeOut) {
-			push(EntityType::StrikeOut);
-		} else if (tag.id == Ui::InputField::kTagCode) {
-			push(EntityType::Code);
-		} else if (tag.id == Ui::InputField::kTagPre) { // #TODO entities
-			push(EntityType::Pre);
-		} else /*if (ValidateUrl(tag.id)) */{ // We validate when we insert.
-			push(EntityType::CustomUrl, tag.id);
+bool IsSeparateTag(QStringView tag) {
+	return (tag == Ui::InputField::kTagCode)
+		|| (tag == Ui::InputField::kTagPre);
+}
+
+QString JoinTag(const QList<QStringView> &list) {
+	if (list.isEmpty()) {
+		return QString();
+	}
+	auto length = (list.size() - 1);
+	for (const auto &entry : list) {
+		length += entry.size();
+	}
+	auto result = QString();
+	result.reserve(length);
+	result.append(list.front());
+	for (auto i = 1, count = int(list.size()); i != count; ++i) {
+		if (!IsSeparateTag(list[i])) {
+			result.append(kTagSeparator).append(list[i]);
 		}
 	}
 	return result;
 }
 
-TextWithTags::Tags ConvertEntitiesToTextTags(const EntitiesInText &entities) {
-	TextWithTags::Tags result;
+QList<QStringView> SplitTags(QStringView tag) {
+	return tag.split(kTagSeparator);
+}
+
+QString TagWithRemoved(const QString &tag, const QString &removed) {
+	if (tag == removed) {
+		return QString();
+	}
+	auto list = SplitTags(tag);
+	list.erase(ranges::remove(list, QStringView(removed)), list.end());
+	return JoinTag(list);
+}
+
+QString TagWithAdded(const QString &tag, const QString &added) {
+	if (tag.isEmpty() || tag == added) {
+		return added;
+	}
+	auto list = SplitTags(tag);
+	const auto ref = QStringView(added);
+	if (list.contains(ref)) {
+		return tag;
+	}
+	list.push_back(ref);
+	std::sort(list.begin(), list.end());
+	return JoinTag(list);
+}
+
+EntitiesInText ConvertTextTagsToEntities(const TextWithTags::Tags &tags) {
+	auto result = EntitiesInText();
+	if (tags.isEmpty()) {
+		return result;
+	}
+
+	constexpr auto kInMaskTypes = std::array{
+		EntityType::Bold,
+		EntityType::Italic,
+		EntityType::Underline,
+		EntityType::StrikeOut,
+		EntityType::Spoiler,
+		EntityType::Code,
+		EntityType::Pre,
+	};
+	struct State {
+		QString link;
+		uint32 mask = 0;
+
+		void set(EntityType type) {
+			mask |= (1 << int(type));
+		}
+		void remove(EntityType type) {
+			mask &= ~(1 << int(type));
+		}
+		[[nodiscard]] bool has(EntityType type) const {
+			return (mask & (1 << int(type)));
+		}
+	};
+
+	auto offset = 0;
+	auto state = State();
+	auto notClosedEntities = QVector<int>(); // Stack of indices.
+	const auto closeOne = [&] {
+		Expects(!notClosedEntities.isEmpty());
+
+		auto &entity = result[notClosedEntities.back()];
+		entity = {
+			entity.type(),
+			entity.offset(),
+			offset - entity.offset(),
+			entity.data(),
+		};
+		if (ranges::contains(kInMaskTypes, entity.type())) {
+			state.remove(entity.type());
+		} else {
+			state.link = QString();
+		}
+		notClosedEntities.pop_back();
+	};
+	const auto closeType = [&](EntityType type) {
+		auto closeCount = 0;
+		const auto notClosedCount = notClosedEntities.size();
+		while (closeCount < notClosedCount) {
+			const auto index = notClosedCount - closeCount - 1;
+			if (result[notClosedEntities[index]].type() == type) {
+				for (auto i = 0; i != closeCount + 1; ++i) {
+					closeOne();
+				}
+				break;
+			}
+			++closeCount;
+		}
+	};
+	const auto openType = [&](EntityType type, const QString &data = {}) {
+		notClosedEntities.push_back(result.size());
+		result.push_back({ type, offset, -1, data });
+	};
+
+	const auto processState = [&](State nextState) {
+		const auto linkChanged = (nextState.link != state.link);
+		if (linkChanged) {
+			if (Ui::InputField::IsCustomEmojiLink(state.link)) {
+				closeType(EntityType::CustomEmoji);
+			} else if (IsMentionLink(state.link)) {
+				closeType(EntityType::MentionName);
+			} else {
+				closeType(EntityType::CustomUrl);
+			}
+		}
+		for (const auto type : kInMaskTypes) {
+			if (state.has(type) && !nextState.has(type)) {
+				closeType(type);
+			}
+		}
+		if (linkChanged && !nextState.link.isEmpty()) {
+			if (Ui::InputField::IsCustomEmojiLink(nextState.link)) {
+				const auto data = Ui::InputField::CustomEmojiEntityData(
+					nextState.link);
+				if (!data.isEmpty()) {
+					openType(EntityType::CustomEmoji, data);
+				}
+			} else if (IsMentionLink(nextState.link)) {
+				const auto data = MentionEntityData(nextState.link);
+				if (!data.isEmpty()) {
+					openType(EntityType::MentionName, data);
+				}
+			} else {
+				openType(EntityType::CustomUrl, nextState.link);
+			}
+		}
+		for (const auto type : kInMaskTypes) {
+			if (nextState.has(type) && !state.has(type)) {
+				openType(type);
+			}
+		}
+		state = nextState;
+	};
+	const auto stateForTag = [&](const QString &tag) {
+		auto result = State();
+		const auto list = SplitTags(tag);
+		for (const auto &single : list) {
+			if (single == Ui::InputField::kTagBold) {
+				result.set(EntityType::Bold);
+			} else if (single == Ui::InputField::kTagItalic) {
+				result.set(EntityType::Italic);
+			} else if (single == Ui::InputField::kTagUnderline) {
+				result.set(EntityType::Underline);
+			} else if (single == Ui::InputField::kTagStrikeOut) {
+				result.set(EntityType::StrikeOut);
+			} else if (single == Ui::InputField::kTagCode) {
+				result.set(EntityType::Code);
+			} else if (single == Ui::InputField::kTagPre) {
+				result.set(EntityType::Pre);
+			} else if (single == Ui::InputField::kTagSpoiler) {
+				result.set(EntityType::Spoiler);
+			} else {
+				result.link = single.toString();
+			}
+		}
+		return result;
+	};
+	for (const auto &tag : tags) {
+		if (tag.offset > offset) {
+			processState(State());
+		}
+		offset = tag.offset;
+		processState(stateForTag(tag.id));
+		offset += tag.length;
+	}
+	processState(State());
+
+	result.erase(ranges::remove_if(result, [](const EntityInText &entity) {
+		return (entity.length() <= 0);
+	}), result.end());
+
+	return result;
+}
+
+TextWithTags::Tags ConvertEntitiesToTextTags(
+		const EntitiesInText &entities) {
+	auto result = TextWithTags::Tags();
 	if (entities.isEmpty()) {
 		return result;
 	}
 
-	result.reserve(entities.size());
+	auto offset = 0;
+	auto current = QString();
+	const auto updateCurrent = [&](int nextOffset, const QString &next) {
+		if (next == current) {
+			return;
+		} else if (nextOffset > offset) {
+			if (!current.isEmpty()) {
+				result.push_back({ offset, nextOffset - offset, current });
+			}
+			offset = nextOffset;
+		}
+		current = next;
+	};
+	auto toRemove = std::vector<std::pair<int, QString>>();
+	const auto removeTill = [&](int nextOffset) {
+		while (!toRemove.empty() && toRemove.front().first <= nextOffset) {
+			updateCurrent(
+				toRemove.front().first,
+				TagWithRemoved(current, toRemove.front().second));
+			toRemove.erase(toRemove.begin());
+		}
+	};
 	for (const auto &entity : entities) {
 		const auto push = [&](const QString &tag) {
-			result.push_back({ entity.offset(), entity.length(), tag });
+			removeTill(entity.offset());
+			updateCurrent(entity.offset(), TagWithAdded(current, tag));
+			toRemove.push_back({ offset + entity.length(), tag });
+			ranges::sort(toRemove);
 		};
 		switch (entity.type()) {
 		case EntityType::MentionName: {
-			auto match = QRegularExpression(R"(^(\d+\.\d+)$)").match(entity.data());
+			const auto match = QRegularExpression(
+				"^(\\d+\\.\\d+:\\d+)$"
+			).match(entity.data());
 			if (match.hasMatch()) {
 				push(kMentionTagStart + entity.data());
 			}
@@ -2090,6 +2216,14 @@ TextWithTags::Tags ConvertEntitiesToTextTags(const EntitiesInText &entities) {
 				push(url);
 			}
 		} break;
+		case EntityType::CustomEmoji: {
+			const auto match = QRegularExpression(
+				"^(\\d+)$"
+			).match(entity.data());
+			if (match.hasMatch()) {
+				push(Ui::InputField::CustomEmojiLink(entity.data()));
+			}
+		} break;
 		case EntityType::Bold: push(Ui::InputField::kTagBold); break;
 		//case EntityType::Semibold: // Semibold is for UI parts only.
 		//	push(Ui::InputField::kTagSemibold);
@@ -2103,8 +2237,12 @@ TextWithTags::Tags ConvertEntitiesToTextTags(const EntitiesInText &entities) {
 			break;
 		case EntityType::Code: push(Ui::InputField::kTagCode); break; // #TODO entities
 		case EntityType::Pre: push(Ui::InputField::kTagPre); break;
+		case EntityType::Spoiler: push(Ui::InputField::kTagSpoiler); break;
 		}
 	}
+	if (!toRemove.empty()) {
+		removeTill(toRemove.back().first);
+	}
 	return result;
 }
 
@@ -2129,6 +2267,42 @@ void SetClipboardText(
 
 } // namespace TextUtilities
 
+TextForMimeData TextForMimeData::WithExpandedLinks(
+		const TextWithEntities &text) {
+	auto result = TextForMimeData{ .rich = text };
+	if (!ranges::contains(
+			text.entities,
+			EntityType::CustomUrl,
+			&EntityInText::type)) {
+		result.expanded = text.text;
+	} else {
+		auto from = 0;
+		for (const auto &entity : text.entities) {
+			if (entity.type() != EntityType::CustomUrl) {
+				continue;
+			}
+			// This logic is duplicated in Ui::Text::String::toText.
+			const auto &data = entity.data();
+			if (!data.startsWith(qstr("internal:"))
+				&& (data != UrlClickHandler::EncodeForOpening(
+					text.text.mid(entity.offset(), entity.length())))) {
+				const auto till = entity.offset() + entity.length();
+				if (const auto add = till - from; add > 0) {
+					result.expanded.append(text.text.data() + from, add);
+					from = till;
+				}
+				result.expanded.append(qstr(" (")).append(data).append(')');
+			}
+		}
+		const auto till = text.text.size();
+		if (const auto add = till - from; add > 0) {
+			result.expanded.append(text.text.data() + from, add);
+			from = till;
+		}
+	}
+	return result;
+}
+
 EntityInText::EntityInText(
 	EntityType type,
 	int offset,
@@ -2146,7 +2320,7 @@ int EntityInText::FirstMonospaceOffset(
 	auto &&monospace = ranges::make_subrange(
 		entities.begin(),
 		entities.end()
-	) | ranges::view::filter([](const EntityInText & entity) {
+	) | ranges::views::filter([](const EntityInText & entity) {
 		return (entity.type() == EntityType::Pre)
 			|| (entity.type() == EntityType::Code);
 	});
