@@ -318,10 +318,14 @@ int FlatLabel::resizeGetHeight(int newWidth) {
 	return countTextHeight(_textWidth);
 }
 
-int FlatLabel::naturalWidth() const {
+int FlatLabel::textMaxWidth() const {
 	return _text.maxWidth();
 }
 
+int FlatLabel::naturalWidth() const {
+	return (_st.align == style::al_top) ? -1 : textMaxWidth();
+}
+
 QMargins FlatLabel::getMargins() const {
 	return _st.margin;
 }
@@ -962,6 +966,9 @@ void FlatLabel::paintEvent(QPaintEvent *e) {
 		: _st.maxHeight
 		? qMax(_st.maxHeight / lineHeight, 1)
 		: ((height() / lineHeight) + 2);
+	const auto paused = _animationsPausedCallback
+		? _animationsPausedCallback()
+		: WhichAnimationsPaused::None;
 	_text.draw(p, {
 		.position = { textLeft, _st.margin.top() },
 		.availableWidth = textWidth,
@@ -970,6 +977,10 @@ void FlatLabel::paintEvent(QPaintEvent *e) {
 		.palette = &_st.palette,
 		.spoiler = Text::DefaultSpoilerCache(),
 		.now = crl::now(),
+		.pausedEmoji = (paused == WhichAnimationsPaused::CustomEmoji
+			|| paused == WhichAnimationsPaused::All),
+		.pausedSpoiler = (paused == WhichAnimationsPaused::Spoiler
+			|| paused == WhichAnimationsPaused::All),
 		.selection = selection,
 		.elisionLines = lines,
 		.elisionBreakEverywhere = renderElided && _breakEverywhere,
@@ -978,7 +989,7 @@ void FlatLabel::paintEvent(QPaintEvent *e) {
 
 DividerLabel::DividerLabel(
 	QWidget *parent,
-	object_ptr<FlatLabel> &&child,
+	object_ptr<RpWidget> &&child,
 	const style::margins &padding,
 	RectParts parts)
 : PaddingWrap(parent, std::move(child), padding)
