@@ -109,7 +109,7 @@
 	[openPanel setResolvesAliases:YES];
 
 	[openPanel beginWithCompletionHandler:^(NSInteger result){
-		if (result == NSFileHandlingPanelOKButton) {
+		if (result == NSModalResponseOK) {
 			completionHandler([openPanel URLs]);
 		}
 	}];
@@ -184,6 +184,8 @@ public:
 
 	void *winId() override;
 
+	void setOpaqueBg(QColor opaqueBg) override;
+
 private:
 	WKUserContentController *_manager = nullptr;
 	WKWebView *_webview = nullptr;
@@ -201,6 +203,7 @@ Instance::Instance(Config config) {
 	[_webview setUIDelegate:_handler];
 	[configuration release];
 
+	setOpaqueBg(config.opaqueBg);
 	init(R"(
 window.external = {
 	invoke: function(s) {
@@ -245,6 +248,13 @@ void *Instance::winId() {
 	return _webview;
 }
 
+void Instance::setOpaqueBg(QColor opaqueBg) {
+	if (@available(macOS 12.0, *)) {
+		[_webview setValue: @NO forKey: @"drawsBackground"];
+		[_webview setUnderPageBackgroundColor:[NSColor clearColor]];
+	}
+}
+
 void Instance::resizeToWindow() {
 }
 
